;**************************************************************************
; CLOCK.ASM					(c) Ing. Miroslav Nmeek
;
; Digitln hodiny s budkem, stopkami, asovaem a datem. Program vyuv
; jednotku P84-LCD s 3.5 mstnm LCD displejem, 2 tlatky a piezomniem.
;--------------------------------------------------------------------------
; Pi programovn je nutno modul LCD odpojit, aby nedolo ke znien displeje.
; Po naprogramovn se doporuuje odpojit jednotku od potae, jinak by hodiny
; byly resetovny pi vypnut potae.
;--------------------------------------------------------------------------
; Nvod na obsluhu hodin:
;
;	Po zapnut hodiny signalizuj bliknm displeje a ppnutm vpadek napjen.
;
;	indiktory:
;		ipka (vlevo nahoe) ... zapnuta signalizace cel hodiny
;		vlnovka (vlevo dole) ... zapnut budk
;		pomlka (vlevo) ........ je odpoledne nebo hodina 20 a vce
;
;	lev tlatko (nastaven pravm tlatkem):
;		1x stisk ............... pepnn budku a signalizace hodin
;		2x stisk ............... nastaven hodin budku
;		3x stisk ............... nastaven minut budku
;		dlouze ................. nastaven msce
;		dlouze + 1x stisk ...... nastaven dne
;		dlouze + 2x stisk ...... nastaven dne v tdnu (0=nedle, 1=pondl, ...)
;		dlouze + 3x stisk ...... nastaven roku (2000 a 2099)
;		dlouze + 4x stisk ...... nastaven hodiny
;					 (cyklickou zmnou pes 24 hodin
;					  lze pepnat md AM/PM nebo 24 hodin)
;		dlouze + 5x stisk ...... nastaven minuty
;		   Po zmn hodiny nebo minuty asu se as zastav,
;		   po stisku nkterho tlatka se rozebhne (sekundy = 0)
;
;	prav tlatko (nastaven levm tlatkem):
;		1x stisk ............... zobrazen msce a dne, po chvli
;					 se zobraz den v tdnu (0=nedle) a rok (20xx),
;					 po dal chvli zmiz samo;
;					 levm tlatkem lze pepnat zobrazen
;					 datum/den v tdnu, zobrazen nezmiz samo
;		2x stisk ............... zobrazen hodiny a minuty budku
;					 po chvli zmiz samo nebo lze
;					 levm tlatkem pepnat melodie budku
;		3x stisk ............... zobrazen sekund, levm tlatkem lze
;					 sekundy vynulovat a dostavit as na celou minutu
;		dlouze ................. asova, levm tlatkem lze aktivovat
;					 asova 1 a 19 minut
;		dlouze + 1x stisk ...... stopky, levm tlatkem lze stopky
;					 spustit/zastavit/vynulovat
;**************************************************************************

	RADIX	DEC

; ----- bity portu A

DATBIT	EQU	0	; datov bit
CLKBIT	EQU	1	; hodinov bit
STRBIT	EQU	2	; strobovac bit
SPKBIT	EQU	3	; bit vstupu na reproduktor
KEYBIT	EQU	4	; bit vstupu z tlatek

; ----- indikan bity (slice Num4)

ISIGNAL	EQU	0	; bit indiktoru signalizace (ipka)
IVALID	EQU	1	; indiktor platnosti asu (dvojteka vlevo)
ITIME	EQU	3	; indiktor bcho asu (dvojteka uprosted)
IND1	EQU	5	; bit lev slice 1
IAMPM	EQU	6	; bit indiktoru AM/PM (znamnko mnus)
IALARM	EQU	7	; bit indiktoru budku (vlnovka)

; ----- konstanty

MELODS	EQU	4	; poet melodi
REPKEY1	EQU	100	; inicializan interval opakovn klves
REPKEY2	EQU	20	; opakovan interval opakovn klves
FREEKEY	EQU	5	; ta pro test uvolnn tlatka
DCFVAL	EQU	60	; ta platnosti daje asu DCF (minut)

;--------------------------------------------------------------------------
;	promnn
;--------------------------------------------------------------------------

; Pozor na peteen velikosti promnnch PIC16F84 - je to tm na doraz!

	CBLOCK

; ----- zobrazovan znaky
; bajty slic Num1 a Num4 mus v pamti za sebou nsledovat!
;
; Obsazen segment pro slice Num1 a Num3:
;	bit 0: segment B (vpravo nahoe)
;	bit 1: segment A (nahoe)
;	bit 2: segment F (vlevo nahoe)
;	bit 3: segment G (uprosted)
;	bit 4: segment C (vpravo dole)
;	bit 5: segment D (dole)
;	bit 6: segment E (vlevo dole)
;	bit 7: segment H (teka)
;
; Obsazen segment pro Num4:
;	bit 0: ipka
;	bit 1: dvojteka vlevo
;	bit 2: inverze displeje
;	bit 3: dvojteka uprosted
;	bit 4: ... nepouito
;	bit 5: slice 1 vlevo
;	bit 6: znamnko mnus
;	bit 7: vlnovka

Num1			; vstupn bajt slice 1 (vpravo)
Num2			; vstupn bajt slice 2 (uprosted)
Num3			; vstupn bajt slice 3 (vlevo)
Num4			; vstupn bajt slice 4 (indikace)

; ----- obsluha zobrazen

CBytes			; ta bajt k odesln
DByte			; vyslan bajt na displej
CBits			; ta bit vyslanho bajtu

TempB			; schova bajtu pi dekdovn
TempC			; pracovn ta pi dekdovn

; ----- ovldn

Flags			; pznaky
			;	bit 0: 1=inverze vstupu
			;	bit 1: 1=lev tlatko stisknuto
			;	bit 2: 1=prav tlatko stisknuto
			;	bit 3: 1=budk zapnut
			;	bit 4: 1=signalizace zapnuta
			;	bit 5: 1=as b
			;	bit 6: 1=asova b
			;	bit 7: 1=stopky b

Flags2			; pznaky 2
			;	bit 0: 1=hodina se zobraz 24 hodin
			;	bit 1: 1=lev tlatko dlouze stisknuto
			;	bit 2: 1=prav tlatko dlouze stisknuto
			;	bit 3: 1=probh pehrvn (vypnutm lze peruit pehrvn)
			;	bit 4: 1=pehrt signalizaci
			;	bit 5: 1=pehrt asova
			;	bit 6: 1=pehrt budk
			;	bit 7: 1=je pauza v melodii

RelKeyL			; ta uvolnn levho tlatka
RelKeyR			; ta uvolnn pravho tlatka
RepKeyL			; ta pro opakovn levho tlatka
RepKeyR			; ta pro opakovn pravho tlatka

State			; aktuln stav (0=vpadek napjen SRESET)
			; (viz tabulka stav ne)

; ----- aktuln as

Hundrth			; setiny sekundy
Second			; aktuln sekunda
Minute			; aktuln minuta
Hour			; aktuln hodina
Week			; aktuln den v tdnu (0=nedle, 1=pondl, ...)
Day			; aktuln den
Month			; aktuln msc
Year			; aktuln rok - ni 2 slice (offset od 2000)

; ----- budk

AMinute			; minuta budku
AHour			; hodina budku
ADisp			; ta doby zobrazen budku a data (pouv se 256, tj. 2.5 sekundy)
AMelody			; slo melodie budku

; ----- asova

THundr			; setiny sekundy asovae
TSecond			; sekunda asovae
TMinute			; minuta asovae

; ----- stopky

SHundr			; setiny sekundy stopek
SSecond			; sekunda stopek
SMinute			; minuta stopek
SHour			; hodina stopek

; ----- pehrvn melodie

Tone			; schova sla tnu
ToneL			; konstanta pro tn LOW (1 a 256)
ToneH			; konstanta pro tn HIGH (1 a 256)
Length			; ta dlky tnu
CountL			; ta pro tn LOW
CountH			; ta pro tn HIGH
PointL			; ukazatel melodie LOW
PointH			; ukazatel melodie HIGH

; ----- natn asu DCF

DCFVal			; ta platnosti daje asu (v minutch)

; ----- schova registr pi peruen

TempW			; schova registru W pi peruen
TempS			; schova registru STATUS pi peruen
	ENDC

; ----- symboly stav

	CBLOCK	0
SRESET			; vpadek napjen (mus mt hodnotu 0)
SCLOCK			; bn zobrazen asu

	;* volba mdu
		;* nastavovn levm tlatkem - krtk stisk
SALARM			; nastaven budku a signalizace
SAHOUR			; nastaven hodin budku
SAMINUT			; nastaven minut budku

		;* nastavovn levm tlatkem - dlouh stisk
SMONTH			; nastaven msce
SDAY			; nastaven dne
SWEEK			; nastaven dne v tdnu
SYEAR			; nastaven roku
SHOUR			; nastaven hodin
SMINUTE			; nastaven minut
SWAIT			; ekn na rozbh hodin (nepouito)

		;* nastavovn pravm tlatkem - krtk stisk
SDDATE			; zobrazen data
SDALARM			; zobrazen budku
SSECOND			; sekundy
SDYEAR			; zobrazen dne v tdnu a roku

		;* nastavovn pravm tlatkem - dlouh stisk
STIMER			; asova
SSTOP			; stopky

	;* volba mdu po zmn
		;* nastavovn levm tlatkem po zmn pravm tlatkem - krtk stisk
SALARM2			; nastaven budku a signalizace (po zmn)
SAHOUR2			; nastaven hodin budku (po zmn)
SAMINU2			; nastaven minut budku (po zmn)

		;* nastavovn levm tlatkem po zmn pravm tlatkem - dlouh stisk
SMONTH2			; nastaven msce (po zmn, nepouito)
SDAY2			; nastaven dne (po zmn, nepouito)
SWEEK2			; nastaven dne v tdnu (po zmn, nepouito)
SYEAR2			; nastaven roku (po zmn, nepouito)
SHOUR2			; nastaven hodin (po zmn)
SMINUT2			; nastaven minut (po zmn)
SWAIT2			; ekn na rozbh hodin (po zmn)

		;* nastavovn pravm tlatkem po zmn levm tlatkem - krtk stisk
SDDATE2			; zobrazen data (po zmn)
SDALRM2			; zobrazen budku (po zmn)
SSECND2			; sekundy (po zmn)
SDYEAR2			; zobrazen dne v tdnu a roku (po zmn)

		;* nastavovn pravm tlatkem po zmn levm tlatkem - dlouh stisk
STIMER2			; asova (po zmn)
SSTOP2			; stopky (po zmn)
	ENDC

;--------------------------------------------------------------------------
;	vektory
;--------------------------------------------------------------------------

	ORG	0
	goto	Start		; skok na start programu

	ORG	4
	goto	Inter		; skok na obsluhu peruen

;--------------------------------------------------------------------------
;	tabulky
;--------------------------------------------------------------------------

; ----- tabulka slic
; Bitov mapa segment slice:
;
;	 -- 1 --
;	|	|
;	2	0
;	|	|
;	 -- 3 --
;	|	|
;	6	4
;	|	|
;   7	 -- 5 --

ChrTab	addwf	PCL,F
	dt	01110111b	; 0
	dt	00010001b	; 1
	dt	01101011b	; 2
	dt	00111011b	; 3
	dt	00011101b	; 4
	dt	00111110b	; 5
	dt	01111110b	; 6
	dt	00010011b	; 7
	dt	01111111b	; 8
	dt	00111111b	; 9
;	dt	01011111b	; A
;	dt	01111100b	; B
;	dt	01100110b	; C
;	dt	01111001b	; D
;	dt	01101110b	; E
;	dt	01001110b	; F

; ----- tabulka zobrazen - lev slo

DspLTab	addwf	PCL,F
	goto	DHour0		; vpadek napjen
	goto	DHour		; bn zobrazen asu

	goto	DAHourB		; nastaven budku a signalizace
	goto	DAHourB		; nastaven hodin budku
	goto	ClrL		; nastaven minut budku

	goto	DMonth		; nastaven msce
	goto	ClrL		; nastaven dne
	goto	DWeek		; nastaven dne v tdnu
	goto	ClrL		; nastaven roku
	goto	DHour		; nastaven hodin
	goto	ClrL		; nastaven minut
	goto	DHour		; ekn na rozbh hodin (nepouito)

	goto	DMonth		; zobrazen data
	goto	DAHour		; zobrazen budku
	goto	ClrL		; sekundy
	goto	DWeek		; zobrazen dne v tdnu a roku

	goto	DTMinut		; asova
	goto	DStop		; stopky

	goto	DAHourB		; nastaven budku a signalizace (po zmn)
	goto	DAHourB		; nastaven hodin budku (po zmn)
	goto	ClrL		; nastaven minut budku (po zmn)

	goto	DMonth		; nastaven msce (po zmn)
	goto	ClrL		; nastaven dne (po zmn)
	goto	DWeek		; nastaven dne v tdnu (po zmn)
	goto	ClrL		; nastaven roku (po zmn)
	goto	DHour		; nastaven hodin (po zmn)
	goto	ClrL		; nastaven minut (po zmn)
	goto	DHour		; ekn na rozbh hodin (po zmn)

	goto	DMonth		; zobrazen data
	goto	ClrL		; zobrazen budku (po zmn)
	goto	ClrL		; sekundy (po zmn)
	goto	DWeek		; zobrazen dne v tdnu a roku

	goto	DTMinut		; asova (po zmn)
	goto	DStop		; stopky (po zmn)

; ----- tabulka zobrazen - prav slo

DspRTab	addwf	PCL,F
	goto	DMinutB		; vpadek napjen
	goto	DMinute		; bn zobrazen asu

	goto	DAMinB		; nastaven budku a signalizace
	goto	DAMPMB		; nastaven hodin budku
	goto	DAMinB		; nastaven minut budku

	goto	ClrR		; nastaven msce
	goto	DDay		; nastaven dne
	goto	ClrR		; nastaven dne v tdnu
	goto	DYear		; nastaven roku
	goto	DAMPM		; nastaven hodin
	goto	DMinute		; nastaven minut
	goto	DMinute		; ekn na rozbh hodin (nepouito)

	goto	DDay		; zobrazen data
	goto	DAMinut		; zobrazen budku
	goto	DSecond		; sekundy
	goto	DYear		; zobrazen dne v tdnu a roku

	goto	DTSecnd		; asova
	return			; stopky

	goto	DAMinB		; nastaven budku a signalizace (po zmn)
	goto	DAMPMB		; nastaven hodin budku (po zmn)
	goto	DAMinB		; nastaven minut budku (po zmn)

	goto	ClrR		; nastaven msce (po zmn)
	goto	DDay		; nastaven dne (po zmn)
	goto	ClrR		; nastaven dne v tdnu (po zmn)
	goto	DYear		; nastaven roku (po zmn)
	goto	DAMPM		; nastaven hodin (po zmn)
	goto	DMinute		; nastaven minut (po zmn)
	goto	DMinute		; ekn na rozbh hodin (po zmn)

	goto	DDay		; zobrazen data
	goto	DAMel		; zobrazen budku (po zmn)
	goto	DSecond		; sekundy (po zmn)
	goto	DYear		; zobrazen dne v tdnu a roku

	goto	DTSecnd		; asova (po zmn)
	return			; stopky (po zmn)

; ----- tabulka zobrazen - dvojteka (a teka) uprosted

DspDTab	addwf	PCL,F
	goto	DekBB		; vpadek napjen
	goto	DekB		; bn zobrazen asu

	goto	DekBOn		; nastaven budku a signalizace
	goto	DekBOn		; nastaven hodin budku
	goto	DekBOn		; nastaven minut budku

	goto	DekTec		; nastaven msce
	goto	DekTec		; nastaven dne
	goto	DekTec		; nastaven dne v tdnu
	goto	DekTec		; nastaven roku
	goto	DekB		; nastaven hodin
	goto	DekB		; nastaven minut
	goto	DekB		; ekn na rozbh hodin (nepouito)

	goto	DekTec		; zobrazen data
	goto	DekBOn		; zobrazen budku
	goto	DekBOn		; sekundy
	goto	DekTec		; zobrazen dne v tdnu a roku

	goto	DekBT		; asova
	goto	DekBS		; stopky

	goto	DekBOn		; nastaven budku a signalizace (po zmn)
	goto	DekBOn		; nastaven hodin budku (po zmn)
	goto	DekBOn		; nastaven minut budku (po zmn)

	goto	DekTec		; nastaven msce (po zmn)
	goto	DekTec		; nastaven dne (po zmn)
	goto	DekTec		; nastaven dne v tdnu (po zmn)
	goto	DekTec		; nastaven roku (po zmn)
	goto	DekBOn		; nastaven hodin (po zmn)
	goto	DekBOn		; nastaven minut (po zmn)
	goto	DekBOn		; ekn na rozbh hodin (po zmn)

	goto	DekTec		; zobrazen data
	goto	DekBOff		; zobrazen budku (po zmn)
	goto	DekBOn		; sekundy (po zmn)
	goto	DekTec		; zobrazen dne v tdnu a roku

	goto	DekBT		; asova (po zmn)
	goto	DekBS		; stopky (po zmn)

; ----- ovldn levm tlatkem

KeyLTab	addwf	PCL,F
	goto	KeyNorm		; vpadek napjen
	goto	KeyLMod		; bn zobrazen asu

	goto	KeyIncL		; nastaven budku a signalizace
	goto	KeyIncL		; nastaven hodin budku
	goto	KeyNorm		; nastaven minut budku

	goto	KeyInc		; nastaven msce
	goto	KeyInc		; nastaven dne
	goto	KeyInc		; nastaven dne v tdnu
	goto	KeyInc		; nastaven roku
	goto	KeyInc		; nastaven hodin
	goto	KeyNorm		; nastaven minut
	goto	KeyNrm0		; ekn na rozbh hodin (nepouito)

	goto	KeySetW		; zobrazen data
	goto	KeyDAl		; zobrazen budku
	goto	KeySec		; sekundy
	goto	KeySetD		; zobrazen dne v tdnu a roku

	goto	KeyTim		; asova
	goto	KeyStop		; stopky

	goto	KeyNorm		; nastaven budku a signalizace (po zmn)
	goto	KeyIncL		; nastaven hodin budku (po zmn)
	goto	KeyNorm		; nastaven minut budku (po zmn)

	goto	KeyInc		; nastaven msce (po zmn)
	goto	KeyInc		; nastaven dne (po zmn)
	goto	KeyInc		; nastaven dne v tdnu (po zmn)
	goto	KeyInc		; nastaven roku (po zmn)
	goto	KeyInc		; nastaven hodin (po zmn)
	goto	KeyInc		; nastaven minut (po zmn)
	goto	KeyNrm0		; ekn na rozbh hodin (po zmn)

	goto	KeyStW2		; zobrazen data (po zmn)
	goto	KeyDAl2		; zobrazen budku (po zmn)
	goto	KeySec2		; sekundy (po zmn)
	goto	KeyStD2		; zobrazen dne v tdnu a roku

	goto	KeyTim2		; asova (po zmn)
	goto	KeySto2		; stopky (po zmn)

; ----- ovldn pravm tlatkem

KeyRTab	addwf	PCL,F
	goto	KeyNorm		; vpadek napjen
	goto	KeyRMod		; bn zobrazen asu

	goto	KeyAlrm		; nastaven budku a signalizace
	goto	KeyAHou		; nastaven hodin budku
	goto	KeyAMin		; nastaven minut budku

	goto	KeyMon		; nastaven msce
	goto	KeyDay		; nastaven dne
	goto	KeyWeek		; nastaven dne v tdnu
	goto	KeyYear		; nastaven roku
	goto	KeyHour		; nastaven hodin
	goto	KeyMin		; nastaven minut
	goto	KeyNrm0		; ekn na rozbh hodin (nepouito)

	goto	KeyIncR		; zobrazen data
	goto	KeyIncR		; zobrazen budku
	goto	KeyNorm		; sekundy
	goto	KeySetA		; zobrazen dne v tdnu a roku

	goto	KeyInc		; asova
	goto	KeyNorm		; stopky

	goto	KeyAlr2		; nastaven budku a signalizace (po zmn)
	goto	KeyAHo2		; nastaven hodin budku (po zmn)
	goto	KeyAMi2		; nastaven minut budku (po zmn)

	goto	KeyMon		; nastaven msce (po zmn)
	goto	KeyDay		; nastaven dne (po zmn)
	goto	KeyWeek		; nastaven dne v tdnu (po zmn)
	goto	KeyYear		; nastaven roku (po zmn)
	goto	KeyHou2		; nastaven hodin (po zmn)
	goto	KeyMin2		; nastaven minut (po zmn)
	goto	KeyNrm0		; ekn na rozbh hodin (po zmn)

	goto	KeyNorm		; zobrazen data (po zmn)
	goto	KeyNorm		; zobrazen budku (po zmn)
	goto	KeyNorm		; sekundy (po zmn)
	goto	KeyNorm		; zobrazen dne v tdnu a roku

	goto	KeyNorm		; asova (po zmn)
	goto	KeyNorm		; stopky (po zmn)

; ----- tabulka tn

TbToneL	addwf	PCL,F
	dt	0
	dt	58		; 1: C
	dt	23		; 2: D
	dt	249		; 3: E
	dt	235		; 4: F
	dt	209		; 5: G
	dt	185		; 6: A
	dt	165		; 7: H
;	dt	155		; 8: C+

TbToneH	addwf	PCL,F
	dt	0
	dt	2		; 1: C
	dt	2		; 2: D
	dt	1		; 3: E
	dt	1		; 4: F
	dt	1		; 5: G
	dt	1		; 6: A
	dt	1		; 7: H
;	dt	1		; 8: C+

; ----- tabulka melodi budku

MelTabL	addwf	PCL,F
	dt	LOW Mel1
	dt	LOW Mel2
	dt	LOW Mel3
	dt	LOW Mel4

MelTabH	addwf	PCL,F
	dt	HIGH Mel1
	dt	HIGH Mel2
	dt	HIGH Mel3
	dt	HIGH Mel4

; ----- definice not a dlek not

C3	EQU	1
D3	EQU	2
E3	EQU	3
F3	EQU	4
G3	EQU	5
A3	EQU	6
H3	EQU	7
;C4	EQU	8

R	EQU	255

L8	EQU	6		; 1/8
L8P	EQU	9		; 1/8.
L4	EQU	12		; 1/4
L4P	EQU	18		; 1/4.
L2	EQU	24		; 1/2
L2P	EQU	36		; 1/2.
L1	EQU	48		; 1
L1P	EQU	72		; 1.
L0	EQU	96		; 0

; ----- melodie signalizace

MelSig	dt	G3,L4
	dt	R,L4
	dt	G3,L4
	dt	0

; ----- melodie asovae

MelTim	dt	C3,L4
	dt	E3,L4
	dt	G3,L4
	dt	C3,L4
	dt	E3,L4
	dt	G3,L4
	dt	C3,L4
	dt	E3,L4
	dt	G3,L2
;	dt	C4,L2
	dt	0

; ----- melodie budku

Mel1	dt	C3,L4		; Oh, Susanna
	dt	D3,L4

	dt	E3,L2
	dt	G3,L2
	dt	G3,L2P
	dt	A3,L4

	dt	G3,L2
	dt	E3,L2
	dt	C3,L2P
	dt	D3,L4

	dt	E3,L2
	dt	E3,L2
	dt	D3,L2
	dt	C3,L2

	dt	D3,L1

	dt	0

Mel2	dt	C3,L2		; Koka leze drou
	dt	D3,L2
	dt	E3,L2
	dt	F3,L2

	dt	G3,L1
	dt	G3,L1

	dt	A3,L1
	dt	A3,L1

	dt	G3,L0
	dt	0

Mel3	dt	C3,L2P		; London's Burning
	dt	C3,L4
	dt	F3,L1
	dt	F3,L1

	dt	C3,L2P
	dt	C3,L4
	dt	F3,L1
	dt	F3,L1

	dt	G3,L2P
	dt	G3,L4
	dt	A3,L1
	dt	A3,L1

	dt	G3,L2P
	dt	G3,L4
	dt	A3,L1
	dt	A3,L1
	dt	0

Mel4	dt	G3,L1		; Blowing in the Wind
	dt	G3,L2
	dt	G3,L2

	dt	A3,L1
	dt	A3,L2
	dt	A3,L2

	dt	G3,L1
	dt	E3,L2
	dt	D3,L2

	dt	C3,L1
	dt	0

; ----- naten dalho bajtu melodie (po nvratu nulovat PCLATH!)
; Bhem provdn funkce nesm bt vyvolno peruen (viz poznmka dle)

GetMel	movf	PointH,W	; naten ukazatele HIGH
	movwf	PCLATH		; nastaven adresy skoku HIGH
	movf	PointL,W	; naten ukazatele LOW
	incf	PointL,F	; zven ukazatele LOW
	btfsc	STATUS,Z	; je peteen pes 0?
	incf	PointH,F	; zven ukazatele HIGH
	movwf	PCL		; skok na naten bajtu

;--------------------------------------------------------------------------
;	start programu
;--------------------------------------------------------------------------

; ----- nastaven smru bit portu A jako vstup

Start	bsf	STATUS,RP0	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
	movlw	10000b		; volba smru bit: 0=vstup, 1=vstup
	movwf	TRISA		; nastaven bit portu A (registr TRISA)

; ----- nastaven asovae Timer0
; Obsluha peruen se vyvolv po 10 ms, tedy 100x za sekundu,
; to je kmitoet pro LCD 50 Hz. Pro krystal 4 MHz je taktovac
; frekvence 1 MHz, poadovan dlic pomr je 10000. Nastavenm
; dlicho pomru peddliky na 64 a nastavovnm tae na
; hodnotu 100 (tj. 156 dopluje do 256) bude perioda vyvoln
; obsluhy peruen 9984 us, musme proto doplnit 16 us instrukcemi.

	movlw	5 + bit7	; dlika TMR0 1/64, pipojena k TMR0, bez pull-up
	movwf	OPTION_REG	; nastaven OPTION registru asova
	bcf	STATUS,RP0	; zapnut banky 0

; ----- nastaven portu A (vypnut dicch signl)

	clrf	PORTA		; nulovn dicch signl

; ----- inicializace pznak a promnnch

	clrf	Num4		; nulovn indiktor

	clrf	Flags		; nulovn pznak
	clrf	Flags2		; nulovn pznak 2
	clrf	RelKeyL		; nulovn tae uvolnn levho tlatka
	clrf	RelKeyR		; nulovn tae uvolnn pravho tlatka
	clrf	RepKeyL		; nulovn tae opakovn levho tlatka
	clrf	RepKeyR		; nulovn tae opakovn pravho tlatka
	clrf	State		; nastaven stavu na RESET

	clrf	Hundrth		; nulovn setin sekundy
	clrf	Second		; nulovn sekund
	clrf	Minute		; nulovn minut
	clrf	Hour		; nulovn hodin 24 hodin
	movlw	6		; 1.1.2000 bylo v sobotu
	movwf	Week		; nulovn dne v tdnu
	movlw	1		; nulovac bajt
	movwf	Day		; nulovn dne
	movwf	Month		; nulovn msce
	clrf	Year		; nulovn roku
	movwf	ToneH		; asovac konstanta pro prvn pauzu

	clrf	AMinute		; nulovn minut budku
	clrf	AHour		; nulovn hodin budku (24 hodin)
	clrf	AMelody		; nulovn melodie budku

	movlw	100		; nulov hodnota setin sekundy asovae
	movwf	THundr		; nulovn setin sekundy asovae
	clrf	TSecond		; nulovn sekund asovae
	clrf	TMinute		; nulovn minut asovae

	clrf	SHundr		; nulovn setin sekundy stopek
	clrf	SSecond		; nulovn sekund stopek
	clrf	SMinute		; nulovn minut stopek
	clrf	SHour		; nulovn hodin stopek

; ----- inicializace obsluhy peruen

	movlw	bit5 + bit7	; povoleno peruen od TMR0 a globln peruen
	movwf	INTCON		; nastaven registru peruen INTCON

; ----- zvukov signalizace vpadku napjen

	bsf	Flags2,4	; poadavek k pehrt signalizace

; ----- obsluha signalizace

Loop	bcf	Flags2,3	; vypnut pznaku pehrvn melodie

	btfss	Flags2,4	; je poadovn signalizace?
	goto	Loop2		; nen signalizace

	bcf	Flags2,4	; vypnut poadavku signalizace
	movlw	LOW MelSig
	movwf	PointL
	movlw	HIGH MelSig
	goto	Loop6

; ----- obsluha asovae

Loop2	btfss	Flags2,5	; je poadovn signalizace asovae?
	goto	Loop4		; nen signalizace asovae

	bcf	Flags2,5	; vypnut poadavku signalizace asovae
	movlw	LOW MelTim
	movwf	PointL
	movlw	HIGH MelTim
	goto	Loop6

; ----- obsluha budku

Loop4	btfss	Flags2,6	; je poadovn signalizace budku?
	goto	Loop		; nen signalizace budku

	bcf	Flags2,6	; vypnut poadavku signalizace budku

	movf	AMelody,W	; naten sla aktivn melodie
	call	MelTabL		; naten offsetu melodie LOW
	movwf	PointL

	movf	AMelody,W	; naten sla aktivn melodie
	call	MelTabH		; naten offsetu melodie HIGH
Loop6	movwf	PointH
	bsf	Flags2,3	; pznak probhajcho pehrvn

; ----- vodn pauza (pro oddlen pi pepnn a pro synchronizaci asu)

	bsf	Flags2,7	; zapnut pauzy
	movlw	10
	goto	PlayLn2

; ----- naten dalho znaku melodie
; V obsluze peruen se pouv pstup k tabulkm pedpokldajc
; PCLATH=0, proto nesm dojt k vyvoln peruen ve chvli voln
; funkce GetMel. To je zajitno synchronizac peruen a pehrvn
; melodie, peruen je vyvolno bhem pehrvn jednoho tnu,
; dal tn je pipravovn ihned po nvratu z peruen.

PlayNxt	call	GetMel		; naten tnu
	clrf	PCLATH		; nulovn adresy HIGH

; ----- konec melodie

	addlw	0		; je konec melodie?
	btfsc	Flags2,3	; pi peruen pehrvn pedasn konec
	btfsc	STATUS,Z	; test, zda je konec melodie
	goto	Loop		; je konec melodie

; ----- nastaven pauzy

	bsf	Flags2,7	; zapnut pauzy
	xorlw	R		; je to pauza?
	btfsc	STATUS,Z	; test, zda je pauza
	goto	PlayLen		; je pauza
	xorlw	R		; nvrat hodnoty W
	bcf	Flags2,7	; nen pauza

; ----- naten konstanty tnu

	movwf	Tone		; schova tnu pro pt
	call	TbToneL		; naten konstanty LOW
	movwf	ToneL		; nastaven konstanty LOW

	movf	Tone,W		; poadovan tn
	call	TbToneH		; naten konstanty HIGH
	movwf	ToneH		; nastaven konstanty HIGH

; ----- nov ta jednoho tnu

PlayLen	call	GetMel		; naten dlky tnu
	clrf	PCLATH		; nulovn adresy HIGH
PlayLn2	movwf	Length		; nastaven dlky tnu

; ----- pprava tae pro jeden impuls

PlayImp	movf	ToneL,W		; konstanta pro tn LOW
	movwf	CountL		; uloen do tae LOW
	movf	ToneH,W		; konstanta pro tn HIGH
	movwf	CountH		; uloen do tae HIGH

; ----- tn po dobu jednoho impulsu

PlayW	decfsz	CountL,F	; tn tae LOW
	goto	PlayW
	decfsz	CountH,F	; tn tae HIGH
	goto	PlayW

; ----- zmna signlu (jen nen-li pauza)

	btfsc	Flags2,7	; test, zda je pauza
	goto	PlayTon		; je pauza - bude dal impuls
	movlw	1 << SPKBIT	; bit inverze reproduktoru
	xorwf	PORTA,F		; inverze signlu pro reproduktor

; ----- test, zda je konec tnu

PlayTon	movf	Length,F	; test, zda uplynula doba tnu
	btfss	STATUS,Z	; uplynula doba tnu?
	goto	PlayImp		; jet neuplynula - dal impuls
	bcf	PORTA,SPKBIT	; vypnut bitu pro reproduktor
	goto	PlayNxt		; dal tn melodie

;--------------------------------------------------------------------------
;	ovldn asovae
;--------------------------------------------------------------------------

KeyTim	call	KeyCh		; zmna na modifikovan stav

KeyTim2	movlw	100		; inicializan hodnota setin sekundy
	movwf	THundr		; nastaven setin sekundy asovae
	clrf	TSecond		; nulovn sekund asovae
	incf	TMinute,F	; inkrementace minut asovae
	bsf	Flags,6		; zapnut asovae
	movlw	20		; maximln hodnota asovae
	subwf	TMinute,W	; test maximln hodnoty asovae
	btfss	STATUS,Z	; pekroena maximln hodnota?
	return			; nen pekroena maximln hodnota
	clrf	TMinute		; nulovn minut
	bcf	Flags,6		; vypnut asovae
	return

;--------------------------------------------------------------------------
;	ovldn stopek
;--------------------------------------------------------------------------

KeyStop	call	KeyCh		; zmna na modifikovan stav

KeySto2	clrf	SHundr		; nulovn setin sekundy stopek
	btfss	Flags,7		; b stopky?
	goto	KeySto4		; stopky neb

	bcf	Flags,7		; zastaven stopek
	return

KeySto4	movf	SSecond,W	; naten sekund stopek
	iorwf	SMinute,W	; pidn minut
	iorwf	SHour,W		; pidn hodin
	btfss	STATUS,Z	; je poitadlo vynulovan?
	goto	KeySto5		; nen vynulovan

	bsf	Flags,7		; zapnut stopek
	return

KeySto5	clrf	SSecond		; nulovn sekund stopek
	clrf	SMinute		; nulovn minut stopek
	clrf	SHour		; nulovn hodin stopek
	return

;--------------------------------------------------------------------------
;	zapnut/vypnut budku
;--------------------------------------------------------------------------

KeyAlrm	call	KeyCh		; zmna na modifikovan stav

KeyAlr2	movlw	bit3		; pznak budku
	xorwf	Flags,F		; inverze pznaku budku

	movlw	bit4		; pznak signalizace
	btfss	Flags,3		; vypnuta signalizace budku?
	xorwf	Flags,F		; inverze pznaku signalizace
	return

;--------------------------------------------------------------------------
;	nastaven hodin budku
;--------------------------------------------------------------------------

KeyAHou	call	KeyCh		; zmna na modifikovan stav

KeyAHo2	bsf	Flags,3		; zapnut budku
	incf	AHour,F		; zven hodin budku
	movlw	24		; max. poet hodin
	subwf	AHour,W		; test peteen hodin
	btfsc	STATUS,C	; je peteen hodin?
	clrf	AHour		; nulovn hodin pi peteen
	return

;--------------------------------------------------------------------------
;	nastaven minut budku
;--------------------------------------------------------------------------

KeyAMin	call	KeyCh		; zmna na modifikovan stav

KeyAMi2	bsf	Flags,3		; zapnut budku
	incf	AMinute,F	; zven minut budku
	movlw	60		; max. poet minut
	subwf	AMinute,W	; test peteen minut
	btfsc	STATUS,C	; je peteen minut?
	clrf	AMinute		; nulovn minut pi peteen
	return

;--------------------------------------------------------------------------
;	nastaven hodin
;--------------------------------------------------------------------------

KeyHour	call	KeyCh		; zmna na modifikovan stav
	bcf	Flags,5		; vypnut pznaku bcho asu

KeyHou2	incf	Hour,F		; zven hodin
	movlw	24		; max. poet hodin
	subwf	Hour,W		; test peteen hodin
	btfsc	STATUS,C	; je peteen hodin?
	clrf	Hour		; nulovn hodin pi peteen
	movlw	bit0		; bit zmny zobrazen
	btfsc	STATUS,Z	; bylo peteen pes 24 hodin?
	xorwf	Flags2,F	; zmna typu zobrazen
	return

;--------------------------------------------------------------------------
;	nastaven minut
;--------------------------------------------------------------------------

KeyMin	call	KeyCh		; zmna na modifikovan stav
	bcf	Flags,5		; vypnut pznaku bcho asu

KeyMin2	incf	Minute,F	; zven minut
	movlw	60		; max. poet minut
	subwf	Minute,W	; test peteen minut
	btfsc	STATUS,C	; je peteen minut?
	clrf	Minute		; nulovn minut pi peteen
	return

;--------------------------------------------------------------------------
;	zmna melodie budku
;--------------------------------------------------------------------------

KeyDAl	call	KeyCh		; zmna na modifikovan stav
	goto	KeyDAl3

KeyDAl2	incf	AMelody,F	; zven melodie

KeyDAl3	bsf	Flags2,6	; zapnut poadavaku signalizace budku	
	bcf	Flags2,3	; peruen pehrvn melodie
	movlw	MELODS		; poet melodi
	subwf	AMelody,W	; test peteen sla melodie
	btfsc	STATUS,C	; je peteen sla melodie?
	clrf	AMelody		; nulovn sla melodie
	return

;--------------------------------------------------------------------------
;	nulovn sekund
;--------------------------------------------------------------------------

KeySec	call	KeyCh		; zmna na modifikovan stav

KeySec2	clrf	Hundrth		; nulovn setin sekundy
	movlw	30		; testovan hodnota poloviny minuty
	subwf	Second,W	; test, zda je druh polovina minuty
	btfsc	STATUS,C	; je druh polovina minuty?
	goto	IncMin		; zven tae minut
	clrf	Second		; jinak jen nulovn tae sekund
	return

;--------------------------------------------------------------------------
;	inicializace mdu pro prav tlatko
;--------------------------------------------------------------------------

KeyRMod	movlw	SDDATE
KeyRMd2	btfsc	Flags2,3	; probh pehrvn melodie?
	goto	KeySet2		; probh pehrvn melodie

KeySet	movwf	State
KeySet2	bcf	Flags2,3	; vypnut pznaku pehrvn melodie
	return

KeySetA	movlw	SDALARM
	goto	KeySet

;--------------------------------------------------------------------------
;	pepnut zobrazen data/dne v tdnu
;--------------------------------------------------------------------------

KeySetW	call	KeyCh		; zmna na modifikovan stav

KeyStW2	movlw	SDYEAR2
	goto	KeySet

KeySetD	call	KeyCh		; zmna na modifikovan stav

KeyStD2	movlw	SDDATE2
	goto	KeySet

;--------------------------------------------------------------------------
;	inicializace mdu pro lev tlatko
;--------------------------------------------------------------------------

KeyLMod	movlw	SALARM
	goto	KeyRMd2

;--------------------------------------------------------------------------
;	zven ukazatele stavu pro lev tlatko
;--------------------------------------------------------------------------

KeyIncL	btfss	Flags2,1	; je dlouh stisk levho tlatka?
	goto	KeyInc		; nen dlouh stisk levho tlatka
	clrf	RepKeyL		; ta opakovn na max. hodnotu
	movlw	SMONTH
	goto	KeySet

;--------------------------------------------------------------------------
;	zven ukazatele stavu pro prav tlatko
;--------------------------------------------------------------------------

KeyIncR	btfss	Flags2,2	; je dlouh stisk pravho tlatka?
	goto	KeyInc		; nen dlouh stisk pravho tlatka
	clrf	RepKeyR		; ta opakovn na max. hodnotu
	movlw	STIMER
	goto	KeySet

;--------------------------------------------------------------------------
;	zven ukazatele stavu
;--------------------------------------------------------------------------

KeyInc	incf	State,F
	return

;--------------------------------------------------------------------------
;	nastaven bnho stavu (as)
;--------------------------------------------------------------------------

KeyNrm0	clrf	Hundrth		; nulovn setin sekundy
	clrf	Second		; nulovn sekund

KeyNorm	bsf	Flags,5		; zapnut pznaku bcho asu
	movlw	SCLOCK
	goto	KeySet

;--------------------------------------------------------------------------
;	md tlatka zmnn druhm tlatkem
;--------------------------------------------------------------------------

KeyCh	movlw	SALARM2-SALARM	; rozdl stav
	addwf	State,F		; posun stavu
	return

;--------------------------------------------------------------------------
;	test rychlho blikn (C=zobrazeno, nen C=mazno)
;--------------------------------------------------------------------------

TBlick	movf	Hundrth,W	; naten tae setin sekundy do W
	sublw	50		; test, zda je druh polovina sekundy
	btfss	STATUS,C	; je druh polovina sekundy?
	addlw	50		; nvrat hodnoty na kladn slo
	sublw	25		; test, zda je druh tvrtina sekundy
	return

;--------------------------------------------------------------------------
;	zobrazen prvnho sla
;--------------------------------------------------------------------------

; ----- indikace vpadku napjen

DHour0	clrw
	goto	DekNLB

; ----- zobrazen msce

DMonth	movf	Month,W
	goto	DekNL

; ----- zobrazen dne v tdnu

DWeek	movf	Week,W
	goto	DekNL

; ----- zobrazen hodiny

DHour	movf	Hour,W
	goto	DekL

; ----- zobrazen hodiny s bliknm

DHourB	movf	Hour,W
	goto	DekLB

; ----- zobrazen hodin budku

DAHour	movf	AHour,W
	goto	DekL

; ----- zobrazen hodin budku s bliknm

DAHourB	movf	AHour,W
	goto	DekLB

; ----- zobrazen asovae

DTMinut	movf	TMinute,W
	goto	DekNL

; ----- zobrazen stopek (lev i prav slo)

DStop	movf	SHour,W
	btfss	STATUS,Z
	goto	DSMin2
	movf	SMinute,W
	sublw	19
	btfss	STATUS,C
	goto	DSMin2
	
	movf	SMinute,W
	call	DekNL
	movf	SSecond,W
	goto	DekR

DSMin2	movf	SHour,W
	call	DekNL
	movf	SMinute,W
	goto	DekR

;--------------------------------------------------------------------------
;	zobrazen druhho sla
;--------------------------------------------------------------------------

; ----- zobrazen sekund

DSecond	movf	Second,W
	goto	DekR

; ----- zobrazen data

DDay	movf	Day,W
	goto	DekR

; ----- zobrazen roku

DYear	movf	Year,W
	goto	DekR

; ----- zobrazen minut

DMinute	movf	Minute,W
	goto	DekR

; ----- zobrazen minut s bliknm

DMinutB	movf	Minute,W
	goto	DekRB

; ----- zobrazen melodie budku

DAMel	incf	AMelody,W
	call	DekR
	clrf	Num2
	return

; ----- zobrazen minut budku

DAMinut	movf	AMinute,W
	goto	DekR

; ----- zobrazen minut budku s bliknm

DAMinB	movf	AMinute,W
	goto	DekRB

; ----- zobrazen asovae

DTSecnd	movf	TSecond,W
	goto	DekR

; ----- zobrazen AM/PM

DAMPM	movlw	01011101b	; znak H
	btfsc	Flags2,0	; je zobrazen 24 hodin?
	goto	DAMPM2		; je zobrazen 24 hodin
	movlw	01011111b	; znak A
	btfsc	Num4,IAMPM	; je odpoledne?
	movlw	01001111b	; znak P
DAMPM2	movwf	Num1		; nastaven znaku 1
	clrf	Num2		; mazn znaku 2
	return

DAMPMB	call	DAMPM
	goto	DekRB2

;--------------------------------------------------------------------------
;	dekdovn W jako lev dvojsl (0 a 23 -> 12 AM/PM)
;--------------------------------------------------------------------------

; ----- schova sla k zobrazen

DekL	movwf	TempB		; schova bajtu k zobrazen

; ----- zobrazen v mdu 24 hodin

	bcf	Num4,IAMPM	; vypnut zobrazen indikace PM
	btfss	Flags2,0	; je zobrazen v mdu 24 hodin?
	goto	DekL0		; je zobrazen v mdu 12 hodin
	sublw	19		; test, zda je 20 hodin a vce
	btfsc	STATUS,C	; je 20 hodin a vce?
	goto	DekL2		; je mn ne 20 hodin	
	bsf	Num4,IAMPM	; zapnut indikace 20 hodin a vce
	movlw	10		; korekce
	subwf	TempB,F		; korekce daje k zobrazen
	goto	DekL2

; ----- dekdovn pznaku AM/PM

DekL0	sublw	11		; test, zda je odpoledne
	btfsc	STATUS,C	; je odpoledne?
	goto	DekL1		; je dopoledne
	bsf	Num4,IAMPM	; zapnut zobrazen indikace PM
	movlw	12		; korekce
	subwf	TempB,F		; korekce daje k zobrazen

; ----- korekce 0 na 12

DekL1	movlw	12		; nhradn hodnota
	movf	TempB,F		; test, zda je hodnota 0
	btfsc	STATUS,Z	; je hodnota 0?
	movwf	TempB		; nhrada hodnotou 12

; ----- dekdovn prvn slice

DekL2	bcf	Num4,IND1	; vypnut zobrazen slice 1
	movlw	10		; bude test pro hodnotu 10 a vce
	subwf	TempB,W		; test, zda je hodnota 10 a vce
	btfss	STATUS,C	; je hodnota 10 a vce?
	goto	DekL3		; nen 10 a vce
	movwf	TempB		; schova upraven slice
	bsf	Num4,IND1	; zapnut zobrazen slice 1

; ----- dekdovn druh slice

DekL3	movf	TempB,W		; naten slice k zobrazen
	call	ChrTab		; dekdovn na segmenty
	movwf	Num3		; nastaven jako slice 3
	return

;--------------------------------------------------------------------------
;	dekdovn W jako lev dvojsl (0 a 19)
;--------------------------------------------------------------------------

; ----- schova sla k zobrazen

DekNL	movwf	TempB		; schova bajtu k zobrazen

; ----- vypnut pznaku AM/PM

	bcf	Num4,IAMPM	; vypnut zobrazen indikace PM
	goto	DekL2		; dekdovn sla

;--------------------------------------------------------------------------
;	dekdovn W jako lev dvojsl (0 a 23 -> 12 AM/PM) s bliknm
;--------------------------------------------------------------------------

; ----- dekdovn sla k zobrazen

DekLB	call	DekL

; ----- test, zda je bliknut

DekLB2	call	TBlick		; test, zda je bliknut
	btfsc	STATUS,C	; bude bliknut?
	return			; nen bliknut

; ----- vymazn levho sla

ClrL	clrf	Num3		; vynulovn slice 3
	bcf	Num4,IAMPM	; vypnut zobrazen indikace PM
	bcf	Num4,IND1	; vypnut zobrazen slice 1
	return

;--------------------------------------------------------------------------
;	dekdovn W jako lev dvojsl (0 a 19) s bliknm
;--------------------------------------------------------------------------

; ----- dekdovn sla k zobrazen

DekNLB	call	DekNL
	goto	DekLB2

;--------------------------------------------------------------------------
;	dekdovn W jako prav dvojsl (0 a 99)
;--------------------------------------------------------------------------

; ----- schova sla k zobrazen

DekR	movwf	TempB		; schova bajtu k zobrazen

; ----- zjitn prvn slice

	clrf	TempC		; nulovn pomocnho tae
	movlw	10		; odetan hodnota

DekR2	incf	TempC,F		; zven pomocnho tae
	subwf	TempB,F		; odeten jedn dekdy
	btfsc	STATUS,C	; je ji podteen?
	goto	DekR2		; zjitn dal dekdy

	addwf	TempB,F		; nvrat posledn platn hodnoty bajtu
	decf	TempC,F		; nvrat posledn platn hodnoty tae

; ----- dekdovn prvn slice

	movf	TempC,W		; naten slice k zobrazen
	call	ChrTab		; dekdovn na segmenty
	movwf	Num2		; nastaven jako slice 2

; ----- dekdovn druh slice

	movf	TempB,W		; naten slice k zobrazen
	call	ChrTab		; dekdovn na segmenty
	movwf	Num1		; nastaven jako slice 1
	return

;--------------------------------------------------------------------------
;	dekdovn W jako prav dvojsl (0 a 99) s bliknm
;--------------------------------------------------------------------------

; ----- dekdovn sla k zobrazen

DekRB	call	DekR

; ----- test, zda je bliknut

DekRB2	call	TBlick		; test, zda je bliknut
	btfsc	STATUS,C	; bude bliknut?
	return			; nen bliknut

; ----- vymazn pravho sla

ClrR	clrf	Num1		; vynulovn slice 1
	clrf	Num2		; vynulovn slice 2
	return

;--------------------------------------------------------------------------
;	dekdovn dvojteky s pomalm bliknm
;--------------------------------------------------------------------------

; ----- blikn podle stopek

DekBS	movf	SHundr,W	; setiny stopek
	goto	DekB2

; ----- blikn podle asovae

DekBT	movf	THundr,W	; setiny asovae
	sublw	100		; doplnk setin do 100
	goto	DekB2

; ----- blikn podle asu

DekB	movf	Hundrth,W	; naten tae setin sekundy do W
DekB2	bsf	Num4,ITIME	; zapnut dvojteky uprosted
	sublw	50		; test, zda je druh polovina sekundy
DekB3	btfss	STATUS,C	; je druh polovina sekundy?

; ----- vypnut dvojteky

DekBOff	bcf	Num4,ITIME	; vypnut dvojteky uprosted
	return

; ----- zapnut dvojteky

DekBOn	bsf	Num4,ITIME	; zapnut dvojteky
	return

;--------------------------------------------------------------------------
;	dekdovn dvojteky s rychlm bliknm
;--------------------------------------------------------------------------

DekBB	bsf	Num4,ITIME	; zapnut dvojteky uprosted
	call	TBlick		; test, zda je bliknut
	goto	DekB3		; obsluha bliknut

;--------------------------------------------------------------------------
;	zapnut teky v datu
;--------------------------------------------------------------------------

DekTec	bsf	Num2,7
	goto	DekBOff

;--------------------------------------------------------------------------
;	nastaven msce (volno i z obsluhy inkrementace data, vrac C=peteen)
;--------------------------------------------------------------------------

KeyMon	incf	Month,F		; zven msce
	movlw	13		; max. poet msc
	subwf	Month,W		; test peteen msc
	movlw	1		; nov hodnota msc
	btfsc	STATUS,C	; je peteen msc?
	movwf	Month		; nulovn msce pi peteen
	return

;--------------------------------------------------------------------------
;	nastaven dne
;--------------------------------------------------------------------------

KeyDay	incf	Day,F		; zven dne
	movlw	32		; max. poet dn
	subwf	Day,W		; test peteen dn
	movlw	1		; nov hodnota dn
	btfsc	STATUS,C	; je peteen dn?
	movwf	Day		; nulovn dne pi peteen
	return

;--------------------------------------------------------------------------
;	nastaven dne v tdnu (volno i z obsluhy inkrementace data)
;--------------------------------------------------------------------------

KeyWeek	incf	Week,F		; zven dne v tdnu
	movlw	7		; max. poet dn v tdnu
	subwf	Week,W		; test peteen dn v tdnu
	btfsc	STATUS,C	; je peteen dn v tdnu?
	clrf	Week		; nulovn dne v tdnu pi peteen
	return

;--------------------------------------------------------------------------
;	nastaven roku (volno i z obsluhy inkrementace data)
;--------------------------------------------------------------------------

KeyYear	incf	Year,F		; zven roku
	movlw	100		; max. poet rok
	subwf	Year,W		; test peteen roku
	btfsc	STATUS,C	; je peteen roku?
	clrf	Year		; nulovn roku pi peteen
	return

;--------------------------------------------------------------------------
;	zven aktulnho asu
;--------------------------------------------------------------------------

; ----- zven tae sekund

IncSec	clrf	Hundrth		; nulovn tae setin sekundy
	incf	Second,F	; zven tae sekund
	movf	Second,W	; naten tae sekund
	sublw	60		; test maximln hodnoty sekund
	btfss	STATUS,Z	; je peteen sekund?
	return

; ----- zven tae minut

IncMin	clrf	Second		; nulovn tae sekund
	incf	Minute,F	; zven tae minut
	movf	Minute,W	; naten tae minut
	sublw	60		; test maximln hodnoty minut
	btfss	STATUS,Z	; je peteen minut?
	return			; nen peteen minut

; ----- zven tae hodin

IncHour	clrf	Minute		; nulovn tae minut
	btfsc	Flags,4		; je poadovna signalizace?
	bsf	Flags2,4	; poadavek k pehrt signalizace
	incf	Hour,F		; zven tae hodin
	movf	Hour,W		; naten tae hodin
	sublw	24		; test maximln hodnoty hodin
	btfss	STATUS,Z	; je peteen hodin?
	return			; nen peteen hodin

; ----- zven tae dn

IncDay	clrf	Hour		; nulovn tae hodin

	call	KeyWeek		; inkrementace dne v tdnu

	incf	Day,F		; zven tae dn
	movf	Day,W		; naten tae dn
	sublw	32		; test maximln hodnoty dn
	btfsc	STATUS,Z	; je peteen dn?
	goto	IncMon		; je peteen dn

	clrw			; W <- 0
	btfsc	Month,2		; je druh polovina roku?
	movlw	bit0		; pro druhou polovinu roku inverze
	xorwf	Month,W		; naten bitu lichosti msce
	andlw	bit0		; maskovn bitu lichosti
	btfsc	STATUS,Z	; je dlouh msc?
	return			; je dlouh msc - nen peteen tae dn

	movf	Day,W		; naten tae dn
	sublw	31		; test maximln hodnoty krtkho msce
	btfsc	STATUS,Z	; je peteen krtkho msce>
	goto	IncMon		; je peteen krtkho msce

	movlw	2		; msc nor
	subwf	Month,W		; test, zda je nor
	btfss	STATUS,Z	; je msc nor?
	return			; nen nor - nen peteen

	movlw	bit0+bit1	; maska pro test pestupnho roku
	andwf	Year,W		; test pestupnosti roku (rok 2000 byl pestupn)
	movlw	30		; poet dn pro pestupn nor
	btfss	STATUS,Z	; je rok pestupn?
	movlw	29		; poet dn pro nepestupn nor
	subwf	Day,W		; test konce nora
	btfss	STATUS,Z	; je konec nora?
	return			; nen konec nora

; ----- zven tae msc a let

IncMon	movlw	1
	movwf	Day		; inicializace tae dn
	call	KeyMon		; inkrementace msce
	btfsc	STATUS,C	; je peteen?
	call	KeyYear		; inkrementace roku
	return

;--------------------------------------------------------------------------
;	obsluha peruen
;--------------------------------------------------------------------------

; ----- schova registr W a STATUS

Inter	movwf	TempW		; schova registru W
	swapf	STATUS,W	; naten STATUS registru do W
	movwf	TempS		; schova STATUS registru

; ----- nulovn pznaku peruen

	bcf	INTCON,2	; nulovn pznaku peruen

; ----- doplujc korekce asovn (dopluje zpodn inicializace na 16 us)
; Vpoet korekce:
;	aktivace vyvoln peruen ........ 2 us
;	instrukce goto Inter ............... 2 us
;	instrukce pro schovu registr ..... 3 us
;	instrukce pro nulovn peruen ... 1 us
;	instrukce pro nastaven tae ..... 2 us
;	dodaten zastaven tae ......... 2 us
;					------------
;	celkem ............................ 12 us
;	do 16 us zbv doplnit ............. 4 us	

	goto	$+1		; doplujc prodleva 2 us
	goto	$+1		; doplujc prodleva 2 us

; ----- inicializace tae TMR0

	movlw	100		; nov hodnota tae do W (doplnk do 256 je 156)
	movwf	TMR0		; nastaven registru asovae TMR0 (mus bt banka 0)

; ----- tn dlky tnu melodie

	decf	Length,F	; snen tae dlky tnu melodie

; ----- tn setin sekundy

	incf	Hundrth,F	; zven tae setin sekundy
	movf	Hundrth,W	; naten tae setin sekundy do W
	sublw	100		; test maximln hodnoty setin sekundy
	btfss	STATUS,Z	; je peteen setin sekundy?
	goto	Inter2		; nen jet peteen setin sekundy
	clrf	Hundrth		; nulovn tae setin sekundy

; ----- tn asu

	btfsc	Flags,5		; je as zastaven?
	call	IncSec		; tn sekund

; ----- tn stopek

Inter2	btfss	Flags,7		; b stopky?
	goto	Inter22		; stopky neb

	incf	SHundr,F	; zven tae setin sekundy stopek
	movf	SHundr,W	; naten tae setin sekundy stopek
	sublw	100		; test maximln hodnoty setin sekundy stopek
	btfss	STATUS,Z	; je peteen setin sekundy stopek?
	goto	Inter22		; nen jet peteen setin sekundy stopek
	clrf	SHundr		; nulovn tae setin sekundy stopek

	incf	SSecond,F	; zven tae sekund stopek
	movf	SSecond,W	; naten tae sekund stopek
	sublw	60		; test maximln hodnoty sekund stopek
	btfss	STATUS,Z	; je peteen sekund stopek?
	goto	Inter22		; nen jet peteen sekund stopek
	clrf	SSecond		; nulovn tae sekund stopek

	incf	SMinute,F	; zven tae minut stopek
	movf	SMinute,W	; naten tae minut stopek
	sublw	60		; test maximln hodnoty minut stopek
	btfss	STATUS,Z	; je peteen minut stopek?
	goto	Inter22		; nen jet peteen minut stopek
	clrf	SMinute		; nulovn tae minut stopek

	incf	SHour,F		; zven tae hodin stopek
	movf	SHour,W		; naten tae hodin stopek
	sublw	19		; test maximln hodnoty hodin stopek
	btfsc	STATUS,Z	; je peteen hodin stopek?
	clrf	SHour		; nulovn hodin stopek

; ----- test dosaen stavu pro budk

Inter22	btfss	Flags,3		; je budk zapnut?
	goto	Inter24		; budk nen zapnut

	movf	Hundrth,W	; naten setin sekundy
	iorwf	Second,W	; pidn sekund
	btfss	STATUS,Z	; jsou setiny i sekundy = 0?
	goto	Inter24		; setiny a sekundy nejsou = 0

	movf	Minute,W	; naten minut
	subwf	AMinute,W	; porovnn s minutou budku
	btfss	STATUS,Z	; je minuta shodn?
	goto	Inter24		; minuta nen shodn

	movf	Hour,W		; naten hodin
	subwf	AHour,W		; porovnn s hodinou budku
	btfss	STATUS,Z	; je hodina shodn?
	goto	Inter24		; hodina nen shodn

	bsf	Flags2,6	; zapnut poadavaku signalizace budku	

; ----- tn asovae

Inter24	btfss	Flags,6		; b asova?
	goto	Inter28		; asova neb

	movf	THundr,F	; test setin sekundy asovae, zda jsou = 0
	btfss	STATUS,Z	; jsou setiny sekundy asovae ji = 0?
	goto	Inter27		; setiny jet nejsou = 0
	movlw	100		; nov inicializan hodnota setin sekundy asovae
	movwf	THundr		; inicializace setin sekundy asovae

	movf	TSecond,F	; test sekund asovae, zda jsou = 0
	btfss	STATUS,Z	; jsou sekundy asovae ji = 0?
	goto	Inter26		; sekundy jet nejsou = 0
	movlw	60		; nov inicializan hodnota sekund asovae
	movwf	TSecond		; inicializace sekund asovae

	decf	TMinute,F	; snen tae minut asovae

Inter26	decf	TSecond,F	; snen tae sekund asovae
	btfss	STATUS,Z	; doshly sekundy = 0?
	goto	Inter27		; sekundy jet nedoshly 0

	movf	TMinute,F	; test minut, zda jsou tak = 0
	btfss	STATUS,Z	; jsou minuty tak = 0?
	goto	Inter27		; minuty jet nejsou = 0

	bcf	Flags,6		; vypnut asovae
	bsf	Flags2,5	; poadavek indikace asovae

Inter27	decf	THundr,F	; snen tae setin sekundy asovae

; ----- zobrazen

Inter28	movf	State,W		; aktuln stav
	call	DspLTab		; zobrazen levho sla

	movf	State,W		; aktuln stav
	call	DspRTab		; zobrazen pravho sla

	movf	State,W		; aktuln stav
	call	DspDTab		; zobrazen dvojteky a teky

	bcf	Num4,IALARM	; vypnut indiktoru budku
	btfsc	Flags,3		; je zapnut budk?
	bsf	Num4,IALARM	; zapnut indiktoru budku

	bcf	Num4,ISIGNAL	; vypnut indiktoru signalizace
	btfsc	Flags,4		; je zapnuta signalizace?
	bsf	Num4,ISIGNAL	; zapnut indiktoru signalizace

	call	ReDisp		; pekreslen displeje

; ----- automatick vypnut zobrazen roku

	movlw	SDYEAR		; stav zobrazen roku
	subwf	State,W		; test, zda je zobrazen rok
	btfsc	STATUS,Z	; je zobrazen rok?
	goto	Inter32		; je zobrazen rok
	
; ----- automatick vypnut zobrazen data

	movlw	SDDATE		; stav zobrazen data
	subwf	State,W		; test, zda je zobrazeno datum
	movlw	SDYEAR		; nov stav
	btfsc	STATUS,Z	; je zobrazeno datum?
	goto	Inter34		; je zobrazeno datum

; ----- automatick vypnut zobrazen budku

	movlw	SDALARM		; stav zobrazen budku
	subwf	State,W		; test, zda je zobrazen budk
	btfss	STATUS,Z	; je zobrazen budk?
	goto	Inter36		; nen zobrazen budk
Inter32	movlw	SCLOCK		; nov stav
Inter34	decf	ADisp,F		; tn zobrazen
	btfsc	STATUS,Z	; je dosaeno 0 asu?
	movwf	State		; nastaven normlnho stavu

; ----- obsluha levho tlatka

Inter36	btfss	Flags,1		; je stisknuto lev tlatko?
	goto	Inter4		; lev tlatko nen stisknuto
	movf	State,W		; aktuln stav
	call	KeyLTab		; obsluha levho tlatka

; ----- obsluha pravho tlatka

Inter4	btfss	Flags,2		; je stisknuto prav tlatko?
	goto	Inter5		; prav tlatko nen stisknuto
	clrf	ADisp		; nulovn tae zobrazen budku a data
	movf	State,W		; aktuln stav
	call	KeyRTab		; obsluha pravho tlatka

; ----- nvrat registr W a STATUS

Inter5	swapf	TempS,0		; pvodn STATUS registr k navrcen
	movwf	STATUS		; nvrat pvodnho STATUS registru
	swapf	TempW,1		; pedpprava pvodnho registru W
	swapf	TempW,0		; nvrat pvodnho W registru
	retfie			; nvrat z obsluhy peruen

;--------------------------------------------------------------------------
;	pekreslen displeje LCD (mus bt volno kadch 10 ms)
;--------------------------------------------------------------------------

; ----- detekce stisku levho tlatka (signlem Clock)

ReDisp	bsf	PORTA,CLKBIT	; zapnut hodinovho signlu

	bcf	Flags,1		; vypnut pznaku stisku levho tlatka
	bcf	Flags2,1	; vypnut pznaku dlouhho stisku levho tlatka

	movf	RelKeyL,F	; test, zda b ta uvolnn levho tlatka
	btfsc	STATUS,Z	; b ta uvolnn levho tlatka?
	goto	ReDisp1		; ta neb

	decfsz	RepKeyL,F	; ta opakovn tlatka
	goto	ReDsp12		; nen jet 0
	movlw	REPKEY2		; opakovan interval opakovn tlatka
	bsf	Flags2,1	; pznak dlouhho stisku levho tlatka
	goto	ReDsp14

ReDsp12	btfsc	PORTA,KEYBIT	; je tlatko stisknuto?
	goto	ReDisp2		; je stisknuto - inicializace tae uvolnn
	decf	RelKeyL,F	; snen tae uvolnn
	goto	ReDisp3

ReDisp1	btfss	PORTA,KEYBIT	; je tlatko stisknuto?
	goto	ReDisp3		; tlatko nen stisknuto

	movlw	REPKEY1		; poten interval opakovn tlatka
ReDsp14	movwf	RepKeyL		; opakovn levho tlatka
	bsf	Flags,1		; pznak stisku levho tlatka
ReDisp2	movlw	FREEKEY		; nov hodnota tae (50 ms)
	movwf	RelKeyL		; nov inicializace tae uvolnn

ReDisp3	bcf	PORTA,CLKBIT	; vypnut hodinovho signlu

; ----- pprava ukazatele dat k odesln

	movlw	Num1		; ukazatel na 1. slo k odesln
	movwf	FSR		; nastaven ukazatele dat

; ----- pprava tae bajt k odesln

	movlw	4		; poet bajt k odesln
	movwf	CBytes		; nastaven tae bajt k odesln

; ----- pprava jednoho bajtu k odesln

ReDisp4	movf	INDF,W		; naten bajtu k odesln
	movwf	DByte		; bajt k odesln

; ----- inverze bajtu

	btfsc	Flags,0		; je inverze vstupu?
	comf	DByte,F		; inverze bajtu k odesln

; ----- pprava potu bit k odesln

	movlw	8		; poet bit k odesln
	movwf	CBits		; ta bit k odesln

; ----- pprava datovho bitu

ReDisp5	rrf	DByte,F		; rotovn nejniho bitu do C
	bcf	PORTA,DATBIT	; nulovn datovho bitu
	btfsc	STATUS,C	; byl bit nastaven?
	bsf	PORTA,DATBIT	; nastaven datovho bitu

; ----- vysln hodinovho impulsu

	bsf	PORTA,CLKBIT	; zapnut hodinovho signlu
	bcf	PORTA,CLKBIT	; vypnut hodinovho signlu

; ----- cyklus pro dal bit

	decfsz	CBits,F		; ta bit k odesln
	goto	ReDisp5		; odesln dalho bitu dat

; ----- zven ukazatele bajt

	incf	FSR,F		; zven ukazatele dat

; ----- cyklus pro dal bajt

	decfsz	CBytes,F	; ta bajt k odesln
	goto	ReDisp4		; odesln dalho bajtu

; ----- strobovn - zpis dat do vstupnch registr,
;	souasn detekce stisku pravho tlatka (signlem Strobe)

	bsf	PORTA,STRBIT	; zapnut strobovacho signlu

	bcf	Flags,2		; vypnut pznaku stisku pravho tlatka
	bcf	Flags2,2	; vypnut pznaku dlouhho stisku pravho tlatka

	movf	RelKeyR,F	; test, zda b ta uvolnn pravho tlatka
	btfsc	STATUS,Z	; b ta uvolnn pravho tlatka?
	goto	ReDisp6		; ta neb

	decfsz	RepKeyR,F	; ta opakovn tlatka
	goto	ReDsp52		; nen jet 0
	movlw	REPKEY2		; opakovan interval opakovn tlatka
	bsf	Flags2,2	; pznak dlouhho stisku pravho tlatka
	goto	ReDsp62

ReDsp52	btfsc	PORTA,KEYBIT	; je tlatko stisknuto?
	goto	ReDisp7		; je stisknuto - inicializace tae uvolnn
	decf	RelKeyR,F	; snen tae uvolnn
	goto	ReDisp8

ReDisp6	btfss	PORTA,KEYBIT	; je tlatko stisknuto?
	goto	ReDisp8		; tlatko nen stisknuto

	movlw	REPKEY1		; poten interval opakovn tlatka
ReDsp62	movwf	RepKeyR		; opakovn pravho tlatka
	bsf	Flags,2		; pznak stisku pravho tlatka
ReDisp7	movlw	FREEKEY		; nov hodnota tae (50 ms)
	movwf	RelKeyR		; nov inicializace tae uvolnn

ReDisp8	bcf	PORTA,STRBIT	; vypnut strobovacho signlu

; ----- peklopen pznaku inverze

	movlw	bit0		; bit k peklopen stavu
	xorwf	Flags,F		; peklopen pznaku inverze
	return
