;**************************************************************************
; LCD.ASM					(c) Ing. Miroslav Nmeek
;
; Demonstran program modulu P84-LCD
;--------------------------------------------------------------------------
; Program demonstruje pouit LCD displeje - zobraz blikajc text":P84".
;**************************************************************************

	CBLOCK

; ----- zobrazovan znaky
; bajty slic Num1 a Num4 mus v pamti za sebou nsledovat!
;
; Obsazen segment pro slice Num1 a Num3:
;	bit 0: segment B (vpravo nahoe)
;	bit 1: segment A (nahoe)
;	bit 2: segment F (vlevo nahoe)
;	bit 3: segment G (uprosted)
;	bit 4: segment C (vpravo dole)
;	bit 5: segment D (dole)
;	bit 6: segment E (vlevo dole)
;	bit 7: segment H (teka)
;
; Obsazen segment pro Num4:
;	bit 0: ipka
;	bit 1: dvojteka vlevo
;	bit 2: inverze displeje
;	bit 3: dvojteka uprosted
;	bit 4: ... nepouito
;	bit 5: slice 1 vlevo
;	bit 6: znamnko mnus
;	bit 7: vlnovka
;
; Bitov mapa segment slice:
;
;	 -- 1 --
;	|	|
;	2	0
;	|	|
;	 -- 3 --
;	|	|
;	6	4
;	|	|
;   7	 -- 5 --

Num1				; vstupn bajt slice 1 (vpravo)
Num2				; vstupn bajt slice 2 (uprosted)
Num3				; vstupn bajt slice 3 (vlevo)
Num4				; vstupn bajt slice 4 (indikace)

Flags				; pznaky
				;  bit 0: 1=je inverze
				;  bit 1: 1=je bliknut

CFlash				; ta blikn
CBytes				; ta bajt k odesln
DByte				; vyslan bajt na displej
CBits				; ta bit vyslanho bajtu
	ENDC

	CONFIG	WDT		; zapnut asova WatchDog

; ----- inicializace portu A

Start	bsf	STATUS,RP0	; zapnut banky 1
	movlw	10000b		; volba smru bit: 0=vstup, 1=vstup
	movwf	TRISA		; nastaven bit portu A (registr TRISA)

; ----- nastaven asovae Watchdog

	movlw	0+bit3		; peddlika k WDT, dlic pomr 1/1
	movwf	OPTION_REG	; nastaven registru OPTION registru
	bcf	STATUS,RP0	; zapnut banky 0

; ----- pprava znak k zobrazen

	movlw	00011101b	; znak "4"
	movwf	Num1		; nastaven znaku 1
	movlw	01111111b	; znak "8"
	movwf	Num2		; nastaven znaku 2
	movlw	01001111b	; znak "P"
	movwf	Num3		; nastaven znaku 3
	movlw	00000010b	; bit indiktoru ":"
	movwf	Num4		; nastaven znaku 4

; ----- inicializace tae blikn

Flash	movlw	20		; doba bliknut
	movwf	CFlash		; nastaven tae blikn

; ----- ekn na impuls od WDT

Loop	SLEEP			; ekn na impuls WDT

; ----- pprava ukazatele dat k odesln

	movlw	Num1		; ukazatel na 1. slo k odesln
	movwf	FSR		; nastaven ukazatele dat

; ----- pprava tae bajt k odesln

	movlw	4		; poet bajt k odesln
	movwf	CBytes		; nastaven tae bajt k odesln

; ----- pprava jednoho bajtu k odesln

OutByte	movf	INDF,W		; naten bajtu k odesln
	movwf	DByte		; pprava bajtu k odesln

; ----- nulovn bajtu pi blikn

	btfsc	Flags,1		; je bliknut?
	clrf	DByte		; nulovn bajtu pi bliknut

; ----- inverze bajtu

	btfsc	Flags,0		; je inverze vstupu?
	comf	DByte,F		; inverze bajtu k odesln

; ----- pprava potu bit k odesln

	movlw	8		; poet bit k odesln
	movwf	CBits		; ta bit k odesln

; ----- pprava datovho bitu

OutBit	rrf	DByte,F		; rotovn nejniho bitu do C
	bcf	PORTA,0		; nulovn datovho bitu
	btfsc	STATUS,C	; byl bit nastaven?
	bsf	PORTA,0		; nastaven datovho bitu

; ----- vysln hodinovho impulsu

	bsf	PORTA,1		; zapnut hodinovho signlu
	bcf	PORTA,1		; vypnut hodinovho signlu

; ----- cyklus pro dal bit

	decfsz	CBits,F		; ta bit k odesln
	goto	OutBit		; odesln dalho bitu dat

; ----- zven ukazatele bajt

	incf	FSR,F		; zven ukazatele dat

; ----- cyklus pro dal bajt

	decfsz	CBytes,F	; ta bajt k odesln
	goto	OutByte		; odesln dalho bajtu

; ----- strobovn - zpis dat do vstupnch registr

	bsf	PORTA,2		; zapnut strobovacho signlu
	bcf	PORTA,2		; vypnut strobovacho signlu

; ----- peklopen pznaku inverze

	movlw	bit0		; bit k peklopen stavu
	xorwf	Flags,F		; peklopen pznaku inverze

; ----- tn blikn

	decfsz	CFlash,F	; tn doby bliknut
	goto	Loop		; nen jet bliknut
	movlw	bit1		; bit k peklopen stavu
	xorwf	Flags,F		; zmna pznaku inverze
	goto	Flash		; je bliknut

