;**************************************************************************
; LEKCE03.ASM					(c) Ing. Miroslav Nmeek
;
; 3. lekce programovn procesor PIC - asov prodleva asovaem Watchdog
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo 4 LED diody pipojen k P84-CPU mezi signly A0-A3 a 0V (zem)
;**************************************************************************
;
; V minul lekci jsme se seznmili s vytvenm asovch prodlev pomoc
; doby provdn instrukc. Uveden metoda je vhodn pedevm k vytven
; krtkch asovch prodlev s velkou pesnost. Nezle-li nm pli
; na pesnosti, meme vyut asova Watchdog.
;
; asova Watchdog procesor PIC je uren pedevm k hldn funknosti
; bcho programu. Sprvn fungujc program pi svm bhu neustle
; nuluje asova Watchdog instrukc CLRWDT. Pokud program z njakho
; dvodu havaruje a nedojde do urit doby k vynulovn asovae, obvod
; Watchdog procesor resetuje a program se rozebhne opt od zatku.
;
; asova Watchdog meme vyut v jeho jinm reimu innosti. Pomoc
; instrukce SLEEP se uvede procesor do stavu klidu (procesor uspme), kdy
; je osciltor procesoru zastaven. Program stoj na nsledujc instrukci
; a krom proudu odebranho pipojenmi periferiemi m minimln odbr
; proudu. Po uplynut jist doby obvody asovae Watchdog procesor opt
; probud, tj. rozebhnou osciltor a procesor pokrauje v provdn
; programu instrukc nsledujc za instrukc SLEEP.
;--------------------------------------------------------------------------
; Vytvome podobn program jako v pedel lekci, tedy blikajc LED
; v piblin jednosekundovch intervalech. Ukzkov jsme vyhodili
; vechny nadbyten povely pekladae. Doplnili jsme nsledujc
; povel CONFIG WDT kter zajist, e obvod Watchdog bude aktivn.


	CONFIG	WDT	; procesor bude pouvat obvod Watchdog


; ----- nastaven smru bit portu A
; Tak jako v pedelch lekcch, musme i zde zajistit zapnut bit
; portu A jako vstup (nastavenm registru TRISA). Nebudeme ale
; vypnat banku 1, budeme ji jet potebovat.

	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 1)
			; ... instrukce BSF f,b nastav bit b registru f

	movlw	10000b	; volba smru bit: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A (registr TRISA)
			; ... instrukce MOVWF f ulo registr W do registru f


; ----- nastaven asovae Watchdog
; asova Watchdog pouv ke sv innosti vnitn RC len procesoru
; s asovou konstantou typicky 18 ms. asov konstanta je zvisl na
; teplot, napjecm napt a li se i u jednotlivch procesor.
; Vrobce pipout toleranci 7 a 33 ms.
;
; K nastaven asovae Watchdog slou registr OPTION, kter m adresu 1
; a je pstupn v bance 1. Pro del as asovae Watchdog meme
; za asova pipojit dliku, kter zajist vynsoben zkladn asov
; konstanty asovae. Od asovae poadujeme dobu 500 ms. Pro 18 ms
; je to dlic konstanta 28, pouijeme nejbli dlic pomr, tedy 32.

	movlw	5+bit3	; dlika WDT 1/32, je pipojena k WDT
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	1	; nastaven OPTION registru asova
			; ... instrukce MOVWF f ulo registr W do registru f

	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 0)
			; ... instrukce BCF f,b vynuluje bit b registru f

; Hodnotu pro nastaven registru jsme si pipravili jako souet 5+bit3.
; slo 5 je dlic pomr dliky, kter nastavujeme do bit 0 a 2
; a pedstavuje dlic pomr pro WDT 1/32, tj doba asi 576 ms. Hodnota bit3
; je peddefinovan konstanta pekladae majc hodnotu 8, pouijeme ji
; k nastaven bitu 3 registru, tm piadme peddliku za obvod WDT.
; Nsledujc instrukc MOVWF 1 registr OPTION nastavme. Zvrem
; zvolme banku registr 0, protoe vstup na port A bude provdn
; pomoc registru PORTA, kter le v bance 0.

; ----- ekn na asova Watchdog
; Jak jsme si ekli ve, instrukce SLEEP usp procesor, tj. vypne jeho
; osciltor, zastav program a pepne procesor do reimu snen spoteby.
; Po uplynut zvolenho asu. tj. asi 0.5 sekundy, probud obvod Watchdog
; procesor a program pokrauje instrukc nsledujc za instrukc SLEEP.

Blikej	SLEEP

; ----- indikace LED
; Na rozdl od pedel lekce budeme provdt indikaci LED instrukc INCF.
; Instrukce zpsob zven hodnoty bajtu na portu A o 1. Port nyn
; funguje jako dlika. To se projev tak, e LED dioda pipojen k bitu 0
; bude blikat v sekundovch intervalech (doba bliknut 0.5 sekundy),
; LED dioda pipojen k bitu 1 bude mt periodu 2 sekundy, bit 2 bude blikat
; s periodou 4 sekundy a bit 3 s periodou 8 sekund.

	incf	5,1	; inkrementace portu A; druh parametr "1" udv, e
			; vsledek operace inkrementace je uloen zpt do registru

; ----- opakovn
; Posledn instrukc programu bude instrukce GOTO, ta zajist opakovan blikn
	
	goto	Blikej


; ----- start programu
; Vylete program do procesoru. LED diody budou blikat jako binrn ta.
; Pomoc stopek mete pozorovat, e blikn se nyn trochu odchyluje
; od poadovan 1 sekundy, zpravidla bude trochu pomalej. Pesnost mete
; zvit, odpojte-li peddliku od WDT a instrukci SLEEP zavolte
; pomoc cyklu (nebo vcensobnm pouitm) opakovan - nejspe 28x.
