;**************************************************************************
; LEKCE05.ASM					(c) Ing. Miroslav Nmeek
;
; 5. lekce programovn procesor PIC - tabulky
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo 4 LED diody pipojen k P84-CPU mezi signly A0-A3 a 0V (zem),
;	     4 LED diody pipojen k P84-CPU mezi signly B0-B3 a 0V (zem)
;
; Rozloen pouit matice LED:
;		B1  B0  A3
;		B2      A2
;		B3  A0  A1
;**************************************************************************
;
; Harvardova architektura procesor PIC s oddlenmi sbrnicemi pro
; program a data zajiuje vy stabilitu bcho programu (dky
; nemonosti sebhnut programu do RAM i pepisu pamti programu)
; a vy rychlost provdn instrukc (dky monosti pouit nestejn
; ky datov a programov sbrnice). Co vak v ppad, kdy potebujeme
; st data z tabulky konstant programu?
;
; Pro prci s tabulkami je urena instrukce RETLW. Instrukce slou
; k nvratu z podprogramu s tm, e do registru W ulo konstantu
; zadanou jako parametr instrukce. Instrukce pitom zabr 1 programov
; slovo stejn tak, jako by zabral datov bajt v pamti programu.
; Tabulku potom tvo posloupnost instrukc RETLW, na zatek tabulky
; zpravidla umisujeme instrukci modifikujc registr PCL, kter je
; ukazatelem programu, m program zajist skok na pslun msto tabulky.
;--------------------------------------------------------------------------
; S vyuitm tabulky si vytvome jednoduch program, kter zajist
; "bhn svtlka" po obvodu matice LED diod jednotky P84-UNI. K asovn
; vyuijeme asova Timer0, ovem namsto peruen ho vyuijeme v jednoduchm
; dotazovacm mdu.

; ----- zen pekladae, datov promnn

	RADIX	DEC	; pouijeme dekadickou selnou soustavu

	CBLOCK
Pointer			; ukazatel v tabulce konstant
Temp			; pechodn uloen bajtu k odesln na port
	ENDC

; ----- start programu
; Datov tabulky v nejjednodum tvaru, jak je pouijeme, mus bt cel
; umstn v prvnch 256 bajtech programu. Proto prvn instrukc bude skok
; na zatek programu. U rozshlejho programu si obas program pelote
; a zkontrolujte ve vpisu, zda se tabulka nedostala mimo prvnch 256 bajt.

	ORG	0	; procesor startuje od adresy 0
	goto	Start	; skok na zatek programu

;--------------------------------------------------------------------------
;	Tabulky
;--------------------------------------------------------------------------

; ----- tabulka indikanch hodnot
; Prvn instrukc tabulky je instrukce ADDWF PCL,F, kter zajist piten
; obsahu registru W k registru PCL (adresa 2). Registr PCL je ukazatelem
; programu. Tak se provede skok na instrukci v tabulce, jej offset
; byl v registru W. Namsto zpisu instrukc RETLW vyuijeme pseudoinstrukce
; pekladae DT, kter zajist vygenerovn kdu instrukce RETLW.
; Pokud bychom chtli vytvoit tabulku lec mimo rozsah potench
; 256 bajt programov pamti, vyuili bychom k tomu registr PCLATH.

Table	addwf	2,1	; piten W k ukazateli programu PCL
			; ... instrukce ADDWF f,d sete registr W s obsahem
			;     registru f. Je-li druh parametr d=1, ulo
			;     vsledek do registru f. (Pro d=0 ulo do W)
	dt	00000001b	; 0: svt dioda A0
	dt	00000010b	; 1: svt dioda A1
	dt	00000100b	; 2: svt dioda A2
	dt	00001000b	; 3: svt dioda A3
	dt	00010000b	; 4: svt dioda B0
	dt	00100000b	; 5: svt dioda B1
	dt	01000000b	; 6: svt dioda B2
	dt	10000000b	; 7: svt dioda B3

; Po prvnch pokusech zkuste obsah tabulky zmodifikovat a vytvoit si
; tak vlastn animovan obrazce z LED.

;--------------------------------------------------------------------------
;	Start programu
;--------------------------------------------------------------------------

; ----- nastaven smru bit port A a B
; Bity 0 a 3 port A i B nastavme jako vstupy.

Start	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 1)
			; ... instrukce BSF f,b nastav bit b registru f

	movlw	11110000b ; volba smru bit portu A a B: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A (registr TRISA)
	movwf	6	; nastaven bit portu B (registr TRISB)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nastaven asovae Timer0
; asova Timer0 pouijeme tak, e jeho peddliku nastavme na dlic
; pomr 1/256 a asova nechme cyklovat pes vech 256 hodnot. Tak doshneme
; periody 256*256 = 65536 takt, pro krystal 4 MHz to odpovd dob 65.5 ms.

	movlw	7+bit7	; dlika TMR0 1/128, pipojena k TMR0, zdrojem hodiny
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	1	; nastaven OPTION registru asova
			; ... instrukce MOVWF f ulo registr W do registru f

	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 0)
			; ... instrukce BCF f,b vynuluje bit b registru f

; ----- inicializace ukazatele tabulky
; Ukazatel v tabulce nastavme na hodnotu 0, co bude zatek tabulky.
; Souasn na toto msto budeme skkat v ppad dosaen konce tabulky.

Restart	clrf	Pointer	; nulovn ukazatele tabulky
			; ... instrukce CLRF f vynuluje registr f

; ----- inicializace obsluhy peruen
; Nebudeme pmo pouvat obsluhu peruen (proto pznak globlnho peruen
; GIE ponechme vypnut), ale budeme pouvat bit T0IF indikujc peteen
; asovae. Na toto msto budeme souasn provdt skok po kadm pchodu
; pznaku peruen, protoe tmto nastavenm pznak peruen nulujeme.

Next	movlw	bit5	; povoleno peruen od TMR0, globln peruen zakzno
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	0Bh	; nastaven registru peruen INTCON
			; ... instrukce MOVWF f ulo registr W do registru f
	
; ----- vstup na port A
; Z tabulky nateme hodnotu pro aktuln ukazatel tabulky. Ukazatel nateme
; do registru W a zatek tabulky volme jako podprogram. Obsluha tabulky
; provede skok na pslunou instrukci RETLW podle hodnoty registru W,
; ta provede nvrat za instrukci CALL s navrcenm sla v registru W.

	movf	Pointer,0 ; naten ukazatele tabulky do W
			; ... instrukce MOVF f,d nate obsah registru f;
			;     je-li parametr d=0, ulo obsah do registru W

	call	Table	; naten dat z tabulky
			; ... instrukce CALL a uschov do zsobnku adres
			;     adresu nsledujc instrukce a provede skok

	movwf	5	; vstup bajtu na port A (bity 4 a 7 se ignoruj)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- vstup na port B
; V registru W nm z pedchzejcho nastaven portu A zstal bajt z tabulky.
; Bajt si ulome do pomocnho registru Temp, ve kterm zamnme horn a
; doln tetrdu (tveici) bajtu a tm dostaneme bity pro port B na sprvnou pozici.

	movwf	Temp	; uloen bajt z W do pechodn promnn
			; ... instrukce MOVWF f ulo registr W do registru f
	
	swapf	Temp,0	; zmna horn a doln tetrdy bajtu a naten do W
			; ... instrukce SWAPF f,d zamn horn a doln tetrdu
			;     obsahu registru f; pro druh parametr d=0
			;     ulo vsledek do W (pro d=1 ulo vsledek do f)

	movwf	6	; vstup bajtu na port B (bity 4 a 7 se ignoruj)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- ekn na peteen asovae
; Abychom vytvoili potebnou asovou prodlevu, budeme testovat pznak
; peteen asovae a ekat, a bude nastaven. To bude trvat 65 ms.

Wait	btfss	0Bh,2	; je pznak peteen od asovae nastaven?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit nastaven, dal instrukci pesko

	goto	Wait	; ekn na peteen asovae

; ----- posun ukazatele tabulky
; Zvme ukazatel tabulky a otestujeme, zda doshl konce - tj. sla 8.
; Pi testovn peteen ukazatele tabulky vyuijeme toho, e slo 8
; m binrn tvar 1000b, take sta testovat nastaven bitu 3. Pokud
; by bylo poteba pout jinou dlku tabulky, provedl by se test konce
; tabulky natenm ukazatele tabulky do registru W, odetenm konstany
; pedstavujc dlku tabulky a nslednm testem bitu Z registru STATUS.

	incf	Pointer,1 ; zvme ukazatel
			; ... instrukce INCF f,d zv obsah registru o 1;
			;     je-li parametr d=1, ulo vsledek zpt do f
	btfss	Pointer,3 ; doshl ukazatel peteen, tj. sla 8?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit nastaven, dal instrukci pesko
	
	goto	Next	; nen jet peteen - me bt dal bajt tabulky

	goto	Restart	; resetovn ukazatele tabulky na zatek

; ----- start programu
; Vylete program do procesoru. Po okruhu matice indikanch LED se
; rozebhne kolem dokola svtlko se smrem proti smru hodinovch
; ruiek a s rychlost asi 2 otky za sekundu.
