;**************************************************************************
; LEKCE06.ASM					(c) Ing. Miroslav Nmeek
;
; 6. lekce programovn procesor PIC - tlatka
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo 9 LED diod pipojench k P84-CPU podle zobrazenho rozloen,
;	     3 tlatka A,B,C pipojen k B5 a B7, pi stisku signl "1"
;
; Rozloen pouit matice LED:
;		B1  B0  A3
;		B2  B4  A2
;		B3  A0  A1
;**************************************************************************
;
; ten vstupu z tlatek je zdnliv jednoduch operace. Nate se stav
; portu a zjist se, zda je tlatko sepnuto. Chceme-li provdt reakci
; na dren tlatko, je ve v podku. Pokud vak chceme reagovat na
; stisk tlatka, pedpokldme, e si uchovme minulou hodnotu a budeme
; zjiovat, zda byl stav tlatka zmnn. A zde nastv kmen razu.
; Kad mechanick tlatko pi svm sepnut toti vygeneruje zkmity,
; tedy opakovan sepnut. Dvodem je prunost a setrvanost mechanickch
; kontakt, kter po dosednut jet odpruj a odsko a vytvo tak
; nkolik krtkch ruivch impuls.
;
; Dobrou konstrukc spna lze odskakovn pi sepnut minimalizovat,
; nikoliv vak pln odstranit. U bnch elektronickch zazen se
; tento problm kompenzuje pomoc kondenztor a Schmittovch vstup.
; Krom toho, e se zazen prodrauje, nelze pout toto een
; u multiplexovanch matic tlatek. Lep situace je u tlatek
; s vodivou gumou, ale i zde me kolsnm pechodovho odporu bhem
; stisku dojt k vcensobnmu sepnut tlatka.
;
; U zazen opatenho mikroprocesorovou jednotkou pichz na adu
; programov oeten. Pi zjitn rozpojen tlatka zaneme mit
; dobu rozpojen. Pijde-li bhem jist krtk doby opt signl
; sepnut tlatka, je zejm, e se jedn pouze o zkmit, kter
; budeme ignorovat.
;--------------------------------------------------------------------------
; Vytvome si program, kter bude jednoduchm kalkultorem. Na displeji
; (matice 3x3 LED) budeme zobrazovat slo 0 a 9 zpsobem jako na hrac
; kostce. Tlatko A bude slouit k zadvn sla. Kad jeho stisk
; zv zadvan slo o 1. Tlatko B pite zadvan slo ke
; stadai a zobraz vsledek. Tlatko C kalkultor vymae.

; ----- zen pekladae, datov promnn

	RADIX	DEC	; pouijeme dekadickou selnou soustavu

	CBLOCK
RegX			; registr X pro zadvan slo
RegY			; stada vsledku operace
Flags			; pznaky
			;	bit 0: 1=probh zadvn registru X
			;	bit 1: 1=tlatko A bylo stisknuto
			;	bit 2: 1=tlatko B bylo stisknuto
			;	bit 3: 1=tlatko C bylo stisknuto
PressA			; ta uvolnn tlatka A
PressB			; ta uvolnn tlatka B
PressC			; ta uvolnn tlatka C
TempW			; schova registru W pi peruen
TempS			; schova registru STATUS pi peruen
	ENDC

; ----- start programu

	ORG	0	; procesor startuje od adresy 0
	goto	Start	; skok na zatek programu

	ORG	4	; zatek obsluhy peruen
	goto	Inter	; skok na obsluhu peruen

;--------------------------------------------------------------------------
;	Tabulky
;--------------------------------------------------------------------------

; Budeme pouvat k indikaci 2 tabulky - jednu pro port A, druhou pro port B
;
; Nvrh vzhledu matice (X=dioda svt):
; 0:-b1 -b0 -a3 1:-b1 -b0 -a3 2:-b1 -b0 -a3 3:-b1 -b0 Xa3 4:Xb1 -b0 Xa3
;   -b2 -b4 -a2   -b2 Xb4 -a2   Xb2 -b4 Xa2   -b2 Xb4 -a2   -b2 -b4 -a2
;   -b3 -a0 -a1   -b3 -a0 -a1   -b3 -a0 -a1   Xb3 -a0 -a1   Xb3 -a0 Xa1
;
; 5:Xb1 -b0 Xa3 6:Xb1 -b0 Xa3 7:Xb1 -b0 Xa3 8:Xb1 Xb0 Xa3 9:Xb1 Xb0 Xa3
;   -b2 Xb4 -a2   Xb2 -b4 Xa2   Xb2 Xb4 Xa2   Xb2 -b4 Xa2   Xb2 Xb4 Xa2
;   Xb3 -a0 Xa1   Xb3 -a0 Xa1   Xb3 -a0 Xa1   Xb3 Xa0 Xa1   Xb3 Xa0 Xa1
;
; ----- tabulka pro port A

TableA	addwf	2,1	; piten W k ukazateli programu PCL
			; ... instrukce ADDWF f,d sete registr W s obsahem
			;     registru f. Je-li druh parametr d=1, ulo
			;     vsledek do registru f. (Pro d=0 ulo do W)
	dt	0000b	; 0
	dt	0000b	; 1
	dt	0100b	; 2
	dt	1000b	; 3
	dt	1010b	; 4
	dt	1010b	; 5
	dt	1110b	; 6
	dt	1110b	; 7
	dt	1111b	; 8
	dt	1111b	; 9

; ----- tabulka pro port B

TableB	addwf	2,1	; piten W k ukazateli programu PCL
			; ... instrukce ADDWF f,d sete registr W s obsahem
			;     registru f. Je-li druh parametr d=1, ulo
			;     vsledek do registru f. (Pro d=0 ulo do W)
	dt	00000b	; 0
	dt	10000b	; 1
	dt	00100b	; 2
	dt	11000b	; 3
	dt	01010b	; 4
	dt	11010b	; 5
	dt	01110b	; 6
	dt	11110b	; 7
	dt	01111b	; 8
	dt	11111b	; 9

;--------------------------------------------------------------------------
;	Start programu
;--------------------------------------------------------------------------

; ----- nastaven smru bit port A a B
; Bity 0 a 3 portu A a bity 0 a 4 portu B nastavme jako vstupy

Start	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 1)
			; ... instrukce BSF f,b nastav bit b registru f

	movlw	10000b	; volba smru bit portu A: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A (registr TRISA)
			; ... instrukce MOVWF f ulo registr W do registru f


	movlw	11100000b ; volba smru bit portu B: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	6	; nastaven bit portu B (registr TRISB)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nastaven asovae Timer0
; asova Timer0 nechme cyklovat pes vech 256 hodnot a jeho peddliku
; nastavme na 1/4. Tm zskme periodu asi 1 ms.

	movlw	1+bit7	; dlika TMR0 1/4, pipojena k TMR0, zdrojem hodiny
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	1	; nastaven OPTION registru asova
			; ... instrukce MOVWF f ulo registr W do registru f

	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 0)
			; ... instrukce BCF f,b vynuluje bit b registru f


; ----- inicializace promnnch a pznak

	clrf	Flags	; nulovn pznak
	clrf	PressA	; nulovn tae uvolnn tlatka A
	clrf	PressB	; nulovn tae uvolnn tlatka B
	clrf	PressC	; nulovn tae uvolnn tlatka C
			; ... instrukce CLRF f vynuluje registr f
	call	Clear	; vynulovn datovch registr kalkultoru

; ----- inicializace obsluhy peruen
; Povolme peruen pi peteen asovae TMR0

	movlw	bit5+bit7 ; povoleno peruen od TMR0 a globln peruen
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	0Bh	; nastaven registru peruen INTCON
			; ... instrukce MOVWF f ulo registr W do registru f
	
; ----- obsluha tlatka A

Loop	btfss	Flags,1	; byl stisk tlatka A?
	goto	NoA	; nebyl stisk tlatka A

	bcf	Flags,1	; vypnut pznaku stisku tlatka A
	bsf	Flags,0	; pznak zahjen zadvn registru X

	incf	RegX,1	; inkrementace obsahu registru X

	movlw	10	; maximln hodnota registru X
	subwf	RegX,0	; kontrola peteen rozsahu registru X
	btfsc	3,0	; je peteen registru X (je pznak C)?
	movwf	RegX	; je peteen - uloen pesahu pes 10
	call	Disp	; zobrazen nov hodnoty registru X

; Popis: Instrukce SUBWF odete od testovanho registru registr W,
; kter obsahoval slo 10. Vsledek zstane v registru W. Vsledkem
; je pesah registru X pes slo 10. Pokud byl obsah registru X
; men ne 10, je nyn vsledkem zporn slo a nen nastaven
; pznak C v registru STATUS.  Byl-li obsah registru X = 10 nebo
; vce, nastav instrukce SUBWF pznak C. Dal instrukc BTFSC
; testujeme pznak C registru STATUS. Je-li nastaven (obsah registru
; X bylo slo 10 a vce), provede se nsledujc instrukce, tj.
; uloen nov hodnoty registru X, tedy pesah pes slo 10.

; ----- obsluha tlatka B

NoA	btfss	Flags,2	; byl stisk tlatka B?
	goto	NoB	; nebyl stisk tlatka B

	bcf	Flags,2	; vypnut pznaku stisku tlatka B
	bcf	Flags,0	; pznak ukonen zadvn registru X

	movf	RegX,0	; naten obsahu registru X do W
	addwf	RegY,1	; piten pvodnho registru X k registru Y
	clrf	RegX	; vynulovn registru X

	movlw	10	; maximln hodnota registru Y
	subwf	RegY,0	; kontrola peteen rozsahu registru Y
	btfsc	3,0	; je peteen registru Y (je pznak C)?
	movwf	RegY	; je peteen - uloen pesahu pes 10
	call	Disp	; zobrazen nov hodnoty registru Y

; ----- obsluha tlatka C

NoB	btfss	Flags,3	; byl stisk tlatka C?
	goto	Loop	; nebyl stisk tlatka C

	bcf	Flags,3	; vypnut pznaku stisku tlatka C
	call	Clear	; vynulovn datovch registr kalkultoru
	goto	Loop

;--------------------------------------------------------------------------
;	Vynulovn datovch registr kalkultoru
;--------------------------------------------------------------------------

Clear	bcf	Flags,0	; pznak ukonen zadvn registru X 
	clrf	RegX	; nulovn registru X
	clrf	RegY	; nulovn registru Y (stada vsledku)
	call	Disp	; zobrazen registr
	return

;--------------------------------------------------------------------------
;	Zobrazen registr
;--------------------------------------------------------------------------

Disp	btfss	Flags,0	; probh zadvn registru X?
	goto	DispY	; neprobh zadvn - bude registr Y

	movf	RegX,0	; naten registru X do W
	call	TableA	; dekdovn bit pro port A
	movwf	5	; zobrazen bit na port A
	movf	RegX,0	; naten registru X do W
	goto	DispXY	; zobrazen bit pro port B

DispY	movf	RegY,0	; naten registru Y do W
	call	TableA	; dekdovn bit pro port A
	movwf	5	; zobrazen bit na port A
	movf	RegY,0	; naten registru Y do W
DispXY	call	TableB	; dekdovn bit pro port B
	movwf	6	; zobrazen bit na port B
	return

;--------------------------------------------------------------------------
;	Obsluha peruen
;--------------------------------------------------------------------------

; ----- schova registr W a STATUS

Inter	movwf	TempW	; schova registru W
			; ... instrukce MOVWF f ulo registr W do registru f

	swapf	3,0	; naten STATUS registru do W
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	movwf	TempS	; schova STATUS registru
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nulovn pznaku peruen T0IF od tae TMR0

	bcf	0Bh,2	; nulovn pznaku peruen T0IF
			; ... instrukce BCF f,b vynuluje bit b registru f

; ----- je-li ta uvolnn tlatka A aktivn, zmna vstupu se ignoruje

	movf	PressA,1 ; test tae uvolnn tlatka A, zda je = 0
	btfss	3,2	; je ta uvolnn = 0 (test bitu Z registru STATUS)?
	goto	InterA3	; ta uvolnn nen = 0

; ----- pznak stisku tlatka A

	btfsc	6,5	; je stisknuto tlatko A?
	bsf	Flags,1	; je stisknuto - zapnut pznaku stisku tlatka A
	goto	InterA4	; inicializace tae uvolnn tlatka A

; ----- je-li tlatko A stisknuto, obnoven tae uvolnn

InterA3	decf	PressA,1 ; tn tae uvolnn tlatka A
InterA4	movlw	50	; nov hodnota tae (50 ms)
	btfsc	6,5	; je stisknuto tlatko A?
	movwf	PressA	; je stisknuto - nov inicializace tae uvolnn

; ----- je-li ta uvolnn tlatka B aktivn, zmna vstupu se ignoruje

	movf	PressB,1 ; test tae uvolnn tlatka B, zda je = 0
	btfss	3,2	; je ta uvolnn = 0 (test bitu Z registru STATUS)?
	goto	InterB3	; ta uvolnn nen = 0

; ----- pznak stisku tlatka B

	btfsc	6,6	; je stisknuto tlatko B?
	bsf	Flags,2	; je stisknuto - zapnut pznaku stisku tlatka B
	goto	InterB4	; inicializace tae uvolnn tlatka B

; ----- je-li tlatko B stisknuto, obnoven tae uvolnn

InterB3	decf	PressB,1 ; tn tae uvolnn tlatka B
InterB4	movlw	50	; nov hodnota tae (50 ms)
	btfsc	6,6	; je stisknuto tlatko B?
	movwf	PressB	; je stisknuto - nov inicializace tae uvolnn

; ----- je-li ta uvolnn tlatka C aktivn, zmna vstupu se ignoruje

	movf	PressC,1 ; test tae uvolnn tlatka C, zda je = 0
	btfss	3,2	; je ta uvolnn = 0 (test bitu Z registru STATUS)?
	goto	InterC3	; ta uvolnn nen = 0

; ----- pznak stisku tlatka C

	btfsc	6,7	; je stisknuto tlatko C?
	bsf	Flags,3	; je stisknuto - zapnut pznaku stisku tlatka C
	goto	InterC4	; inicializace tae uvolnn tlatka C

; ----- je-li tlatko C stisknuto, obnoven tae uvolnn

InterC3	decf	PressC,1 ; tn tae uvolnn tlatka C
InterC4	movlw	50	; nov hodnota tae (50 ms)
	btfsc	6,7	; je stisknuto tlatko C?
	movwf	PressC	; je stisknuto - nov inicializace tae uvolnn

; ----- nvrat registr W a STATUS

IntRet	swapf	TempS,0	; pvodn STATUS registr k navrcen
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	movwf	3	; nvrat pvodnho STATUS registru
			; ... instrukce MOVWF f ulo registr W do registru f

	swapf	TempW,1	; pedpprava pvodnho registru W
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	swapf	TempW,0	; nvrat pvodnho W registru
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	retfie		; nvrat z obsluhy peruen
			; ... instrukce RETFIE navrt procesor na msto,
			;     odkud bylo peruen vyvolno, souasn opt
			;     povol pznak globlnho peruen GIE

