;**************************************************************************
; ROBOT.ASM					(c) Ing. Miroslav Nmeek
;--------------------------------------------------------------------------
; Program ovld dic jednotkou pro rameno automatickho manipultoru.
; Manipultor je pipojen tmto zpsobem:
;
; zen posunu fze krokovch motor nbnou hranou signlu:
;   B0 ... nahoru (vlevo/dol/vped/sevt)
;   B1 ... dol (vpravo/nahoru/zpt/rozevt)
;
; Vbr krokovho motoru hodnotou 0 signlu:
;   B2 ... vped/zpt
;   B3 ... dol/nahoru
;   B4 ... vlevo/vpravo
;   B5 ... sevt/rozevt
;
; Program pro innost manipultoru je uloen v EEPROM procesoru. Z programu
; v EEPROM je mon volat peddefinovan podprogramy pro speciln kony,
; kter mohou bt pedpipraveny v programov pamti procesoru. Doplnnm
; obsluhy sriovho portu nebo doplnnm ovldn klvesnic s displejem
; je mon peprogramovvn innosti manipultoru pmo na mst pouit
; zmnou programu v EEPROM procesoru.
;**************************************************************************

	RADIX	DEC

PSTEPS	EQU	100	; poet krok na programovou jednotku

; ----- promnn

	CBLOCK
CStep1			; ta kroku 1 (vnj)
CStep2			; ta kroku 2 (vnitn)
CPause			; ta pauzy
CSteps			; ta krok pi pohybu
CMove			; ta programovch jednotek pohybu
Command			; kd povelu
DStep			; prodleva kroku (nsobek 770 us)
PointL			; ukazatel provdnho programu LOW
PointH			; ukazatel provdnho programu HIGH (pro EEPROM se nepouv)
RetPoint		; uschovan ukazatel LOW pro EEPROM
SubProg			; zpracovvan podprogram 1 a 16 (0=EEPROM)
	ENDC

; ----- start programu

	ORG	0		; adresa potku programu
	goto	Start

;**************************************************************************
;
;	Program innosti robota v EEPROM (me bt mnno procesorem)
;
;**************************************************************************

; Poloky v tabulce se skldaj ze soutu kdu povelu a sla 0 a 15,
; kter udv poet programovch jednotek pohybu (0 pedstavuje hodnotu 16)

STOP	EQU	0*16	; konec programu nebo podprogramu
LEFT	EQU	1*16	; vlevo
RIGHT	EQU	2*16	; vpravo
DOWN	EQU	3*16	; dol
UP	EQU	4*16	; nahoru
FRWRD	EQU	5*16	; vped
BACK	EQU	6*16	; zpt
CLOSE	EQU	7*16	; sevt
OPEN	EQU	8*16	; rozevt
LDOWN	EQU	9*16	; vlevo a dol
LUP	EQU	10*16	; vlevo a nahoru
RDOWN	EQU	11*16	; vpravo a dol
RUP	EQU	12*16	; vpravo a nahoru
SUB	EQU	13*16	; podprogram (lze volat pouze z EEPROM!)
SPEED	EQU	14*16	; nastaven rychlosti kroku (nsobek 770 us)
PAU	EQU	15*16	; pauza (nsobek doby 10 krok)

; ----- program k interpretaci v EEPROM (max. 64 B)
; pozor - povely zanaj pesudoinstrukc DE (na rozdl od tabulky v ROM)

	ORG	2100h	; adresa potku EEPROM

; zkuebn program - manipultor prohazuje 2 pedmty mezi 3 pozicemi
; rameno zan ped uchopenm prostednho pedmtu
	de	SUB+12	; uchopen pedmtu (uprosted)
	de	SUB+13	; krtk pesun vlevo
	de	SUB+11	; poloen pedmtu (vlevo)
	de	SUB+0	; dlouh pesun vpravo
	de	SUB+12	; uchopen pedmtu (vpravo)
	de	SUB+13	; krtk pesun vlevo
	de	SUB+11	; poloen pedmtu (uprosted)
	de	SUB+13	; krtk pesun vlevo
	de	SUB+12	; uchopen pedmtu (vlevo)
	de	SUB+0	; dlouh pesun vpravo
	de	SUB+11	; poloen pedmtu (vpravo)
	de	SUB+13	; krtk pesun vlevo

	de	STOP	; oznaen konce tabulky

;**************************************************************************
;
;	Tabulky v ROM
;
;**************************************************************************

	ORG	1		; adresa za instrukc GOTO START

; ----- skoky na obsluhu pohyb (W=slo obsluhy 0 a 15)
; Tabulka mus zstat v prvnch 256 slovech programu!

Pohyb	clrf	PCLATH
	addwf	PCL,F
	goto	SubRet		; 0: konec programu/podprogramu
	goto	Left0		; 1: vlevo
	goto	Right0		; 2: vpravo
	goto	Down0		; 3: dol
	goto	Up0		; 4: nahoru
	goto	Frwrd0		; 5: vped
	goto	Back0		; 6: zpt
	goto	Close0		; 7: sevt
	goto	Open0		; 8: rozevt
	goto	LDown0		; 9: vlevo a dol
	goto	LUp0		; 10: vlevo a nahoru
	goto	RDown0		; 11: vpravo a dol
	goto	RUp0		; 12: vpravo a nahoru
	goto	SubCall		; 13: podprogram
	goto	SetStep		; 14: nastaven rychlosti kroku
	goto	Pause		; 15: pauza 10 krok

; ----- adresy podprogram
; Tabulky mus zstat v prvnch 256 slovech programu!

GetPrgL	clrf	PCLATH
	addwf	PCL,F
	dt	0
	dt	LOW SubPrg1
	dt	LOW SubPrg2
	dt	LOW SubPrg3
	dt	LOW SubPrg4
	dt	LOW SubPrg5
	dt	LOW SubPrg6
	dt	LOW SubPrg7
	dt	LOW SubPrg8
	dt	LOW SubPrg9
	dt	LOW SubPrg10
	dt	LOW SubPrg11
	dt	LOW SubPrg12
	dt	LOW SubPrg13
	dt	LOW SubPrg14
	dt	LOW SubPrg15
	dt	LOW SubPrg16

GetPrgH	clrf	PCLATH
	addwf	PCL,F
	dt	0
	dt	HIGH SubPrg1
	dt	HIGH SubPrg2
	dt	HIGH SubPrg3
	dt	HIGH SubPrg4
	dt	HIGH SubPrg5
	dt	HIGH SubPrg6
	dt	HIGH SubPrg7
	dt	HIGH SubPrg8
	dt	HIGH SubPrg9
	dt	HIGH SubPrg10
	dt	HIGH SubPrg11
	dt	HIGH SubPrg12
	dt	HIGH SubPrg13
	dt	HIGH SubPrg14
	dt	HIGH SubPrg15
	dt	HIGH SubPrg16

; ----- podprogramy
; pozor - povely zanaj pesudoinstrukc DT (na rozdl od tabulky v EEPROM)

SubPrg1				; rozeven elist
	dt	SPEED+4
	dt	OPEN+7
	dt	STOP

SubPrg2				; zaven elist
	dt	SPEED+4
	dt	CLOSE+7
	dt	STOP

SubPrg3				; dol
	dt	SPEED+4
	dt	DOWN+3
	dt	STOP

SubPrg4				; nahoru
	dt	SPEED+4
	dt	UP+3
	dt	STOP

SubPrg5				; vlevo
	dt	SPEED+4
	dt	LEFT+3
	dt	STOP

SubPrg6				; vpravo
	dt	SPEED+4
	dt	RIGHT+3
	dt	STOP

SubPrg7				; vlevo a dol
	dt	SPEED+4
	dt	LDOWN+3
	dt	STOP

SubPrg8				; vpravo a nahoru
	dt	SPEED+4
	dt	RUP+3
	dt	STOP

SubPrg9				; vlevo a nahoru (nepouvat - motory nejedou souasn)
	dt	SPEED+4
	dt	LUP+3
	dt	STOP

SubPrg10			; vpravo a dol (nepouvat - motory nejedou souasn)
	dt	SPEED+4
	dt	RDOWN+3
	dt	STOP

SubPrg11			; poloen pedmtu
	dt	SPEED+4
	dt	PAU+10
	dt	OPEN+6
	dt	PAU+10
	dt	STOP

SubPrg12			; uchopen pedmtu
	dt	SPEED+4
	dt	PAU+10
	dt	CLOSE+6
	dt	PAU+10
	dt	STOP

SubPrg13			; krtk pesun vlevo
	dt	SPEED+4
	dt	UP+10
	dt	LEFT+5
	dt	DOWN+10
	dt	STOP

SubPrg14			; krtk pesun vpravo
	dt	SPEED+4
	dt	UP+10
	dt	RIGHT+5
	dt	DOWN+10
	dt	STOP

SubPrg15			; dlouh pesun vlevo
	dt	SPEED+4
	dt	UP+10
	dt	LEFT+10
	dt	DOWN+10
	dt	STOP

SubPrg16			; dlouh pesun vpravo (zadvat SUB+0 !)
	dt	SPEED+4
	dt	UP+10
	dt	RIGHT+10
	dt	DOWN+10
	dt	STOP

;--------------------------------------------------------------------------
;	naten bajtu programu
;--------------------------------------------------------------------------

; ----- test, zda b program z EEPROM

Program	movf	SubProg,F	; probh zpracovn programu v EEPROM?
	btfss	STATUS,Z	; byla hodnota 0?
	goto	Progrm2		; nen hodnota 0 - nen EEPROM

; ----- naten bajtu z EEPROM
	
	movf	PointL,W	; naten ukazatele LOW
	incf	PointL,F	; zven ukazatele adresy
	bcf	STATUS,RP0	; nastaven banky 0
	movwf	EEADR		; nastaven adresy ke ten z EEPROM
	bsf	STATUS,RP0	; nastaven banky 1
	bsf	EECON1,RD	; zahjen operace ten z EEPROM
	bcf	STATUS,RP0	; nastaven banky 0
	movf	EEDATA,W	; naten bajtu z EEPROM
	return

; ----- naten bajtu z ROM (z aktivnho podprogramu)

Progrm2	movf	PointH,W	; naten ukazatele HIGH
	movwf	PCLATH		; nastaven adresy skoku HIGH
	movf	PointL,W	; naten ukazatele LOW
	incf	PointL,F	; zven ukazatele LOW
	btfsc	STATUS,Z	; je peteen pes 0?
	incf	PointH,F	; zven ukazatele HIGH
	movwf	PCL		; skok na naten bajtu

;--------------------------------------------------------------------------
;	voln podprogramu (pouze z EEPROM!)
;--------------------------------------------------------------------------

SubCall	movf	CMove,W		; slo podprogramu
	movwf	SubProg		; schova sla volanho podprogramu

	movf	PointL,W	; naten aktulnho ukazatele v EEPROM
	movwf	RetPoint	; schova ukazatele v EEPROM

	movf	SubProg,W	; naten sla podprogramu
	call	GetPrgL		; naten adresy podprogramu LOW
	movwf	PointL		; nulovn ukazatele programu LOW

	movf	SubProg,W	; naten sla podprogramu
	call	GetPrgH		; naten adresy podprogramu HIGH
	movwf	PointH		; nulovn ukazatele programu HIGH

RetMove	movlw	1		; hodnota pro ta (aby nebyl voln vcekrt)
	movwf	CMove		; nulovn tae
	return

;--------------------------------------------------------------------------
;	nvrat podprogramu
;--------------------------------------------------------------------------

SubRet	movf	RetPoint,W	; uschovan ukazatel programu
	movf	SubProg,F	; probh zpracovn programu v EEPROM?
	btfsc	STATUS,Z	; byla hodnota 0?
	movlw	0		; resetovn EEPROM od zatku
	movwf	PointL		; nvrat ukazatele programu
	movlw	0		; bude probhat zpracovn EEPROM
	movwf	SubProg		; nulovn sla podprogramu na EEPROM
	goto	RetMove

;--------------------------------------------------------------------------
;	nastaven rychlosti kroku
;--------------------------------------------------------------------------

SetStep	movf	CMove,W		; konstanta k nastaven
	movwf	DStep		; schova nov konstanty pro krok
	goto	RetMove

;**************************************************************************
;
;	Start a interpretace programu
;
;**************************************************************************

; ----- inicializace portu B

Start	bsf	STATUS,5	; volba banky 1
	movlw	11000000b	; bity 0 a 5 jako vstup
	movwf	TRISB		; nastaven bit portu B

	bcf	STATUS,5	; volba banky 0
	movlw	3Ch		; inicializan slovo
	movwf	PORTB		; nastaven vstup portu B

; ----- inicializace promnnch

	movlw	13		; implicitn prodleva 10 ms
	movwf	DStep		; prodleva na 1 krok
	movlw	0		; W <- 0
	movwf	PointL		; ukazatel programu pro EEPROM
	movwf	SubProg		; zpracovvan program = EEPROM

; ----- naten dalho kroku pohybu

Dalsi	call	Program		; naten dalho kroku programu
	movwf	Command		; schova kdu povelu

; ----- pprava tae krok jednotek pohybu

	andlw	0fh		; maskovn potu opakovn
	btfsc	STATUS,Z	; je poet = 0 ?
	movlw	16		; nhrada 0 potem 16
	movwf	CMove		; ta krok pohybu

; ----- pprava kdu povelu

	rrf	Command,F
	rrf	Command,F
	rrf	Command,F
	rrf	Command,F	; rotace kdu povelu na pozici
	movlw	0Fh
	andwf	Command,F	; maskovn povelu (kd 0 a 15)

; ----- proveden povelu pro pohyb

Proved	movf	Command,W	; naten kdu povelu
	call	Pohyb		; proveden jednoho programovho kroku
	decfsz	CMove,F		; ta pohybu
	goto	Proved		; dal programov krok
	goto	Dalsi		; dal povel

;**************************************************************************
;
;	Proveden vce krok najednou (vstup: W=poet krok 1 a 255, 0=256)
;
;**************************************************************************

;--------------------------------------------------------------------------
;	vce krok vlevo (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Left0	movlw	PSTEPS		; poet krok na programovou jednotku

Left	movwf	CSteps
Left2	call	Left1
	call	Step
	decfsz	CSteps,F
	goto	Left2
	return

;--------------------------------------------------------------------------
;	vce krok vpravo (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Right0	movlw	PSTEPS		; poet krok na programovou jednotku

Right	movwf	CSteps
Right2	call	Right1
	call	Step
	decfsz	CSteps,F
	goto	Right2
	return

;--------------------------------------------------------------------------
;	vce krok dol (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Down0	movlw	PSTEPS		; poet krok na programovou jednotku

Down	movwf	CSteps
Down2	call	Down1
	call	Step
	decfsz	CSteps,F
	goto	Down2
	return

;--------------------------------------------------------------------------
;	vce krok nahoru (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Up0	movlw	PSTEPS		; poet krok na programovou jednotku

Up	movwf	CSteps
Up2	call	Up1
	call	Step
	decfsz	CSteps,F
	goto	Up2
	return

;--------------------------------------------------------------------------
;	vce krok vped (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Frwrd0	movlw	PSTEPS		; poet krok na programovou jednotku

Frwrd	movwf	CSteps
Frwrd2	call	Frwrd1
	call	Step
	decfsz	CSteps,F
	goto	Frwrd2
	return

;--------------------------------------------------------------------------
;	vce krok zpt (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Back0	movlw	PSTEPS		; poet krok na programovou jednotku

Back	movwf	CSteps
Back2	call	Back1
	call	Step
	decfsz	CSteps,F
	goto	Back2
	return

;--------------------------------------------------------------------------
;	vce krok klet zavt (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Close0	movlw	PSTEPS		; poet krok na programovou jednotku

Close	movwf	CSteps
Close2	call	Close1
	call	Step
	decfsz	CSteps,F
	goto	Close2
	return

;--------------------------------------------------------------------------
;	vce krok klet otevt (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

Open0	movlw	PSTEPS		; poet krok na programovou jednotku

Open	movwf	CSteps
Open2	call	Open1
	call	Step
	decfsz	CSteps,F
	goto	Open2
	return

;--------------------------------------------------------------------------
;	vce krok vlevo a dol (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

LDown0	movlw	PSTEPS		; poet krok na programovou jednotku

LDown	movwf	CSteps
LDown2	call	LDown1
	call	Step
	decfsz	CSteps,F
	goto	LDown2
	return

;--------------------------------------------------------------------------
;	vce krok vlevo a nahoru (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

LUp0	movlw	PSTEPS		; poet krok na programovou jednotku

LUp	movwf	CSteps
LUp2	call	Left1
	call	Step
	call	Up1
	call	Step
	decfsz	CSteps,F
	goto	LUp2
	return

;--------------------------------------------------------------------------
;	vce krok vpravo a dol (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

RDown0	movlw	PSTEPS		; poet krok na programovou jednotku

RDown	movwf	CSteps
RDown2	call	Right1
	call	Step
	call	Down1
	call	Step
	decfsz	CSteps,F
	goto	RDown2
	return

;--------------------------------------------------------------------------
;	vce krok vpravo a nahoru (W=poet krok 1 a 255, 0 je 256)
;--------------------------------------------------------------------------

RUp0	movlw	PSTEPS		; poet krok na programovou jednotku

RUp	movwf	CSteps
RUp2	call	RUp1
	call	Step
	decfsz	CSteps,F
	goto	RUp2
	return

;**************************************************************************
;
;	Obsluha proveden jednoho kroku (po obsluze neek)
;
;**************************************************************************

;--------------------------------------------------------------------------
;	jeden krok vlevo
;--------------------------------------------------------------------------

Left1	movlw	2ch
	movwf	PORTB
	bsf	PORTB,0
	return

;--------------------------------------------------------------------------
;	jeden krok vpravo
;--------------------------------------------------------------------------

Right1	movlw	2ch
	movwf	PORTB
	bsf	PORTB,1
	return

;--------------------------------------------------------------------------
;	jeden krok dol
;--------------------------------------------------------------------------

Down1	movlw	34h
	movwf	PORTB
	bsf	PORTB,0
	return

;--------------------------------------------------------------------------
;	jeden krok nahoru
;--------------------------------------------------------------------------

Up1	movlw	34h
	movwf	PORTB
	bsf	PORTB,1
	return

;--------------------------------------------------------------------------
;	jeden krok vped
;--------------------------------------------------------------------------

Frwrd1	movlw	38h
	movwf	PORTB
	bsf	PORTB,0
	return

;--------------------------------------------------------------------------
;	jeden krok zpt
;--------------------------------------------------------------------------

Back1	movlw	38h
	movwf	PORTB
	bsf	PORTB,1
	return

;--------------------------------------------------------------------------
;	jeden krok klet sevt
;--------------------------------------------------------------------------

Close1	movlw	1ch
	movwf	PORTB
	bsf	PORTB,0
	return

;--------------------------------------------------------------------------
;	jeden krok klet otevt
;--------------------------------------------------------------------------

Open1	movlw	1ch
	movwf	PORTB
	bsf	PORTB,1
	return

;--------------------------------------------------------------------------
;	jeden krok vlevo dol
;--------------------------------------------------------------------------

LDown1	movlw	24h
	movwf	PORTB
	bsf	PORTB,0
	return

;--------------------------------------------------------------------------
;	jeden krok vpravo nahoru
;--------------------------------------------------------------------------

RUp1	movlw	24h
	movwf	PORTB
	bsf	PORTB,1
	return

;**************************************************************************
;
;	Obsluha asovn
;
;**************************************************************************

;--------------------------------------------------------------------------
;	dlouh prodleva (10 krok)
;--------------------------------------------------------------------------

Pause	movlw	10	; asov konstanta pro dlouhou prodlevu
	movwf	CPause	; pprava tae dlouh prodlevy

Pause2	call	Step	; krtk prodleva 10 ms
	decfsz	CPause,F ; tn tae dlouh prodlevy
	goto	Pause2	; ekn na dlouhou prodlevu
	return

;--------------------------------------------------------------------------
;	ekn po dobu 1 kroku motor (pro DStep=13 je doba 10 ms, tj. 100 Hz)
;--------------------------------------------------------------------------

Step	movf	DStep,W	; asov konstanta kroku motor
	movwf	CStep1	; pprava tae pro krok motor

; ----- vnitn asov smyka
;  1 prchod cyklem trv 1+2 = 3 takty, tj. 3 us
;  256 prchod trv 3*255 + 2 = 767 takt, tj. 767 us

Step2	decfsz	CStep2,F ; tn tae 2 (vnitnho)
	goto	Step2	; ekn po dobu 767 us

; ----- vnj asov smyka
;  1 prchod trv 767 + 3 = 770 takt, tj. 770 us
;  13 prchod trv asi 770*13 = 10010 takt, tj. asi 10 ms

	decfsz	CStep1,F ; tn tae 1 (vnjho)
	goto	Step2	; ekn po dobu 1 kroku
	return
