;**************************************************************************
; TRANS.ASM					(c) Ing. Miroslav Nmeek
;
; Program slou k testu sriov komunikace s PC. Program pijm od PC
; znaky, zv je o 1 a odele je opt zpt. Ke komunikaci se vyuv
; programovac sriov port jednotky P84-CPU. Na PC se spust program
; TERMINAL, zvol se pslun COM, zapne vstupn signl DSR a vypne
; signl DTR. Zvol se penosov rychlost 115200 Baud, 8 bit bez parity,
; 1 STOP bit, ECHO vysln se vypne. Pi sprvn funkci jsou znaky
; psanho textu zobrazovny o 1 vt (A->B, 2->3).
;
; Komunikace pomoc programovac ppojky jednotky probh na principu
; poloduplexnho provozu, tj. na shodnm datovm kanlu s pepnnm
; pijmn a vysln. V bnm provozu se penos dje po blocch pomoc
; paket, take je sprvn rozdlena doba vysln i pjmu.
;--------------------------------------------------------------------------
; POZOR - je-li sputn program TERMINAL, nen jednotka P84-CPU pes
;         port COM pstupn (nelze ji programovat)!
;**************************************************************************

	RADIX	DEC

	CBLOCK
Byte				; pijat/vyslan bajt
Count				; ta dvojic datovch bit pi vysln/pjmu
Maska				; maska pijmanho datovho bitu
	ENDC

	ORG	0

Start	call	Receive		; pjem jednoho znaku

Start2	decfsz	Count,F		; prodleva pro ukonen vysln
	goto	Start2

	addlw	1		; zven hodnoty znaku
	call	Send		; odesln znaku

Start3	decfsz	Count,F		; prodleva pro ukonen vysln
	goto	Start3

	goto	Start		; dal znak

;--------------------------------------------------------------------------
; Pjem bajtu z RB7 (rychlost 115200 Baud, bez parity, 8 bit, 1 STOP) -> W
;--------------------------------------------------------------------------

; ----- inicializace portu B7 pro pjem

Receive	bsf	Status,5	; zapnut banky 1
	bsf	TrisB,7		; bit RB7 nastaven jako vstup

; ----- inicializace pjmu bajtu dat

	movlw	0		; W <- 0
	movwf	Byte		; vynulovn pijmanho bajtu
	movlw	4		; poet dvojic datovch bit k pjmu
	movwf	Count		; ta k pjmu datovch bit
	movlw	1		; maska pijmanho bitu
	movwf	Maska		; pprava registru masky

; ----- ekn na START bit (m rove 1)

	bcf	Status,5	; zapnut banky 0
RecStrt	btfss	PortB,7		; je ji START bit?
	goto	RecStrt		; ekn na START bit

; Pro penosovou rychlost 115200 Baud je doba jednoho
; bitu 8,68 us. Dostatenou pesnost doshneme, budeme-li
; ekat 8,5 us, tj. budeme stdat 8 a 9 us a pijmat
; po dvojicch bit.

; ----- doplkov ekn po START bitu do 9 us
; Potme-li as od zatku instrukce BTFSS po zatek
; dal instrukce BTFSS, je to 2 us + ? + 2 us + 1 us,
; do 9 us nm tedy chyb 4 us, pro sporu msta pouijeme
; instrukce GOTO, kter zaberou dobu 2 us

	goto	$+1
	goto	$+1

; ----- doplkov ekn na sted bitu
; Pedelmi instrukcemi bychom doshli asovn zatku
; datovho bitu. Abychom doshli dostatenho rozptylu
; tolerance, naasujeme se na sted datovch bit. Proto
; nyn doplnme jet ekn po dobu 1/2 datovho bitu, tj. 4 us

	goto	$+1
	goto	$+1

; ----- doplkov ekn do 9 us (pedchzelo DECFSZ a GOTO)
; zde ztrcme 1 us

RecBit	nop

; ----- pjem 1 bitu (bit=0 m hodnotu 1 a naopak)
; zde ztrcme 3 us

	movf	Maska,W		; naten masky do W
	btfss	PortB,7		; je bit 0?
	iorwf	Byte,F		; pro hodnotu=0 nastavme hodnotu bitu=1
	
; ----- pprava pro dal datov bit
; zde ztrcme 2 us

	bcf	Status,0	; nulovn pznaku C
	rlf	Maska,F		; rotovn masky o 1 bit vlevo

; ----- doplkov ekn do 8 us
; zde ztrcme 3 us

	goto	$+1
	nop

; ----- pjem 1 bitu (bit=0 m hodnotu 1 a naopak)
; zde ztrcme 3 us

	movf	Maska,W		; naten masky do W
	btfss	PortB,7		; je bit 0?
	iorwf	Byte,F		; pro hodnotu=0 nastavme hodnotu bitu=1
	
; ----- pprava pro dal datov bit
; zde ztrcme pi opakovn 5 us

	bcf	Status,0	; nulovn pznaku C
	rlf	Maska,F		; rotovn masky o 1 bit vlevo
	decfsz	Count,F		; tn dvojic datovch bit
	goto	RecBit		; skok na pjem dalho bitu

; ----- pjem je ukonen, pijat bajt navrtme ve W

	movf	Byte,W		; naten pijatho bajtu do W
	return

;--------------------------------------------------------------------------
; Vysln bajtu na RB7 (rychlost 115200 Baud, bez parity, 8 bit, 1 STOP) <- W
;--------------------------------------------------------------------------

; ----- inicializace portu B7 pro vysln

Send	bsf	Status,5	; zapnut banky 1
	bcf	TrisB,7		; bit RB7 nastaven jako vstup

; ----- inicializace vysln bajtu dat

	movwf	Byte		; uloen vyslanho bajtu
	movlw	4		; poet dvojic datovch bit k vysln
	movwf	Count		; ta k vysln datovch bit

; ----- vysln START bitu (m rove 1)

	bcf	Status,5	; zapnut banky 0
	bsf	PortB,7		; zapnut START bitu

; Pro penosovou rychlost 115200 Baud je doba jednoho
; bitu 8,68 us. Dostatenou pesnost doshneme, budeme-li
; ekat 8,5 us, tj. budeme stdat 8 a 9 us a pijmat
; po dvojicch bit.

; ----- doplkov ekn po START bitu do 9 us
; Potme-li as od zatku instrukce BSF po zatek
; dal instrukce MOVWF, je to 1 us + ? + 4 us,
; do 9 us nm tedy chyb 4 us, pro sporu msta pouijeme
; instrukce GOTO, kter zaberou dobu 2 us

	goto	$+1
	goto	$+1

; ----- doplkov ekn do 9 us (pedchzelo DECFSZ a GOTO)
; zde ztrcme 0 us - prodleva je ji zajitna

SendBit

; ----- vysln 1 bitu (bit=0 m hodnotu 1 a naopak)
; zde ztrcme 5 us

	movf	PortB,W		; naten hodnoty portu B
	andlw	7fh		; vynulovn bitu 7
	btfss	Byte,0		; m bt bit 0?
	iorlw	80h		; nastaven bitu 7
	movwf	PortB		; odesln hodnoty bitu

; ----- pprava pro dal datov bit
; zde ztrcme 1 us

	rrf	Byte,F		; rotovn datovho bajtu o 1 bit vpravo

; ----- doplkov ekn do 8 us
; zde ztrcme 2 us

	goto	$+1

; ----- vysln 1 bitu (bit=0 m hodnotu 1 a naopak)
; zde ztrcme 5 us

	movf	PortB,W		; naten hodnoty portu B
	andlw	7fh		; vynulovn bitu 7
	btfss	Byte,0		; m bt bit 0?
	iorlw	80h		; nastaven bitu 7
	movwf	PortB		; odesln hodnoty bitu
	
; ----- pprava pro dal datov bit
; zde ztrcme pi opakovn 4 us, jinak 3 us

	rrf	Byte,F		; rotovn datovho bajtu o 1 bit vpravo
	decfsz	Count,F		; tn dvojic datovch bit
	goto	SendBit		; skok na vysln dalho bitu

; ----- te dokonme asovn pro posledn datov bit
; od poslednho vstupu dat uplynuly 4 us, doplnme na 9 us

	goto	$+1
	goto	$+1
	nop

; ----- zapnut STOP bitu

	bcf	PortB,7		; zapnut STOP bitu

; ----- ekn po dobu STOP bitu (pro oddlen vyslanch bajt mezi sebou)
; Musme zajistit dobu alespo 8 us. Spoteme-li dobu pi nejrychlejm
; opakovanm voln od konce STOP bitu po zatek ptho START bitu,
; vyjde nm 1 us (BCF) + 2 us (RETURN) + 1 us (asi MOVLW) + 2 us (CALL) + 6 us (vod)
; tj. 12 us, co je doba a dostaten dlouh

	return
