;**************************************************************************
; CYCLOMET.ASM					(c) Ing. Miroslav Nmeek
;--------------------------------------------------------------------------
; Zazen slou k men informac sportovnho zazen nebo byciklu.
; Men probh pomoc snmacho idla (spnac kontakt), spnajc
; po urit vzdlenosti drhy (vzdlenost uruje v programu konstanta LENINC).
;
; Zobrazuj se tyto daje: ubhl as, urazen vzdlenost, aktuln rychlost,
; celkov urazen vzdlenost. Md zobrazen se pepn stiskem tlatka.
;
; Ped zobrazenm daje se vype text nzvu daje:
;	"tIM" ... as (time) ve formtu MM.SS nebo HH.MM - m pi nenulov rychlosti
;	"diS" ... vzdlenost (distance) ve formtu kk.kk nebo kkk.k (v kilometrech)
;	"SPE" ... rychlost (speed) ve formtu sss.s (v kilometrech za hodinu)
;	"odo" ... celkov vzdlenost (odometer) ve formtu kkkk (v kilometrech)
;	"Aut" ... automatick zmna zobrazen (po 4 sekundch)
;
; Podrenm tlatka po dobu 2 sekund se ukazatele vynuluj (krom celkov
; vzdlenosti). Celkovou vzdlenost lze nulovat odpojenm napjecho zdroje.
; Po 30 sekundov neaktivit se mi vypne. Tlatkem nebo senzorem lze
; opt aktivovat.
;
; Mi lze napjet 2 tukovmi monolnky. Pro minimalizaci spoteby proudu
; je pouit krystal 32 kHz. Pekreslovn displeje probh proto pomaleji,
; ne je bn (s kmitotem 16 Hz), displej me pi bonm pohledu blikat.
; Paraleln k senzoru (na vstupu ze senzoru) je pipojen kondenztor o kapacit
; piblin 22 nF, kter zajiuje prodlouen pli krtkch impuls z idla
; a vyhlazen zakmitvn kontakt idla. Kapacita kondenztoru zvis na
; pouitm senzoru a na jeho spnac dob. Pli velk kapacita kondenztoru
; se projev vpadky men pi velkch rychlostech (poitadlo se zastavuje).
; Pli nzk kapacita se projev nepravidelnost daje a nepravidelnm nrustem
; rychlosti pi nzkch rychlostech.
;**************************************************************************

	RADIX	DEC
	CONFIG	LP

; Oznaen segment displeje:
;
;		 -- A3 -	 -- A2 -	 -- A1 -
;	|	|	|	|	|	|	|
;	|	F3	B3	F2	B2	F1	B1
;	|	|	|	|	|	|	|
;	H3	 -- G3 -	 -- G2 -	 -- G1 -
;	|	|	|	|	|	|	|
;	|	E3	C3	E2	C2	E1	C1
;	|	|	|	|	|	|	|
;    		 -- D3 -   H2 	 -- D2 -    H1	 -- D1 -
;
; Bity port:
;	A0 ... C3, F2/REG1, H2/REG2
;	A1 ... A3, G3/REG1, H3/REG2
;	A2 ... zpis do registru REG1 (hodnotou "1")
;	A3 ... zpis do registru REG2 (hodnotou "1")
;	A4 ... spolen elektroda ("1" = inverze)
;
;	B0 ... F1, A1/REG1, D1/REG2
;	B1 ... G1, A2/REG1, C2/REG2
;	B2 ... H1, B2/REG1, D2/REG2
;	B3 ... E1, B1/REG1, C1/REG2
;	B4 ... vstup z tlatka ("1" = stisknuto)
;	B5 ... vstup ze senzoru ("1" = sepnuto)
;	B6 ... B3, G2/REG1, E2/REG2
;	B7 ... D3, F3/REG1, E3/REG2

; ----- konstanty

; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
LENINC	EQU	20	; prustek vzdlenosti na impuls (cm)
			; - tato konstanta zvis na zazen
; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

QUITMAX	EQU	30	; ta pro uklidnn (sekund)
VELTIME	EQU	115	; ta pro men rychlosti (=3.6 sekundy)
CSTATE	EQU	4*32	; ta automatickho pepnn (=4 sekundy)
CIND	EQU	24	; ta doby indikace zmny stavu (zlomky sekund)

;--------------------------------------------------------------------------
;	promnn
;--------------------------------------------------------------------------

	CBLOCK

; ----- vstupn registry

Reg0			; vstupn registr 0 (bez mezizpisu)
			;	bit0 ... F1
			;	bit1 ... G1
			;	bit2 ... H1
			;	bit3 ... E1
			;	bit4 ... C3
			;	bit5 ... A3
			;	bit6 ... B3
			;	bit7 ... D3

Reg1			; vstupn registr 1
			;	bit0 ... A1
			;	bit1 ... A2
			;	bit2 ... B2
			;	bit3 ... B1
			;	bit4 ... F2
			;	bit5 ... G3
			;	bit6 ... G2
			;	bit7 ... F3

Reg2			; vstupn registr 2
			;	bit0 ... D1
			;	bit1 ... C2
			;	bit2 ... D2
			;	bit3 ... C1
			;	bit4 ... H2
			;	bit5 ... H3
			;	bit6 ... E2
			;	bit7 ... E3

DByte			; bajt k dekdovn nebo k odesln na displej

TByte			; pomocn stada k zobrazen
TByte2			; pomocn stada k zobrazen 2

; ----- ovldn

Flags			; pznaky
			;	bit 0: 1=inverze vstupu
			;	bit 1: 1=tlatko stisknuto
			;	bit 2: 1=senzor sepnut
			;	bit 3: 1=dlouh stisknut tlatka
			;	bit 4: 1=automatick zmna stavu
			;	bit 5: 1=as b
			;	bit 6: 1=zobrazena indikace stavu
			;	bit 7: 1=zobrazena indikace Aut

RelKey			; ta uvolnn tlatka
LongKey			; ta dlouhho stisknut tlatka

State			; aktuln stav (0 a 3) (viz tabulka stav ne)
CState			; ta pro automatickou zmnu stavu
CInd			; ta indikace novho stavu

Quit			; ta pro klid (sekund)

HlasN			; uschovan slo hlen (pi zobrazen hlen)

; ----- ubhl as

Hundr			; 1/32 sekundy
Second			; ubhl sekundy
Minute			; ubhl minuty
Hour			; ubhl hodiny

; ----- urazen vzdlenost

LenCM			; urazen vzdlenost (centimetry, 0 a 99)
LenM			; urazen vzdlenost (metry, 0 a 9)
LenDM			; urazen vzdlenost (dekametry, 0 a 9)
LenHM			; urazen vzdlenost (hektometry, 0 a 9)
LenKM			; urazen vzdlenost (kilometry, 0 a 199)

; ----- celkov urazen vzdlenost

SLenDM			; celkov urazen vzdlenost (dekametry, 0 a 99)
SLenKM			; celkov urazen vzdlenost (kilometry, 0 a 9)
SLenHM			; celkov urazen vzdlenost (destky kilometr, 0 a 199)

; ----- rychlost (tn metr po 3.6 sekundy odpovd rychlosti v km/hod)

VelTim			; ta mc doby pro rychlost
CVelCM			; ta rychlosti v cm (0 a 99)
CVelM			; ta rychlosti v m (0 a 199)

VelHM			; rychlost v hektometrech (0 a 9)
VelKM			; rychlost v kilometrech (0 a 199)

; ----- schova registr pi peruen

TempW			; schova registru W pi peruen
TempS			; schova registru STATUS pi peruen
	ENDC

; ----- symboly stav

STIME	EQU	0	; zobrazen ubhlho asu
SLENGTH	EQU	1	; zobrazen urazen vzdlenosti
SSPEED	EQU	2	; zobrazen rychlosti
SSLEN	EQU	3	; zobrazen celkov urazen vzdlenosti

SAUT	EQU	4	; automatick zobrazen (konstanta urena jen pro obsluhu indikace)

;--------------------------------------------------------------------------
;	vektory
;--------------------------------------------------------------------------

	ORG	0
	goto	Start		; skok na start programu

	ORG	4
	goto	Inter		; skok na obsluhu peruen

;--------------------------------------------------------------------------
;	tabulky
;--------------------------------------------------------------------------

; ----- tabulka slic
; Bitov mapa segment slice:
;
;	 -- 0 --
;	|	|
;	5	1
;	|	|
;	 -- 6 --
;	|	|
;	4	2
;	|	|
;	 -- 3 --

ChrTab	addwf	PCL,F
	dt	00111111b	; 0
	dt	00000110b	; 1
	dt	01011011b	; 2
	dt	01001111b	; 3
	dt	01100110b	; 4
	dt	01101101b	; 5
	dt	01111101b	; 6
	dt	00000111b	; 7
	dt	01111111b	; 8
	dt	01101111b	; 9
	dt	01110111b	; 10: A
	dt	00011100b	; 11: u
	dt	01111000b	; 12: t
	dt	01011110b	; 13: d
	dt	01011100b	; 14: o
	dt	01110011b	; 15: P
	dt	00110111b	; 16: M
	dt	01111001b	; 17: E

ACHAR	EQU	10		; A
UCHAR	EQU	11		; u
TCHAR	EQU	12		; t (time)
DCHAR	EQU	13		; d (distance)
OCHAR	EQU	14		; o (odometer)
SCHAR	EQU	5		; S (speed)
PCHAR	EQU	15		; P
MCHAR	EQU	16		; M
ECHAR	EQU	17		; E
ICHAR	EQU	1		; I

; ----- obsluha zobrazen podle stavu

StatTab	addwf	PCL,F
	goto	DspTim		; zobrazen ubhlho asu
	goto	DspDis		; zobrazen urazen vzdlenosti
	goto	DspSpe		; zobrazen rychlosti
	goto	DspOdo		; zobrazen celkov vzdlenosti

; ----- texty pro indikaci stavu

HlasT1	addwf	PCL,F
	dt	TCHAR		; as 't'
	dt	DCHAR		; vzdlenosti 'd'
	dt	SCHAR		; rychlost 'S'
	dt	OCHAR		; celkov vzdlenost 'o'
	dt	ACHAR		; automatika 'A'

HlasT2	addwf	PCL,F
	dt	ICHAR		; as 'I'
	dt	ICHAR		; vzdlenosti 'I'
	dt	PCHAR		; rychlost 'P'
	dt	DCHAR		; celkov vzdlenost 'd'
	dt	UCHAR		; automatika 'u'


HlasT3	addwf	PCL,F
	dt	MCHAR		; as 'M'
	dt	SCHAR		; vzdlenosti 'S'
	dt	ECHAR		; rychlost 'E'
	dt	OCHAR		; celkov vzdlenost 'o'
	dt	TCHAR		; automatika 't'

;--------------------------------------------------------------------------
;	start programu
;--------------------------------------------------------------------------

; ----- nastaven smru bit port

Start	bsf	STATUS,RP0	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)

	movlw	00000b		; volba smru bit: 0=vstup, 1=vstup
	movwf	TRISA		; nastaven bit portu A (registr TRISA)
	movlw	00110000b	; volba smru bit: 0=vstup, 1=vstup
	movwf	TRISB		; nastaven bit portu B (registr TRISB)

; ----- nastaven asovae Timer0
; Obsluha peruen se vyvolv 32x za sekundu, kmitoet pro LCD
; je tedy 16 Hz. Pro krystal 32768 Hz je taktovac frekvence 8192 Hz,
; poadovan dlic pomr je 256, peddlika bude odpojena.

	movlw	0 + bit3 + bit7	; dlika odpojena, pipojena k WDT, bez pull-up
	movwf	OPTION_REG	; nastaven OPTION registru asova
	bcf	STATUS,RP0	; zapnut banky 0

; ----- inicializace pznak a promnnch

	clrf	Reg0		; nulovn registru 0
	clrf	Reg1		; nulovn registru 1
	clrf	Reg2		; nulovn registru 2

	clrf	Flags		; inicializace pznak
	clrf	RelKey		; nulovn tae uvolnn tlatka
	clrf	LongKey		; nulovn tae dlouhho stisku

	call	Clear		; nulovn registr

	clrf	SLenDM		; celkov vzdlenost v Dm
	clrf	SLenKM		; celkov vzdlenost v km
	clrf	SLenHM		; celkov vzdlenost v destkch kilometr

; ----- inicializace obsluhy peruen

Loop	movlw	bit3 + bit5 + bit7 ; povoleno peruen od TMR0, RB4-RB7 a globln peruen
	movwf	INTCON		; nastaven registru peruen INTCON

; ----- inicializace tae pro klid

	movlw	QUITMAX		; konstanta pro uklidnn
	movwf	Quit		; nastaven tae pro klid

; ----- dosaen stavu klidu
	
Loop2	movf	Quit,F		; test, zda je stav klidu
	btfss	STATUS,Z	; dosaen stav klidu?
	goto	Loop3		; nen stav klidu

	movlw	bit3		; povoleno vzbuzen RB4-RB7, zkaz peruen
	movwf	INTCON		; nastaven registru peruen INTCON

	clrf	PORTB		; vynulovn vstupu na port B
	clrf	PORTA		; vynulovn vstupu na port A
	bsf	PORTA,2		; vynulovn registru 1
	bcf	PORTA,2
	bsf	PORTA,3		; vynulovn registru 2
	bcf	PORTA,3

	SLEEP			; klid

	movlw	64		; interval dlouhho sisku tlatka (2 sekundy)
	movwf	LongKey		; ta dlouhho stisku tlatka
	movlw	2		; nov hodnota tae uvolnn (60 ms)
	movwf	RelKey		; inicializace tae uvolnn

	goto	Loop		; nov inicializace tae klidu

Loop3	goto	Loop2

;--------------------------------------------------------------------------
;	vymazn registr
;--------------------------------------------------------------------------

Clear	clrf	Hundr		; nulovn setin sekundy
	clrf	Second		; nulovn sekund
	clrf	Minute		; nulovn minut
	clrf	Hour		; nulovn hodin 24 hodin

	clrf	LenCM		; nulovn vzdlenosti v cm
	clrf	LenM		; nulovn vzdlenosti v m
	clrf	LenDM		; nulovn vzdlenosti v dm
	clrf	LenHM		; nulovn vzdlenosti v hm
	clrf	LenKM		; nulovn vzdlenosti v km

	movlw	CSTATE		; ta automatick zmny stavu
	movwf	CState		; inicializace tae automatick zmny

	clrf	State		; inicializace stavu
	bsf	Flags,4		; zapnut automatiky

	movlw	VELTIME		; konstanta tae doby rychlosti
	movwf	VelTim		; ta mc doby rychlosti
	clrf	CVelCM		; ta rychlosti v cm
	clrf	CVelM		; ta rychlosti v m
	clrf	VelHM		; rychlost v hektometrech
	clrf	VelKM		; rychlost v kilometrech
	return

;--------------------------------------------------------------------------
;	zobrazen sla W jako posledn 2 slice (0 a 99)
;--------------------------------------------------------------------------

; ----- schova bajtu k zobrazen

DispN1	movwf	TByte		; schova bajtu k zobrazen
	clrf	TByte2		; nulovn stadae

; ----- test bitu 3 slice

	movlw	80
	subwf	TByte,W		; test sla 80
	btfss	STATUS,C	; je slo 80 a vce?
	goto	DispN12		; nen slo 80 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,3	; nastaven bitu 3

; ----- test bitu 2 slice

DispN12	movlw	40
	subwf	TByte,W		; test sla 40
	btfss	STATUS,C	; je slo 40 a vce?
	goto	DispN13		; nen slo 40 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,2	; nastaven bitu 2

; ----- test bitu 1 slice

DispN13	movlw	20
	subwf	TByte,W		; test sla 20
	btfss	STATUS,C	; je slo 20 a vce?
	goto	DispN14		; nen slo 20 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,1	; nastaven bitu 1

; ----- test bitu 0 slice

DispN14	movlw	10
	subwf	TByte,W		; test sla 10
	btfss	STATUS,C	; je slo 10 a vce?
	goto	DispN15		; nen slo 10 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,0	; nastaven bitu 0

; ----- zobrazen vyho sla

DispN15	movf	TByte2,W	; naten vyho sla
	call	DekNum2		; zobrazen vyho sla

; ----- zobrazen niho sla

	movf	TByte,W		; naten niho sla
	call	DekNum1		; zobrazen niho sla
	return

;--------------------------------------------------------------------------
;	zobrazen sla W jako prvn 2 slice (0 a 19)
;--------------------------------------------------------------------------

; ----- schova bajtu k zobrazen

DispN2	movwf	TByte		; schova bajtu k zobrazen
	clrf	TByte2		; nulovn stadae

; ----- test bitu 0 slice

	movlw	10
	subwf	TByte,W		; test sla 10
	btfss	STATUS,C	; je slo 10 a vce?
	goto	DispN25		; nen slo 10 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,0	; nastaven bitu 0

; ----- zobrazen vyho sla

DispN25	movf	TByte2,W	; naten vyho sla
	call	DekNum4		; zobrazen vyho sla

; ----- zobrazen niho sla

	movf	TByte,W		; naten niho sla
	call	DekNum3		; zobrazen niho sla
	return

;--------------------------------------------------------------------------
;	zobrazen sla W jako posledn 3 slice (0 a 199)
;--------------------------------------------------------------------------

; ----- schova bajtu k zobrazen

DispN3	movwf	TByte		; schova bajtu k zobrazen

; ----- zobrazen sla < 10 (kvli potlaen vodnch nul)

	movlw	10		; bude se testovat 10 a vce
	subwf	TByte,W		; test sla 10 a vt
	btfsc	STATUS,C	; je slo 10 a vt?
	goto	DispN30		; nen slo 10 a vt

	call	DekNm40		; vypnut nejvy slice "1"
	clrw			; W <- 0
	call	DekNm30		; vymazn sla 3
	goto	DispN36		; dekdovn nejniho sla

; ----- test bitu 0 nejvy slice

DispN30	clrf	TByte2		; nulovn stadae
	movlw	100
	subwf	TByte,W		; test sla 100
	btfss	STATUS,C	; je slo 100 a vce?
	goto	DispN31		; nen slo 100 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,0	; nastaven bitu 0

; ----- zobrazen nejvyho sla

DispN31	movf	TByte2,W	; naten nejvyho sla
	call	DekNum4		; zobrazen nejvyho sla

; ----- test bitu 3 slice

	movlw	80
	subwf	TByte,W		; test sla 80
	btfss	STATUS,C	; je slo 80 a vce?
	goto	DispN32		; nen slo 80 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,3	; nastaven bitu 3

; ----- test bitu 2 slice

DispN32	movlw	40
	subwf	TByte,W		; test sla 40
	btfss	STATUS,C	; je slo 40 a vce?
	goto	DispN33		; nen slo 40 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,2	; nastaven bitu 2

; ----- test bitu 1 slice

DispN33	movlw	20
	subwf	TByte,W		; test sla 20
	btfss	STATUS,C	; je slo 20 a vce?
	goto	DispN34		; nen slo 20 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,1	; nastaven bitu 1

; ----- test bitu 0 slice

DispN34	movlw	10
	subwf	TByte,W		; test sla 10
	btfss	STATUS,C	; je slo 10 a vce?
	goto	DispN35		; nen slo 10 a vce
	movwf	TByte		; zkorigovan hodnota
	bsf	TByte2,0	; nastaven bitu 0

; ----- zobrazen vyho sla

DispN35	movf	TByte2,W	; naten vyho sla
	call	DekNum3		; zobrazen vyho sla

; ----- zobrazen niho sla

DispN36	movf	TByte,W		; naten niho sla
	call	DekNum2		; zobrazen niho sla
	return

;--------------------------------------------------------------------------
;	dekdovn slice 1 (W=0 a 9)
;--------------------------------------------------------------------------

; ----- uloen bajtu k dekdovn

DekNum1	call	ChrTab		; dekdovn na segmenty
	movwf	DByte		; schova bajtu k dekdovn

; ----- segment A

	bcf	Reg1,0
	btfsc	DByte,0
	bsf	Reg1,0
	
; ----- segment B

	bcf	Reg1,3
	btfsc	DByte,1
	bsf	Reg1,3

; ----- segment C

	bcf	Reg2,3
	btfsc	DByte,2
	bsf	Reg2,3

; ----- segment D

	bcf	Reg2,0
	btfsc	DByte,3
	bsf	Reg2,0

; ----- segment E

	bcf	Reg0,3
	btfsc	DByte,4
	bsf	Reg0,3

; ----- segment F

	bcf	Reg0,0
	btfsc	DByte,5
	bsf	Reg0,0

; ----- segment G

	bcf	Reg0,1
	btfsc	DByte,6
	bsf	Reg0,1
	return

;--------------------------------------------------------------------------
;	dekdovn slice 2 (W=0 a 9)
;--------------------------------------------------------------------------

; ----- uloen bajtu k dekdovn

DekNum2	call	ChrTab		; dekdovn na segmenty
	movwf	DByte		; schova bajtu k dekdovn

; ----- segment A

	bcf	Reg1,1
	btfsc	DByte,0
	bsf	Reg1,1
	
; ----- segment B

	bcf	Reg1,2
	btfsc	DByte,1
	bsf	Reg1,2

; ----- segment C

	bcf	Reg2,1
	btfsc	DByte,2
	bsf	Reg2,1

; ----- segment D

	bcf	Reg2,2
	btfsc	DByte,3
	bsf	Reg2,2

; ----- segment E

	bcf	Reg2,6
	btfsc	DByte,4
	bsf	Reg2,6

; ----- segment F

	bcf	Reg1,4
	btfsc	DByte,5
	bsf	Reg1,4

; ----- segment G

	bcf	Reg1,6
	btfsc	DByte,6
	bsf	Reg1,6
	return

;--------------------------------------------------------------------------
;	dekdovn slice 3 (W=0 a 9)
;--------------------------------------------------------------------------

; ----- uloen bajtu k dekdovn

DekNum3	call	ChrTab		; dekdovn na segmenty
DekNm30	movwf	DByte		; schova bajtu k dekdovn

; ----- segment A

	bcf	Reg0,5
	btfsc	DByte,0
	bsf	Reg0,5
	
; ----- segment B

	bcf	Reg0,6
	btfsc	DByte,1
	bsf	Reg0,6

; ----- segment C

	bcf	Reg0,4
	btfsc	DByte,2
	bsf	Reg0,4

; ----- segment D

	bcf	Reg0,7
	btfsc	DByte,3
	bsf	Reg0,7

; ----- segment E

	bcf	Reg2,7
	btfsc	DByte,4
	bsf	Reg2,7

; ----- segment F

	bcf	Reg1,7
	btfsc	DByte,5
	bsf	Reg1,7

; ----- segment G

	bcf	Reg1,5
	btfsc	DByte,6
	bsf	Reg1,5
	return

;--------------------------------------------------------------------------
;	dekdovn slice 4 (W=0 a 1)
;--------------------------------------------------------------------------

; ----- dekdovn slice

DekNum4	bsf	Reg2,5		; zapnut slice "1"
	iorlw	0		; test, zda je slo 0
	btfsc	STATUS,Z	; je slo 0?
DekNm40	bcf	Reg2,5		; vypnut slice "1"
	return

;--------------------------------------------------------------------------
;	vypnut desetinn teky
;--------------------------------------------------------------------------

SetPT0	bcf	Reg0,2
	bcf	Reg2,4
	return

;--------------------------------------------------------------------------
;	zobrazen desetinn teky na pozici 1
;--------------------------------------------------------------------------

SetPT1	bsf	Reg0,2
	bcf	Reg2,4
	return

;--------------------------------------------------------------------------
;	zobrazen desetinn teky na pozici 2
;--------------------------------------------------------------------------

SetPT2	bcf	Reg0,2
	bsf	Reg2,4
	return

;--------------------------------------------------------------------------
;	nulovn vstupnch registr (zachovv registr W)
;--------------------------------------------------------------------------

NulReg	clrf	Reg0		; nulovn registru 0
	clrf	Reg1		; nulovn registru 1
	clrf	Reg2		; nulovn registru 2
	return

;--------------------------------------------------------------------------
;	indikace stavu W (nebo aktulnho stavu Indik0)
;--------------------------------------------------------------------------

; ----- pouije se aktuln stav

Indik0	movf	State,W		; naten aktulnho stavu

; ----- vynulovn vstupnch registr

Indik	call	NulReg		; vynulovn vstupnch registr

; ----- dekdovn levho znaku

	movwf	HlasN		; schova sla hlen
	call	HlasT1		; naten prvnho znaku
	call	DekNum3		; dekdovn prvnho znaku

; ----- dekdovn stednho znaku

	movf	HlasN,W		; naten sla hlen
	call	HlasT2		; naten druhho znaku
	call	DekNum2		; dekdovn druhho znaku

; ----- dekdovn pravho znaku

	movf	HlasN,W		; naten sla hlen
	call	HlasT3		; naten tetho znaku
	call	DekNum1		; dekdovn prvnho znaku

; ----- zapnut pznaku zobrazen indikace

	bsf	Flags,6		; zapnut pznaku indikace

; ----- nastaven tae doby indikace

	movlw	CIND		; doba indikace
	movwf	CInd		; nastaven tae indikace
	return

;--------------------------------------------------------------------------
;	zobrazen ubhlho asu
;--------------------------------------------------------------------------

; ----- zobrazen desetinn teky

DspTim	call	SetPT0		; vypnut desetinn teky
	btfsc	Flags,5		; b as? (pokud ne, teka svt vdy)
	btfss	Hundr,4		; je druh polovina sekundy?
	call	SetPT2		; zobrazen desetinn teky

; ----- rozlien formtu zobrazen asu

	movf	Hour,F		; test, zda jsou njak hodiny
	btfss	STATUS,Z	; jsou njak hodiny?
	goto	DspTim2		; jsou hodiny
	movlw	20		; maximln minut
	subwf	Minute,W	; test peteen minut
	btfsc	STATUS,C	; je peteen minut?
	goto	DspTim2		; zobrazen pro 20 a vce minut

; ----- zobrazen as MM.SS

	movf	Second,W	; naten sekund
	call	DispN1		; zobrazen sekund jako prav slo
	movf	Minute,W	; naten minut
	goto	DspTim3		; zobrazen minut

; ----- zobrazen as HH.MM

DspTim2	movf	Minute,W	; naten minut
	call	DispN1		; zobrazen minut jako prav slo
	movf	Hour,W		; naten hodin
DspTim3	call	DispN2		; zobrazen levho sla
	return

;--------------------------------------------------------------------------
;	zobrazen urazen vzdlenosti
;--------------------------------------------------------------------------

; ----- rozlien mdu zobrazen

DspDis	movlw	20		; maximln vzdlenost v km
	subwf	LenKM,W		; test peteen vzdlenosti
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	DspDis2		; nen peteen - zobrazen v mdu kk.kk

; ----- zobrazen v mdu kkk.k

	movf	LenHM,W		; vzdlenost ve stovkch m
	call	DekNum1		; dekdovn posledn slice
	movf	LenKM,W		; vzdlenost v km
	call	DispN3		; dekdovn jednotek km

	call	SetPT1		; zobrazen desetinn teky
	return

; ----- zobrazen v mdu kk.kk

DspDis2	movf	LenDM,W		; vzdlenost v destkch m
	call	DekNum1		; dekdovn posledn slice
	movf	LenHM,W		; vzdlenost ve stovkch m
	call	DekNum2		; dekdovn pedposledn slice
	movf	LenKM,W		; vzdlenost v km
	call	DispN2		; dekdovn jednotek km

	call	SetPT2		; zobrazen desetinn teky
	return

;--------------------------------------------------------------------------
;	zobrazen rychlosti
;--------------------------------------------------------------------------

DspSpe	movf	VelHM,W		; ni slice rychlosti
	call	DekNum1		; dekdovn posledn slice

	call	SetPT1		; zobrazen desetinn teky

	movf	VelKM,W		; vy slo
	call	DispN3		; zobrazen destek km
	return

;--------------------------------------------------------------------------
;	zobrazen celkov vzdlenosti (vstupn registry mus bt vynulovny)
;--------------------------------------------------------------------------

DspOdo	movf	SLenKM,W	; celkov vzdlenost v km
	call	DekNum1		; dekdovn posledn slice

	movf	SLenHM,W	; celkov vzdlenost v destkch km
	btfss	STATUS,Z	; jsou destky km = 0?
	call	DispN3		; zobrazen destek km
	return

;--------------------------------------------------------------------------
;	obsluha peruen
;--------------------------------------------------------------------------

; ----- schova registr W a STATUS

Inter	movwf	TempW		; schova registru W
	swapf	STATUS,W	; naten STATUS registru do W
	movwf	TempS		; schova STATUS registru

; ----- test, zda je peruen od portu

	btfss	INTCON,0	; je peruen od portu?
	goto	Inter1		; nen peruen od portu
	bcf	INTCON,0	; nulovn pznaku peruen od portu

; ----- inicializace tae pro klid

	movlw	QUITMAX		; konstanta pro uklidnn
	movwf	Quit		; nastaven tae pro klid

; ----- test, zda je senzor sepnut

	btfsc	PORTB,5		; je senzor sepnut?
	goto	Inter0		; senzor je sepnut
	bcf	Flags,2		; vypnut pznaku sepnut senzoru
	goto	Inter1

; ----- test, zda senzor byl sepnut

Inter0	btfsc	Flags,2		; byl senzor ji sepnut?
	goto	Inter1		; senzor byl ji sepnut
	bsf	Flags,2		; zapnut pznaku sepnut senzoru
	
; ----- zapnut bhu asu

Intr00	bsf	Flags,5		; zapnut bhu asu

; ----- tn urazen vzdlenosti

	movlw	LENINC		; prustek vzdlenosti na impuls
	addwf	LenCM,F		; zven urazen vzdlenosti v cm
	movlw	100		; max. hodnota vzdlenosti v cm
	subwf	LenCM,W		; test peteen vzdlenosti v cm
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter02		; nen peteen vzdlenosti
	movwf	LenCM		; petkajc hodnota

	incf	LenM,F		; zven vzdlenosti v m
	movlw	10		; max. hodnota vzdlenosti v m
	subwf	LenM,W		; test peteen vzdlenosti v m
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter02		; nen peteen vzdlenosti
	movwf	LenM		; petkajc hodnota

	incf	LenDM,F		; zven vzdlenosti v Dm
	movlw	10		; max. hodnota vzdlenosti v Dm
	subwf	LenDM,W		; test peteen vzdlenosti v Dm
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter01		; nen peteen vzdlenosti
	movwf	LenDM		; petkajc hodnota

	incf	LenHM,F		; zven vzdlenosti v Hm
	movlw	10		; max. hodnota vzdlenosti v Hm
	subwf	LenHM,W		; test peteen vzdlenosti v Hm
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter01		; nen peteen vzdlenosti
	movwf	LenHM		; petkajc hodnota
	
	incf	LenKM,F		; zven vzdlenosti v km
	movlw	200		; max. hodnota vzdlenosti v km
	subwf	LenKM,W		; test peteen vzdlenosti v km
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter01		; nen peteen vzdlenosti
	movwf	LenKM		; petkajc hodnota

; ----- tn celkov vzdlenosti (po zmn vzdlenosti v Dm)

Inter01	incf	SLenDM,F	; zven vzdlenosti v Dm
	movlw	100		; max. hodnota vzdlenosti v Dm
	subwf	SLenDM,W	; test peteen vzdlenosti v Dm
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter02		; nen peteen vzdlenosti
	movwf	SLenDM		; petkajc hodnota

	incf	SLenKM,F	; zven vzdlenosti v km
	movlw	10		; max. hodnota vzdlenosti v km
	subwf	SLenKM,W	; test peteen vzdlenosti v km
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter02		; nen peteen vzdlenosti
	movwf	SLenKM		; petkajc hodnota
	
	incf	SLenHM,F	; zven vzdlenosti v destkch km
	movlw	200		; max. hodnota vzdlenosti v destkch km
	subwf	SLenHM,W	; test peteen vzdlenosti v destkch km
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter02		; nen peteen vzdlenosti
	movwf	SLenHM		; petkajc hodnota

; ----- tn rychlosti

Inter02	movlw	LENINC		; prustek vzdlenosti na impuls
	addwf	CVelCM,F	; zven urazen vzdlenosti v cm
	movlw	100		; max. hodnota vzdlenosti v cm
	subwf	CVelCM,W	; test peteen vzdlenosti v cm
	btfss	STATUS,C	; je peteen vzdlenosti?
	goto	Inter1		; nen peteen vzdlenosti
	movwf	CVelCM		; petkajc hodnota

	incf	CVelM,F		; zven vzdlenosti v m

; ----- test, zda je peruen od TMR0

Inter1	btfss	INTCON,2	; je peruen od asovae?
	goto	Inter9		; nen peruen od asovae
	bcf	INTCON,2	; nulovn pznaku peruen od TMR0

; ----- tn setin sekundy

	incf	Hundr,F		; zven tae setin sekundy
	movf	Hundr,W		; naten tae setin sekundy do W
	sublw	32		; test maximln hodnoty setin sekundy
	btfss	STATUS,Z	; je peteen setin sekundy?
	goto	Inter2		; nen jet peteen setin sekundy
	clrf	Hundr		; nulovn tae setin sekundy

; ----- tn stavu klidu

	decf	Quit,F		; ta stavu klidu

; ----- test, zda b as

	btfss	Flags,5		; b as?
	goto	Inter2		; as neb

; ----- zven tae sekund

	incf	Second,F	; zven tae sekund
	movf	Second,W	; naten tae sekund
	sublw	60		; test maximln hodnoty sekund
	btfss	STATUS,Z	; je peteen sekund?
	goto	Inter2
	clrf	Second		; nulovn tae sekund

; ----- zven tae minut

	incf	Minute,F	; zven tae minut
	movf	Minute,W	; naten tae minut
	sublw	60		; test maximln hodnoty minut
	btfss	STATUS,Z	; je peteen minut?
	goto	Inter2
	clrf	Minute		; nulovn tae minut

; ----- zven tae hodin

	incf	Hour,F		; zven tae hodin
	movf	Hour,W		; naten tae hodin
	sublw	20		; test maximln hodnoty hodin
	btfsc	STATUS,Z	; je peteen hodin?
	clrf	Hour		; nulovn tae hodin

; ----- zobrazen indikace

Inter2	btfss	Flags,6		; je zapnuta indikace?
	goto	Inter20		; nen zapnuta indikace
	decfsz	CInd,F		; ta doby indikace
	goto	Inter28		; nen jet konec indikace

	bcf	Flags,6		; vypnut indikace
	btfss	Flags,7		; je indikace automatiky?
	goto	Inter20		; nen indikace automatiky

	bcf	Flags,7		; vypnut indikace automatiky
	call	Indik0		; indikace aktulnho stavu
	goto	Inter28

; ----- zobrazen podle stavu

Inter20	call	NulReg		; vynulovn vstupnch registr
	movf	State,W		; naten aktulnho stavu
	call	StatTab		; zobrazen

; ----- zobrazen displeje a vstup z tlatka

Inter28	call	ReDisp		; pekreslen displeje

; ----- automatick pepnn zobrazen

	btfss	Flags,4		; je automatick pepnn?
	goto	Inter3		; nen automatick pepnn
	decfsz	CState,F	; tn automatick zmny
	goto	Inter3		; nen automatick zmna

	movlw	CSTATE		; ta automatick zmny stavu
	movwf	CState		; inicializace tae automatick zmny

	incf	State,F		; zven tae stavu
	movlw	3		; maximln stav
	subwf	State,W		; test maximlnho stavu
	btfsc	STATUS,Z	; je maximln stav?
	clrf	State		; nulovn stavu	

	call	Indik0		; zobrazen novho stavu

; ----- vstup z tlatka

Inter3	btfss	Flags,1		; je stisk tlatka?
	goto	Inter5		; nen stisk tlatka

; ----- nulovn

	btfss	Flags,3		; je dlouh stisk tlatka?
	goto	Inter32		; nen dlouh stisk tlatka

	call	Clear		; vymazn registr

	movlw	0xff		; vechny bity budou zapnuty
	movwf	Reg0		; zapnut bit registru 0
	movwf	Reg1		; zapnut bit registru 1
	movwf	Reg2		; zapnut bit registru 2

	bsf	Flags,6		; zapnut pznaku indikace

	movlw	32*4		; doba indikace (4 sekundy)
	movwf	CInd		; nastaven tae indikace

	goto	Inter5

; ----- posun stavu

Inter32	btfss	Flags,4		; je automatick zmna stavu?
	goto	Inter33		; nen automatick zmna stavu

	bcf	Flags,4		; vypnut automatick zmny stavu
	movlw	STIME		; pechod na zobrazen asu
	movwf	State		; zkladn stav
	goto	Inter34		; indikace stavu

Inter33	incf	State,W		; zven tae stav
	andlw	11b		; maskovn hodnoty
	movwf	State		; nov hodnota stavu
	btfss	STATUS,Z	; je pechod pes konec?
	goto	Inter34		; nen pechod pes konec

	bsf	Flags,4		; zapnut automatick zmny
	movlw	CSTATE		; ta automatick zmny stavu
	movwf	CState		; inicializace tae automatick zmny

	bsf	Flags,7		; zapnut pznaku automatiky
	movlw	SAUT		; stav automatiky
Inter34	call	Indik		; zobrazen stavu

; ----- tn doby men rychlosti

Inter5	decfsz	VelTim,F	; tn doby men
	goto	Inter9		; nen jet konec men

	movlw	VELTIME		; konstanta tae doby rychlosti
	movwf	VelTim		; ta mc doby rychlosti

	clrf	VelHM		; rychlost v hektometrech
	movf	CVelCM,W	; ta rychlosti v cm

Inter52	incf	VelHM,F		; ta rychlosti v hektometrech
	addlw	-10		; tn dekd
	btfsc	STATUS,C	; je peteen?
	goto	Inter52		; spoten dekd
	decf	VelHM,F		; nvrat posledn hodnoty

	movf	CVelM,W		; ta rychlosti v m
	movwf	VelKM		; rychlost v km

	clrf	CVelCM		; ta rychlosti v cm
	clrf	CVelM		; ta rychlosti v m

	bsf	Flags,5		; zapnut bhu asu
	iorwf	VelHM,W		; test, zda je rychlost = 0
	btfsc	STATUS,Z	; je njak rychlost?
	bcf	Flags,5		; vypnut bhu asu

; ----- nvrat registr W a STATUS

Inter9	swapf	TempS,0		; pvodn STATUS registr k navrcen
	movwf	STATUS		; nvrat pvodnho STATUS registru
	swapf	TempW,1		; pedpprava pvodnho registru W
	swapf	TempW,0		; nvrat pvodnho W registru
	retfie			; nvrat z obsluhy peruen

;--------------------------------------------------------------------------
;	odesln bajtu W na displej (s rozlienm inverze)
;--------------------------------------------------------------------------

; ----- inverze bajtu

OutByte	btfsc	Flags,0		; je inverze zobrazen?
	xorlw	0xff		; inverze bajtu

; ----- schova bajtu k odesln

	movwf	DByte		; schova bajtu k odesln

; ----- vstup na port B (bit 4 a 5 nevyuit, jsou to vstupy)

	movwf	PORTB		; vstup na port B

; ----- vstup na port A

	bcf	PORTA,0		; vypnut bitu 0 portu A
	bcf	PORTA,1		; vypnut bitu 1 portu A
	swapf	DByte,W		; naten bajtu k vstupu
	andlw	bit0 + bit1	; maskovn bit k nastaven
	iorwf	PORTA,F		; vstup bit na port A
	return

;--------------------------------------------------------------------------
;	pekreslen displeje LCD
;--------------------------------------------------------------------------

; ----- nastaven registru 2

ReDisp	movf	Reg2,W		; naten registru 2
	call	OutByte		; vstup bajtu na port
	bsf	PORTA,3		; zapnut vstupu do registru 2
	bcf	PORTA,3		; vypnut vstupu do registru 2

; ----- nastaven registru 1

	movf	Reg1,W		; naten registru 1
	call	OutByte		; vstup bajtu na port
	bsf	PORTA,2		; zapnut vstupu do registru 1
	bcf	PORTA,2		; vypnut vstupu do registru 1

; ----- nastaven registru 0

	movf	Reg0,W		; naten registru 0
	call	OutByte		; vstup bajtu na port

; ----- nastaven spolen elektrody

	bcf	PORTA,4		; vypnut spolen elektrody
	btfsc	Flags,0		; je inverze zobrazen?
	bsf	PORTA,4		; zapnut spolen elektrody

; ----- peklopen pznaku inverze

	movlw	bit0		; bit k peklopen stavu
	xorwf	Flags,F		; peklopen pznaku inverze

; ----- obsluha vstupu z tlatka

	bcf	Flags,1		; vypnut pznaku stisku tlatka
	bcf	Flags,3		; vypnut pznaku dlouhho stisku tlatka

	movf	RelKey,F	; test, zda b ta uvolnn tlatka
	btfsc	STATUS,Z	; b ta uvolnn tlatka?
	goto	ReDsp32		; ta neb

	decfsz	LongKey,F	; ta opakovn tlatka
	goto	ReDsp31		; nen jet 0
	bsf	Flags,3		; pznak dlouhho stisku tlatka
	goto	ReDsp34

ReDsp31	btfsc	PORTB,4		; je tlatko stisknuto?
	goto	ReDsp36		; je stisknuto - inicializace tae uvolnn
	decf	RelKey,F	; snen tae uvolnn
	goto	ReDisp4

ReDsp32	btfss	PORTB,4		; je tlatko stisknuto?
	goto	ReDisp4		; tlatko nen stisknuto

ReDsp34	movlw	64		; interval dlouhho sisku tlatka (2 sekundy)
	movwf	LongKey		; ta dlouhho stisku tlatka
	bsf	Flags,1		; pznak stisku tlatka

ReDsp36	movlw	2		; nov hodnota tae (60 ms)
	movwf	RelKey		; nov inicializace tae uvolnn

ReDisp4	return
