;**************************************************************************
; EYEWATCH.ASM					(c) Ing. Miroslav Nmeek
;--------------------------------------------------------------------------
; Modul EyeWatch monitoruje dobu strvenou u obrazovky potae. Jeho
; hlavnm poslnm je hldn zraku uivatele ped nadmrnm namhnm.
; Modul sleduje dobu aktivity potae a v ppad pekroen maximln
; povolen doby (zpravidla 6 hodin) o tom informuje uivatele bliknm
; kontroln LED diody.
;--------------------------------------------------------------------------
; Tento program je nejjednodu variantou softwarov podpory modulu
; EyeWatch. Doba povolen aktivity je urena v programu procesoru (konstanta
; MAXTIME) a nen podporovno zen modulu softwarov. Po aktivaci modulu
; zane modul odpotvat as zbvajc povolen aktivity. Po dosaen
; zbvajcho asu WARTIME zmn indikaci na varovnou indikaci (lut LED),
; po dosaen stavu 0 rozblik ervenou LED jako indikaci konce doby.
;
; Program EYEWATCH.SCR je stmvaem obrazovky. Pi svm sputn vysl
; do modulu EyeWatch synchronizan data, kter zastavuj tn asu.
; Synchronizan data jsou jakkoliv data (doporueno 00h) vyslan
; s penosovou rychlost asi 1200 Baud a s frekvenc 1 bajt za sekundu.
;--------------------------------------------------------------------------
; Doporuen pro budouc rozen software modulu:
;   - Pomoc sriovho portu nastavovat konfiguraci modulu
;   - Uchovvat dosaen stav tae v EEPROM pro ppad resetovn nebo
;     vypnut potae; pi novm startu potae je do modulu odeslna
;     informace o aktulnm asu, o aktuln smn atd.; porovnnm
;     s uloenmi daji me modul zjistit, e je ji dal smna a
;     e m inicializovat ta asu pro jinho pracovnka
;**************************************************************************

MAXTIME	EQU	6*12		; maximln povolen doba aktivity v 5-minutch
WARTIME	EQU	6		; zbvajc doba pro varovn hlen v 5-minutch

	RADIX	DEC
	CONFIG	LP

;--------------------------------------------------------------------------
;	promnn
;--------------------------------------------------------------------------

	CBLOCK
Hundr				; ta zlomk 1/8 sekundy (= 8 a 1)
Second				; ta sekund v minut (= 60 a 1)
Minute				; ta minut v 5-minutovm kroku (= 5 a 1)
Rest				; ta zbytku 5-minutovch krok (0=konec)

Indik				; stav indikace
				;	bit 0: zelen svt
				;	bit 1: lut svt
				;	bit 2: erven svt
				;	bit 3: zelen blik
				;	bit 4: lut blik
				;	bit 5: erven blik

BlinkC				; ta posuvnho blikn (0=nen blikn)
StopC				; ta indikace zastaven

TempD				; schova W pi zobrazen
TempW				; schova registru W pi peruen
TempS				; schova registru STATUS pi peruen
	ENDC


	ORG	0
	goto	Start		; start programu

	ORG	4
	goto	Inter		; obsluha peruen

;--------------------------------------------------------------------------
;	start programu
;--------------------------------------------------------------------------

; ----- nastaven smru port A a B

Start	bsf	STATUS,RP0	; zapnut banky 1

	movlw	11101b		; RA1 jako vstup
	movwf	TRISA		; nastaven smru port A

	movlw	11001111b	; RB4 a RB5 jako vstup
	movwf	TRISB		; nastaven smru port B

; ----- nastaven asovae Timer0
; Obsluha peruen se vyvolv 8x za sekundu. Pro krystal 32768 Hz
; je taktovac frekvence 8192 Hz, poadovan dlic pomr je 256 * 4.

	movlw	1 + bit7	; dlika 1/4, pipojena k TMR0, bez pull-up
	movwf	OPTION_REG	; nastaven OPTION registru asova
	bcf	STATUS,RP0	; zapnut banky 0

; ----- inicializace registr

	movlw	8		; ta zlomku sekundy
	movwf	Hundr		; inicializace tae zlomku sekundy
	movlw	60		; ta sekund v minut
	movwf	Second		; inicializace tae sekund v minut
	movlw	5		; ta minut v 5-minutovm kroku
	movwf	Minute		; inicializace tae minut v 5-minutovm kroku
	movlw	MAXTIME		; ta 5-minutovch krok
	movwf	Rest		; inicializace tae 5-minutovch krok

	movlw	bit0		; bude svtit zelen
	movwf	Indik		; nastaven indikace
	call	Disp		; zobrazen pznak
	movlw	3*3		; poet bliknut
	movwf	BlinkC		; nastaven tae bliknut
	clrf	StopC		; nulovn tae indikace vypnut

; ----- inicializace obsluhy peruen

	movlw	bit5 + bit7	; povoleno peruen od TMR0 a globln peruen
	movwf	INTCON		; nastaven registru peruen INTCON

; ----- ekn na synchronizan bit (m rove 1)

Loop	btfss	PORTA,0		; je synchronizan bit?
	goto	Loop		; ekn na synchronizan bit

; ----- zkaz globlnho peruen

	bcf	INTCON,GIE	; zkaz globlnho peruen

; ----- b stmva obrazovky - nulovn tae asu

	movlw	8		; ta zlomku sekundy
	movwf	Hundr		; inicializace tae zlomku sekundy
	movlw	60		; ta sekund v minut
	movwf	Second		; inicializace tae sekund v minut
	movf	StopC,W		; naten tae zastaven
	iorlw	18h		; stav tae zastaven (asi 4 sekundy), ponech nejni 3 bity
	movwf	StopC		; nastaven tae indikace zastaven

; ----- povolen globlnho peruen

	bsf	INTCON,GIE	; povolen globlnho peruen

; ----- ekn na konec synchronizanho bitu

Loop2	btfsc	PORTA,0		; je konec synchronizanho bitu?
	goto	Loop2		; ekn na konec synchronizanho bitu
	goto	Loop

;--------------------------------------------------------------------------
;	zobrazen indikace (W: bit0=zelen, bit1=lut, bit2=erven)
;--------------------------------------------------------------------------

; ----- schova bajtu k zobrazen

Disp	movwf	TempD		; schova bajtu k zobrazen

; ----- zelen

	btfsc	TempD,0		; m svtit zelen?
	bcf	PORTA,1		; zapnut zelen
	btfss	TempD,0		; nem svtit zelen?
	bsf	PORTA,1		; vypnut zelen

; ----- lut

	btfsc	TempD,1		; m svtit lut?
	bcf	PORTB,5		; zapnut lut
	btfss	TempD,1		; nem svtit lut?
	bsf	PORTB,5		; vypnut lut

; ----- erven

	btfsc	TempD,2		; m svtit erven?
	bcf	PORTB,4		; zapnut erven
	btfss	TempD,2		; nem svtit erven?
	bsf	PORTB,4		; vypnut erven
	return

;--------------------------------------------------------------------------
;	obsluha peruen (vyvolvna 8x za sekundu)
;--------------------------------------------------------------------------

; ----- schova registr W a STATUS

Inter	movwf	TempW		; schova registru W
	swapf	STATUS,W	; naten STATUS registru do W
	movwf	TempS		; schova STATUS registru

; ----- nulovn poadavku peruen od TMR0

	bcf	INTCON,2	; nulovn pznaku peruen od TMR0

; ----- test, zda je ji peteen asu

	movf	Rest,F		; test, zda je peteen men asu
	btfsc	STATUS,Z	; je peteen men asu?
	goto	Inter2		; je peteen men asu

; ----- tn zlomku sekundy

	decfsz	Hundr,F		; tn zlomku sekundy
	goto	Inter2		; nen jet cel sekunda
	movlw	8		; ta zlomku sekundy
	movwf	Hundr		; inicializace tae zlomku sekundy

; ----- tn sekund v minut

	decfsz	Second,F	; tn sekund v minut
	goto	Inter2		; nen jet cel minuta
	movlw	60		; ta sekund v minut
	movwf	Second		; inicializace tae sekund v minut

; ----- tn minut v 5-minutovm kroku

	decfsz	Minute,F	; tn minut v 5-minutovm kroku
	goto	Inter2		; nen jet cel 5-minutov krok
	movlw	5		; ta minut v 5-minutovm kroku
	movwf	Minute		; inicializace tae minut v 5-minutovm kroku

; ----- tn 5-minutovch krok

	decf	Rest,F		; tn 5-minutovch krok

; ----- test, zda je posuvn blikn

Inter2	movf	BlinkC,F	; test, zda je posuvn blikn
	btfsc	STATUS,Z	; je posuvn blikn?
	goto	Inter3		; nen posuvn blikn
	decf	BlinkC,F	; tn posuvnho blikn

; ----- posun indikace pi posuvnm blikn

	bcf	STATUS,C	; nulovn pznaku C
	rlf	Indik,W		; naten indikace s rotovnm
	bcf	Indik,0		; nulovn pznaku zelen
	bcf	Indik,1		; nulovn pznaku lut
	bcf	Indik,2		; nulovn pznaku erven
	andlw	111b		; maskovn pznak
	btfsc	STATUS,Z	; je peteen?
	movlw	bit0		; nov stav
	iorwf	Indik,F		; nastaven novch pznak
	goto	Inter8		; zobrazen novho stavu

; ----- nulovn indikac (ponech se pouze erven, kter me blikat)

Inter3	movlw	bit2		; bude ponechn pouze pznak erven LED
	andwf	Indik,F		; maskovn pouze erven LED

; ----- indikace peteen asu

	movf	Rest,F		; test, zda je peteen men asu
	btfss	STATUS,Z	; je peteen men asu?
	goto	Inter4		; nen peteen men asu
	bsf	Indik,5		; zapnut pznaku blikn erven
	goto	Inter7

; ----- test, zda je ji varovn hlen

Inter4	movlw	WARTIME+1	; as pro varovnou indikaci
	subwf	Rest,W		; test, zda je ji varovn indikace
	movlw	bit0		; bn indikace
	btfss	STATUS,C	; je varovn indikace?
	movlw	bit1		; je varovn indikace
	movwf	Indik		; nastaven novch pznak

; ----- obsluha indikace zastaven

	movf	StopC,W		; test, zda b indikace zastaven
	btfsc	STATUS,Z	; b indikace zastaven?
	goto	Inter7		; nen indikace zastaven
	decf	StopC,F		; dekrementace indikace zastaven

	andlw	7		; test, zda je 1/8 asu
	btfsc	STATUS,Z	; je 1/8 asu?
	clrf	Indik		; bliknut kadou sekundu

; ----- obsluha blikn zelen

Inter7	movlw	bit0		; pznak zelen
	btfsc	Indik,3		; blik zelen?
	xorwf	Indik,F		; peklopen zelen

; ----- obsluha blikn lut

	movlw	bit1		; pznak lut
	btfsc	Indik,4		; blik lut?
	xorwf	Indik,F		; peklopen lut

; ----- obsluha blikn erven

	movlw	bit2		; pznak erven
	btfsc	Indik,5		; blik erven?
	xorwf	Indik,F		; peklopen erven

; ----- zobrazen novho stavu

Inter8	movf	Indik,W		; naten pznak indikace
	call	Disp		; zobrazen pznak

; ----- nvrat registr W a STATUS

Inter9	swapf	TempS,0		; pvodn STATUS registr k navrcen
	movwf	STATUS		; nvrat pvodnho STATUS registru
	swapf	TempW,1		; pedpprava pvodnho registru W
	swapf	TempW,0		; nvrat pvodnho W registru
	retfie			; nvrat z obsluhy peruen

