;**************************************************************************
; LEKCE01.ASM					(c) Ing. Miroslav Nmeek
;
; 1. lekce programovn procesor PIC - zkladn seznmen
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo LED dioda pipojen k P84-CPU mezi signl A0 (anoda) a zem
;**************************************************************************
;
; V naich lekcch se budeme zabvat 8-bitovmi jednoipovmi mikroadii
; ady PIC16xxx. Jedn se o vrobky firmy MICROCHIP.
; 
; Snad nejvce proslavil procesory PIC procesor PIC16C84, pedevm dky
; nzk cen, malm hardwarovm poadavkm a monosti rychlho a snadnho
; peprogramovn pamti.
;
; My se budeme v lekcch zabvat dnes nejtypitjm zstupcem ady,
; procesorem PIC16F84, kter nahradil PIC16C84 a li se od nj pouze vt
; pamt a typem pamti FLASH. Ostatn procesory roziuj PIC16F84 o dal
; vybaven a zatm se jimi nemusme zabvat. Prakticky vechny programy je
; mon pevst do vych typ procesor bez vtch problm, je pouze
; vhodn provst nov peklad (jin adresovn EEPROM), zkontrolovat adresy
; datovch promnnch a zkontrolovat konfiguran slovo, je-li pouito.
;
; Procesory PIC jsou procesory typu RISC, tedy procesory s omezenou
; instrukn sadou. To znamen, e k dispozici je pouze nkolik zkladnch
; instrukc procesoru (35), dky emu zpracovn instrukc probh velmi
; rychle (1 a 2 strojov takty) a kad instrukce zabr pouze 1 slovo
; programu.
;
; Procesory PIC pouvaj tzv. Harvardovu architekturu s oddlenmi
; sbrnicemi pro program a data. To umouje zven vkonu procesoru
; samostatnm zpracovnm programu a dat a monost pouvat programovou
; sbrnici s vce bity ne m datov sbrnice. Programov sbrnice m
; ku 14 bit (datov 8), co je ka jednoho programovho slova,
; tj. jedn instrukce.
;
; Jak je obvykl u RISC, procesor m k dispozici pouze 1 pracovn registr
; (8-bitov), oznaovan W (working register, u jinch procesor znm
; jako A, ili akumultor). Vechny ostatn registry jsou 8-bitov datov
; promnn F (files). Registry se oznauj slem, kter pedstavuje adresu
; registru v datov pamti. Nkter registry maj speciln vznam (porty,
; stavov registr), jin mohou bt programem obecn pouiteln (GPR
; registry).
;
; V na vodn lekci se seznmme s metodou programovn procesor PIC
; pomoc jednoduchho programu, kter pouze zajist rozsvcen LED diody
; na vstupnm portu (LED je zapojena mezi signl A0 a zem, anoda je pitom
; na A0).
;
;--------------------------------------------------------------------------
; Po stisku tlatka "Nov" vm program PicDev pednabdne pipraven
; program s nkolika zkladnmi pkazy. Jako prvn je zde pkaz RADIX:


	RADIX	DEC	; pouijeme dekadickou selnou soustavu


; Pkaz RADIX slou k volb seln soustavy, v kter budete zadvat
; seln konstanty v programu. seln konstanty mus vdy zanat
; slic, krom ppadu prefixovho zadn seln soustavy. Nen-li
; v kdu konstanty udno jinak, interpretuje se konstanta v seln
; soustav zadan pkazem RADIX. Za pkazem RADIX me jako parametr
; nsledovat DEC (dekadick seln soustava), HEX (hexadecimln),
; OCT (oktalov), BIN (binrn) nebo me nsledovat seln parametr
; 2, 8, 10 nebo 16 (udno v naposledy zvolen seln soustav). Bez
; zadn pkazu RADIX je implicitn pouita dekadick seln soustava.
; Pozor pi pebrn program z jinch peklada - nkter pekladae
; pedpokldaj implicitn hexadecimln selnou soustavu.
;
; Formty pro zadvn sla v rznch selnch soustavch:
;	B'01101101'	- prefix B, za kterm nsleduje binrn slo
;			  uzaven do jednoduchch uvozovek
;	O'123456'	- prefix O (t nula), za kterm nsleduje oktalov
;			  slo uzaven do jednoduchch uvozovek
;	D'6789'		- prefix D, za kterm nsleduje dekadick slo
;			  uzaven do jednoduchch uvozovek
;	H'23AB'		- prefix H, za kterm nsleduje hexadecimln slo
;			  uzaven do jednoduchch uvozovek
;	A'K'		- prefix A, za kterm nsleduje jeden ASCII znak
;			  uzaven do jednoduchch uvozovek
;	'K'		- ASCII znak uzaven do jednoduchch uvozovek
;	0x23AB		- prefix 0x, za kterm nsleduje hexadecimln slo
;	.1234		- prefix teka, za kterm nsleduje dekadick slo
;	01101101b	- sufix B za binrnm slem (nelze pout, je-li
;			  implicitn hexadecimln seln soustava)
;	123456o		- sufix O za oktalovm slem
;	6789d		- sufix D za dekadickm slem (nelze pout, je-li
;			  implicitn hexadecimln seln soustava)
;	23ABh		- sufix H za hexadecimlnm slem (zan slic)
;	$		- ukazatel aktuln adresy (poitadlo ORG)
;--------------------------------------------------------------------------
; Dalm pkazem je CONFIG. Slou k nastaven konfigurace procesoru.
;
;
;	CONFIG	0x3FF9	; ponechme odstaveno, nen poteba pout
;
;
; Procesory PIC umouj konfiguranm slovem volit typ osciltoru,
; zapnut asovae Watchdog, zapnut ochrany programu atd. Konfiguran
; slovo zvis na typu procesoru a na hardwarovm zapojen procesoru,
; proto je vhodnj pro nejtypitj volby pouvat symbolick pepnae
; konfigurace, kter na procesoru zvisl nejsou:
;	CONFIG	XT	- osciltor s krystalem XT (bn, volit pro P84-CPU)
;	CONFIG	LP	- osciltor s krystalem LP (nzkopkonov)
;	CONFIG	HS	- osciltor s krystalem HS (vysokorychlostn)
;	CONFIG	RC	- osciltor s RC lenem
;	CONFIG	WDT	- zapnout asova Watchdog (implicitn vypnuto)
;--------------------------------------------------------------------------
; Nsleduje blok CBLOCK.


	CBLOCK		; pro PIC16F84 je poten hodnota 0Ch
	ENDC		; CBLOCK je przdn, nepotebujeme dn registry


; Blok CBLOCK oznauje blok selnch konstant. Zpravidla ho pouvme
; k definici datovch promnnch. Zan dkem s pkazem CBLOCK a kon
; dkem s pkazem ENDC. Za pkazem CBLOCK me nsledovat seln
; parametr udvajc, kter prvn seln hodnota bude konstantm piazena.
; Bez udn parametru je poten hodnota nastavena na slo prvnho
; datovho registru nebo na koncov slo z minulho datovho bloku CBLOCK.
; Do dk mezi pkazy CBLOCK a ENDC je mon vpisovat nvt, kterm
; budou postupn piazovny seln hodnoty. Za nvtm a dvojtekou me
; bt seln parametr pedstavujc prustek dal hodnoty. Jinak je
; kad dal hodnota vy o 1.
;--------------------------------------------------------------------------
; Ped zatkem programu zpravidla uvdme pkaz ORG.


	ORG	0	; pi startu se program rozebhne od adresy 0


; Pkaz ORG slou k nastaven ukazatele, kam bude do pamti ukldn
; peloen program nebo data pro EEPROM. sti programu mohou bt
; uvedeny v libovolnm poad, kvli pehlednosti ovem upednostujeme
; bn sekvenn uspodn. Po startu procesoru se program rozebhne
; od adresy 0. Proto ped prvn instrukc zpravidla uvdme ORG 0.
; Pkaz ORG pouvme tak pi definici dat pro EEPROM (adresa 2100h).
;--------------------------------------------------------------------------
; Nvt zatku programu (nen nutn, je jen pipraveno pro pt pouit)


Start			; na stejnm dku za nvtm me bt pkaz


; Nvt v programu uvdme od zatku dku. Mohou obsahovat psmena,
; slice, podtrtko "_" a otaznk, na prvn pozici me bt jenom psmeno
; nebo podtrtko. Jsou rozliovna mal a velk psmena. Za nvtm
; me nsledovat dvojteka. Pkazy programu nesm zanat na zatku dku.
;--------------------------------------------------------------------------
; Volba banky registr 1


	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; ... instrukce BSF f,b nastav bit b registru f

;	bsf	STATUS,RP0 ; to je jin zpsob zpisu, pomoc symbol


; Nkter registry procesoru jsou rozdleny do vce bank. To znamen, e
; stejn adresa me pedstavovat rzn registry. Registry z banky 0 oznaujeme
; slem 0 a 7Fh, registry z banky 1 slem 80h a FFh. Chceme-li adresovat
; registry z banky 1, musme aktivovat banku registr 1, k registrm potom
; meme pistupovat bnm zpsobem (meme je adresovat slem 80h a FFh
; stejn tak jako 00h a 7Fh, pouije se pouze nich 7 bit sla
; registru). Nkter registry jsou spolen pro vechny banky, k tmto
; registrm meme pistupovat nezvisle na zvolen bance. slo zvolen
; banky nastavujeme ve stavovm registru bitem 5, u jinch typ procesor
; s vce bankami jet bitem 6. Bli informace ke stavovmu registru zskte,
; zvolte-li v prun npovd programu PicDev po prav stran oblast
; npovdy "Registry" a poloku "STATUS". Instrukce BSF nastav bit registru,
; v naem ppad bit 5 registru 3. Ble viz npovda "Instrukce".
;--------------------------------------------------------------------------
; Zapnut vech bit portu A jako vstupy


	movlw	00000b	; volba smru bit: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A
			; ... instrukce MOVWF f ulo registr W do registru f

;	movwf	TRISA	; jin zpsob zpisu - symbolick (TRISA = 85h)

;	clrf	TRISA	; jin zpsob - vechny bity budou jako vstup

; Registr TRISA je umstn v bance 1 procesoru (adresa 5 resp. 85h) a definuje
; bity portu A, kter budou pouity jako vstup a kter jako vstup. Nastavenm
; pslunho bitu registru TRISA na hodnotu 0 bude bit pouit jako vstup,
; nastavenm na hodnotu 1 bude pouit jako vstup. Zpisem bajtu s hodnotou 0
; jsme nastavili vechny bity portu A jako vstupn. Ve skutenosti m port A
; pouze 5 bit (A0 a A4), ale to nen na zvadu.
;--------------------------------------------------------------------------
; Volba banky registr 0


	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; ... instrukce BCF f,b vynuluje bit b registru f

;	bcf	STATUS,RP0 ; jin zpsob zpisu - symbolick


; Pokud jsme nastavili smr signl portu A, musme zapnout zpt banku
; registr 0, protoe nsledujc registr PORTA je pstupn pouze z banky 0.
;--------------------------------------------------------------------------
; Nastaven vstupnch signl portu A


	movlw	01111b	; volba hodnot vstupnch bit portu A
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven hodnot bit portu A
			; ... instrukce MOVWF f ulo registr W do registru f

;	movwf	PORTA	; jin zpsob zpisu - symbolick (PORTA = 5)


; Nastavenm bitu 0 portu A na hodnotu 1 zajistme, e na pslunm vvodu
; procesoru bude napt +5V. Nastavenm hodnoty 0 bude na vvodu napt 0V.
; Zapojme-li mezi vvod a zem LED diodu (svtc dioda) pes omezujc odpor
; nap. 330 ohm (anoda na vvod), meme nastavenm hodnoty 1 LED diodu
; rozsvtit. Zapojme-li naopak LED diodu mezi vvod a +5V (anoda na +5V),
; rozsvtme LED diodu naopak nastavenm hodnoty 0. Upozornn - port A4
; je vstup s otevenm kolektorem. To znamen, e vvod sepn proud pouze
; proti zemi, take na tomto vvodu rozsvtme LED diodu pouze pi zapojen
; proti +5V a ne pi zapojen proti 0V.
;--------------------------------------------------------------------------
; Zablokovn programu


	goto	$	; takto vytvome nekonenou smyku
			; ... instrukce GOTO sko na zadanou adresu

;	SLEEP		; jin monost zastaven (nesm bt zapnut WDT)


; Od procesoru toho moc nechceme. Pokud zapnul LED na vstupech portu A,
; meme ho nechat zastavit. To meme udlat nap. nekonenou smykou,
; tj. instrukce GOTO skkajc sama na sebe. Jinou monost je instrukce
; SLEEP, kter uvede procesor do stavu klidu - vypne osciltor a
; zastav chod programu, pitom se sn pkon procesoru, zstanou pouze
; aktivovan vstupn porty. V tomto ppad nesm bt zapnut asova
; Watchdog (WDT), ten by toti po chvilce procesor zase probudil a
; nechal by ho pokraovat dle bhvkam.
;--------------------------------------------------------------------------
; Peklad a uloen programu
;
; V prbhu psan programu je uiten obas spustit peklad programu.
; Dozvte se tak vas o chybch a peklepech v programu a mete si
; tak prohldnout vpis programu. Peklad programu vyvolte tlatkem
; "Peklad". Krom kontrolnho pekladu provdjte tak asto ukldn
; vaeho programu tlatkem "Uloit" nebo "uloit Jako". Chcete-li program
; penst do prosted jinho programovacho nstroje, mete program
; exportovat do HEX tvaru - ve volb "uloit Jako" zvolte typ soubor
; "kd programu (*.hex)".
;--------------------------------------------------------------------------
; Programovn ipu
;
; Mte-li program pipraven, mete vyvolat okno pro programovn ipu.
; Pedtm ovem muste pipojit jednotku P84-CPU k nktermu z port COM
; a zvolit sprvn port COM v programu PicDev (pp. pout autodetekci).
; V okn programovn se dozvte informaci o potu zaplnnch slov programu
; a o potu obsazench bajt pamti EEPROM, dle zde mete zvolit typ
; osciltoru (pro jednotku P84-CPU ponechejte vdy volbu XT), zvolit
; pouit asovae Watchdog (automatick reset i automatick buzen),
; pouit asovae Power-up (prodleva pro rozbh napjecho napt)
; i program uzamknout proti ten. Tlatkem "Programovat" vyvolte
; odesln programu do pamti ipu. Pokud je program v podku, ml
; by se po dokonen zpisu rozebhnout, tj. pipojen LED by se mla
; rozsvtit. Mte-li pipojenu jednotku P84-UNI, rozsvt se 1 LED dioda
; uprosted dole a vechny 3 LED diody po prav stran.
;--------------------------------------------------------------------------
