;**************************************************************************
; LEKCE02.ASM					(c) Ing. Miroslav Nmeek
;
; 2. lekce programovn procesor PIC - asov prodleva instrukcemi
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo LED dioda pipojen k P84-CPU mezi signl A0 (anoda) a zem
;**************************************************************************
;
; Jednou ze zkladnch poteb pi programovn jednoipovch procesor
; je vytven ekacch prodlev, s jejich pomoc asujeme innost procesoru.
; Prvn metoda, s kterou se seznmme, vychz ze skutenosti, e kad
; instrukce procesoru je provdna po uritou znmou dobu.
;
; Zkladn mrnou jednotkou innosti je strojov takt. Bhem jednoho strojovho
; taktu procesor provd jednu instrukci a souasn pipravuje nsledujc
; instrukci. Vtina instrukc trv pesn jeden strojov takt. Pouze nkter
; instrukce, kter naru plynul tok provdn instrukc (skokov instrukce,
; nesplnn podmnky) trvaj 2 takty, protoe procesor nevyuije pipravenou
; instrukci a mus natat jinou.
;
; Jeden strojov takt trv 4 hodinov impulsy. Jednotka P84-CPU pouv
; nejobvyklej typ krystalu pro procesor PIC16F84 - krystal 4 MHz.
; Strojov takt m v tomto ppad frekvenci 1 MHz, to znamen,
; e jeden strojov takt trv pesn 1 mikrosekundu (v textu pouvme
; pro mikrosekundu oznaen us, abycho se vyhnuli znaku mikro).
; Tak se nm bude velmi snadno potat asovn procesoru, protoe
; kad instrukce trv 1 pop. 2 us.
;
;--------------------------------------------------------------------------
; Vytvome si jednoduch program, kter rozblik LED diodu na vvodu A0
; v jednosekundovch intervalech. Na zatku ponechme bn inicializan
; pkazy, v datovch promnnch si pipravme 2 promnn jako tae.
; Namsto definice pomoc CBLOCK mete adresovat registry ta pomoc
; absolutnch sel (nap. 10h a 11h) nebo pout piazen pomoc EQU.

	RADIX	DEC	; pouijeme dekadickou selnou soustavu
			; Dekadick soustava je v PicDev implicitn, nkter
			; jin pekladae pouvaj implicitn HEX soustavu

	CBLOCK		; vchoz hodnota bloku CBLOCK pro PIC16F84 je 0Ch,
			; u jinch procesor to me bt nap. 20h.
			; Pozor pi pevodu z jinch peklada - nkter
			; pekladae pedpokldaj implicitn adresu 0

Citac1			; ta pro vnj ekac smyku
Citac2			; ta pro stedn ekac smyku
Citac3			; ta pro vnitn ekac smyku
	ENDC

	ORG	0	; pi startu se program rozebhne od adresy 0

Start			; nvt oznauje zatek programu (nen zde nutn)

; ----- nastaven smru bit portu A jako vstup
; Registr TRISA slouc k nastaven smru bit portu A le v bance 1,
; proto ped jeho nastavenm musme nastavit banku 1. Banku 1 nastavme
; zapnutm bitu 5 (RP0) registru 3 (STATUS). Ze zatku u jednoduch
; program uvdjte v instrukcch pmo sla registr a bit, abyste si
; utvoili pedstavu o rozloen registr v pamti. A po pr prvnch
; programech zante pouvat symboly, dky kterm si nebudete muset
; sla registr pamatovat.

	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; ... instrukce BSF f,b nastav bit b registru f

	movlw	10000b	; volba smru bit: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A (registr TRISA)
			; ... instrukce MOVWF f ulo registr W do registru f

	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; ... instrukce BCF f,b vynuluje bit b registru f


; Po pepnut na banku 1 jsme nastavili ni 4 bity portu A jako vstupy.
; Je-li pipojeno vce LED, meme nechat blikat vechny. Pipojte k jednotce
; P84-CPU extern zdroj, napjen z portu COM by tolik LED neuivilo.
; Po nastaven smru bit portu A zapneme zpt banku 0, protoe banku 1
; ji v naem programu nebudeme potebovat. Vimnte si, e nenastavujeme
; hodnotu bit portu A. Vstupy budeme peklpt mezi hodnotami 0 a 1 a nezle
; nm na tom, zda po startu zane blikn rozsvcenm nebo tmou.

; ----- inicializace ta smyek
; Na tuto adresu bude provdn skok po ukonen vech smyek. Budeme pouvat
; 3 smyky do sebe vnoen. Prodleva zajitn pomoc instrukc se bn pouv
; pouze pro krtk asov seky a proto pozdji nebudeme nuceni pouvat tolik
; vnoench cykl.

Blikej	movlw	3	; vchoz hodnota vnjho tae 1 do registru W
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	Citac1	; uloen hodnoty do vnjho tae Citac1
			; ... instrukce MOVWF f ulo registr W do registru f

	movlw	137	; vchoz hodnota stednho tae 2 do registru W
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	Citac2	; uloen hodnoty do stednho tae Citac2
			; ... instrukce MOVWF f ulo registr W do registru f

; Hodnoty registr si vypotme v nsledujc sekci smyek. Vypustili jsme
; nastaven vnitn smyky, protoe je to pro ns ji zbyten velk pesnost.
; Vnitn smyka pouije jako vchoz hodnotu posledn stav tae, tedy 0.

; ----- ekac smyky
; Nsleduj ekac smyky, kter zajist poadovanou prodlevu. Nejdve si
; vysvtlme, jak smyky funguj. Instrukce DECFSZ f,d sn hodnotu zadanho
; registru f o 1 (operaci kme dekrementace). Je-li druh parametr 
; instrukce d=1, vsledek operace se ulo zpt do registru f. Pokud by byl
; parametr d=0, vsledek by se uloil do registru W a obsah registru f by se
; nezmnil. V pevn vtin ppad budeme chtt, aby se obsah registru tae
; mnil, budeme tedy vtinou pouvat parametr 1. Nen-li vsledek dekrementace
; slo 0, provede se nsledujc instrukce, uveden za instrukc DECFSZ.
; V pevn vtin bude nsledovat instrukce GOTO, kter zajist opakovn
; prchodu smykou, ale me nsledovat jakkoliv jin instrukce. Je-li vsledek
; dekrementace 0, nsledujc instrukce se neprovede a program pokrauje a
; dal instrukc. Takto pracuj vechny 3 vnoen smyky. Je-li nsledujc
; smykou znovu zahjeno tn vnoenho cyklu, je cyklus zahjen s vchoz
; hodnotou 0 (od minulho tn), kterou dekrementuje prvnm prchodem na
; hodnotu 255. Opakovn vnitnho cyklu tedy provede 256 prchod.

Cekej	decfsz	Citac3,1 ; dekrementace tae 3 - vnitn smyka (1 znamen,
			; e vsledek dekrementace bude uloen zpt do tae 3),
			; po dosaen 0 pesko dal instrukci
	goto	Cekej	; skok na nov prchod smykou; po dosaen 0 tae se
			; tato instrukce neprovede

	decfsz	Citac2,1 ; dekrementace tae 2 - stedn smyka, vsledek do tae 2
	goto	Cekej	; skok na nov prchod smykou; po dosaen 0 tae se neprovede

	decfsz	Citac1,1 ; dekrementace tae 1 - vnj smyka, vsledek do tae 1
	goto	Cekej	; skok na nov prchod smykou; po dosaen 0 tae se neprovede

; asovn vnitnho cyklu
; Jak jsme si ji ekli, proveden kad instrukce trv 1 strojov takt, pouze
; v ppad peruen souvislho toku provdn instrukc to budou 2 takty. Instrukce
; DECFSZ vnitn smyky bude tedy pi bnm prchodu (opakovn) trvat 1 takt. Pi
; bnm prchodu se provede nsledujc instrukce GOTO, kter zajist opakovan
; prchod smykou. Jedn se o peruen provdn instrukc, take instrukce
; trv 2 strojov takty. Kad opakovan prchod smykou tedy zabere celkem 3 strojov
; takty. Vchoz hodnotou vnitn smyky je hodnota 0. Opakovanch prchod smykou
; je tedy 255. Pi poslednm prchodu instrukce DECFSZ zjist, e nsledujc instrukce
; se nem provst a proto ji pesko. To zabere 2 strojov takty. Instrukce se v tto
; chvli chov tak, jakoby nahradila kd nsledujc instrukce instrukc NOP, kter
; nic neprovede. Kad obsluha vech prchod vnitn smykou zabere 3*255 + 2 = 767
; takt, pi asovn 1 us na takt to je 767 us.
;
; asovn stednho cyklu
; Pi jednom prchodu stedn smykou se provedou nejdve vechny prchody vnitn
; smykou a potom instrukce DECFSZ a GOTO (pi opakovanm prchodu 3 takty, pi
; poslednm 2). Ozname-li si vchoz hodnotu tae 2 symbolem X, bude celkov
; doba vech prchod stedn smykou (767+3)*(X-1) + 767 + 2 = 770*X - 1 (pro
; hodnotu X=0 musme potat hodnotu X=256).
;
; Vpoet vchozch hodnot ta
; V naem programu budeme chtt, aby LED dioda blikala v sekundovch intervalech.
; Stav LED diody budeme peklpt, take potebujeme asovou prodlevu 0.5 sekundy.
; Pro celkovou dobu 0.5 sekundy potebujeme 500000 strojovch takt trvajcch 1 us.
; Pi vpotech zanedbme dobu provdn instrukc vnjho cyklu a dobu provdn
; ostatnch obslunch innost, jako je peklopen stavu portu i inicializace
; novch hodnot ta. Vyjdeme-li jenom ze stedn smyky, vyjde nm poadovan
; hodnota X = 649, co by ml bt poet prchod stedn smykou. Poet prchod
; smykou je omezen na maximln 256. Potebujeme tedy 2 prchody stedn smykou
; s vchoz hodnotou 256 (tedy 0), zstane nm X=137. To znamen, e vchoz hodnota
; vnjho cyklu bude 3 a stednho cyklu 137. Pi prvnm prchodu vnjm cyklem se
; provede stedn cyklus pro hodnotu 137, tj. 105489 takt. Pi ostatnch dvou
; cyklech se provede stedn cyklus pro hodnotu 256 (tj. 0), tedy 197119 takt.
; Celkov doba prchodu smykami je 105489+2*197119=499727. Pokud by nm zleelo
; na pesnosti, mohli bychom nastavit jet vchoz hodnotu vnitnho tae a
; zapotat do asovn i doby provdn vnj smyky a ostatnch instrukc.
	
; ----- indikace LED
; Nsledujc instrukce COMF provede peklopen vech bit portu A. To uskuten
; tak, e nate stav bit portu, invertuje jejich hodnotu a nov stav pole
; zpt na port A. Pestoe jsou bity portu A nastaveny jako vstup, je mon
; jejich stav st jako vstup. teme samozejm stav, kter jsme v minulm
; prchodu na vstup nastavili, take te meme snadno stav zmnit.

	comf	5,1	; inverze bit portu A; druh parametr "1" udv, e
			; vsledek operace inverze je uloen zpt do registru,
			; tedy na port A. Pokud by parametr ml hodnotu 0,
			; vsledek by se uloil do W (LED by neblikaly)

; Poznmka: ten bit nastavench jako vstupy se realizuje tenm stavu
; na pinech portu. Bn tato hodnota odpovd nastaven hodnot vstupu,
; me se ale liit nap. u pinu RA4, kter pouv vstup s otevenm
; kolektorem. Nen-li pes odpor spojen s kladnm naptm, me na nm
; bt pi ten trvale hodnota 0.

; ----- opakovn
; Posledn instrukc programu bude GOTO, kter zajist opakovn blikn
	
	goto	Blikej


; ----- start programu
; Vylete-li nyn hotov program do procesoru, mla by se vstupn LED (nebo
; vce LED) rozblikat s 1-sekundovou periodou. Mete si vzt hodinky se
; sekundovkou a ovit, zda nae vpoty byly pesn.
