;**************************************************************************
; LEKCE04.ASM					(c) Ing. Miroslav Nmeek
;
; 4. lekce programovn procesor PIC - asov prodleva peruenm
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;	nebo 4 LED diody pipojen k P84-CPU mezi signly A0-A3 a 0V (zem)
;	     a 1 LED dioda pipojen k P84-CPU mezi signl B4 a 0V (zem)
;**************************************************************************
;
; V pedelch lekcch jsme provdli asovn zastavenm vykonvn
; programu po dobu ekn na zmnu stavu. Ve vtin aplikac zejm
; budeme vyadovat, aby procesor vykonval njakou svou jinou innost
; a nezdroval se eknm na bliknut indikan LED. K tomu je uren
; vnitn asova Timer0.
;
; Timer0 je 8-bitov ta/asova. V reimu tae pot impulsy na
; vstupu RA4/T0CKI. V reimu asovae pot instrukn cykly procesoru.
; To znamen, e se jeho hodnota pi kad instrukci zv o 1, lze
; tedy vyut jako vnitn hodiny. Ped modul Timer0 je mon pedadit
; dliku (s kterou jsme se seznmili v minul lekci u obvodu Watchdog)
; s dlicm pomrem 1/2 a 1/256. Po dosaen maximln hodnoty 255
; ta dal inkrementac petee na hodnotu 0. Peteen modulu Timer0
; me vyvolat peruen.
;
; Peruen je odskok procesoru z bn innosti na zvltn obslunou
; proceduru. Peruen me bt vyvolno peteenm tae/asovae
; Timer0, nktermi vstupy portu B atd. Pi peruen je proveden odskok
; programu na adresu 4, kde zan obsluha peruen. Pi peruen je
; uschovna nvratov adresa do zsobnku adres. Bhem obsluhy peruen
; je automaticky zakzna obsluha dalho ppadnho peruen a do
; okamiku ukonen obsluhy peruen, kdy je dal peruen opt
; povoleno a program se navrt do msta, odkud byl peruen.
;--------------------------------------------------------------------------
; Vytvome si opt jednoduch "bliktko" ovldan peruenm od
; asovae. O obsluhu indikanch LED se bude start obsluha peruen,
; zatmco hlavn program bude vykonvat svou hlavn innost. Protoe
; jde jen o ukzkov program, nebude innost hlavnho programu pli
; smyslupln. Pouze ho nechme sledovat innost indikanch LED a
; peklpt doplkovou indikan LED.

; ----- zen pekladae, datov promnn

	RADIX	DEC	; pouijeme dekadickou selnou soustavu

	CBLOCK
TempW			; schova registru W pi peruen
TempS			; schova registru STATUS pi peruen
Counter			; ta peruen pro jeden krok indikace
Flags			; pomocn pznaky
			;	bit 0: posun indikace je smrem nahoru
	ENDC


; ----- start programu
; Na rozdl od pedelch lekc te ji program neme zanat na
; adrese 0, protoe na adresu 4 je provdn odskok na obsluhu peruen.
; Zpravidla umisujeme na adresy 0 a 4 instrukce skoku na zatek
; obsluh programu a peruen. Od adresy 5 obvykle umisujeme tabulky,
; o tch a v dalch lekcch.

	ORG	0	; Pestoe je pekladaem adresa 0 ji zajitn,
			; zpravidla sem tento pkaz vkldme, abychom
			; si jednoznan urili zatek programu.

	goto	Start	; skok na start hlavnho programu

	ORG	4	; To je adresa odskoku pi peruen. Peklada
			; ponech za posledn instrukc "goto Start"
			; 3 slova s hodnotou 3FFFh a pokrauje
			; v pekladu od adresy 4. Namsto tohoto povelu
			; jsme stejn tak mohli uvst nap. 3 x NOP.

	goto	Inter	; Skok na obsluhu peruen. Obsluha peruen
			; by mohla nsledovat ihned, vhodnj je
			; pracovat s n jako s bnm podprogramem,
			; kter meme umstit do libovolnho msta
			; v programu. Navc obas potebujeme doplnit
			; na nsledujc adresy tabulky.

;--------------------------------------------------------------------------
;	Start hlavnho programu
;--------------------------------------------------------------------------

; ----- nastaven smru bit port A a B
; Na tomto mst zan start hlavnho programu. Nejdve
; nastavme smr bit port A a B. Banku 1 nechme aktivn.

Start	bsf	3,5	; zapnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 1)
			; ... instrukce BSF f,b nastav bit b registru f

	movlw	10000b	; volba smru bit portu A: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven bit portu A (registr TRISA)
			; ... instrukce MOVWF f ulo registr W do registru f

	movlw	11100000b ; volba smru bit portu B: 0=vstup, 1=vstup
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	6	; nastaven bit portu B (registr TRISB)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nastaven asovae Timer0
; Pi vpotech asovn jsme se rozhodli pro nsledujc variantu:
; Potejme s krystalem 4 MHz. Jeden takt procesoru bude trvat 1 us.
; Peddliku asovae nastavme na dlic pomr 1/128. asova bude
; tedy inkrementovn kadch 128 us. Pi kadm peruen budeme nastavovat
; ta na hodnotu 217. ta vyvol peruen po 39 inkrementacch
; (doplnk do stavu peteen 256). Perioda peruen bude 39*128=4992 us.
; Pomoc opodnho zpisu do registru asovae o 8 us (8 instrukc)
; upesnme periodu na 5000 us, tedy 5 ms, co odpovd frekvenci 200 Hz.
; Pomoc tae s periodou 25 prodloume periodu na 125 ms, tedy 8x za sekundu.

	movlw	6+bit7	; dlika TMR0 1/128, pipojena k TMR0, zdrojem hodiny
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	1	; nastaven OPTION registru asova
			; ... instrukce MOVWF f ulo registr W do registru f

	bcf	3,5	; vypnut bitu RP0 (bit 5) registru STATUS (slo 3)
			; (tm zapneme banku registr slo 0)
			; ... instrukce BCF f,b vynuluje bit b registru f

; K nastaven asovae Timer0 slou opt registr OPTION, kter m adresu 1
; a je pstupn z banky 1. Dliku jsme nastavili na pomr 1/128,
; pipojili jsme ji k modulu Timer0 (bit3 PSA vynulovn) a jako zdroj
; tn zvolili vnitn hodiny - instrukn cyklus (bit5 T0CS vynulovn).
; Vypnuli jsme pull-up portu B, tedy podprn kladn proud tekouc do vstup.
; Zvrem jsme zvolili banku registr 0, banku 1 ji dle nebudeme potebovat.

; ----- inicializace tae peruen
; Abychom doshli poadovanch 125 ms, pouijeme pomocn ta Counter,
; kter zajist obsluhu indikace po kadm 25. prchodu. Proto nastavme
; nyn jeho vchoz hodnotu, aby prvn posun indikace nastal po zapnut
; procesoru po sprvnch 125 ms a ne a nap. po 1 sekund (kdyby byl = 0)

	movlw	25	; hodnota pro pomocn ta, k dosaen doby 125 ms
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	Counter	; nastaven pomocnho tae peruen
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- inicializace pznak
; Pro obsluhu indikace budeme pouvat pomocnou promnnou obsahujc
; v bitu 0 pznak, e indikan bit se posouv smrem k vym bitm.

	movlw	bit0	; pznak, e posun indikace je smrem nahoru
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	Flags	; nastaven pomocnho registru pznak
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nastaven port A a B
; Obsluha peruen bude posouvat bit s hodnotou 1 cyklicky mezi bity 0 a 3
; a to na portu A i na portu B, proto vchozm stavem bude bajt s hodnotou 1

	movlw	bit0	; obsluha peruen bude posouvat 1 mezi bity 0 a 3
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	5	; nastaven port A (registr PORTA)
	movwf	6	; nastaven port B (registr PORTB)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- inicializace obsluhy peruen
; K zen peruen slou registr INTCON, jeho adresa je 0Bh (dekadicky 11)
; a je pstupn ze vech bank. Budeme pouvat pouze peruen pi peruen
; od asovae, proto nastavme bit 5 (INTE) a povolme globln (celkov)
; peruen nastavenm bitu 7 (GIE). Ostatn bity ponechme vynulovan.

	movlw	bit5+bit7 ; povoleno peruen od TMR0 a globln peruen
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	0Bh	; nastaven registru peruen INTCON
			; ... instrukce MOVWF f ulo registr W do registru f
	
; Od tohoto okamiku je ji systm peruen inicializovn a je vyvolvno
; kadch 5 ms peruen od asovae.

; ----- obsluha hlavnho programu
; Hlavn program bude ve skutenost provdt pouze pomocnou indikan
; innost. Obsluhu peruen ponechme posouvat svtc LED diodu mezi
; bity A0 a A3 tam a zpt. Hlavn program bude sledovat bity A0 a A3
; a podle jejich stavu peklpt bit B4 (indikan LED na portu B).

Vypni	btfss	5,0	; doshla indikan 1 bitu 0 portu A?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit nastaven, dal instrukci pesko

	goto	Vypni	; ekn na zapnut LED na bitu 0 portu A

	bcf	6,4	; vypnut indikan LED - bit 4 portu B
			; ... instrukce BCF f,b vynuluje bit b registru f

Zapni	btfss	5,3	; doshla indikan 1 bitu 3 portu A?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit nastaven, dal instrukci pesko

	goto	Zapni	; ekn na zapnut LED na bitu 3 portu A

	bsf	6,4	; zapnut indikan LED - bit 4 portu B
			; ... instrukce BSF f,b nastav bit b registru f

	goto	Vypni	; nov cyklus - ekn na vypnut indikan LED

;--------------------------------------------------------------------------
;	Obsluha peruen
;--------------------------------------------------------------------------

; ----- schova registr W a STATUS
; Kad obsluha peruen by mla zanat schovou registr. Nevme toti,
; z jakho msta v hlavnm programu jsme odskoili. V obsluze peruen
; pravdpodobn budeme pouvat registr W i stavov registr (mnoho instrukc
; stavov registr modifikuje). Hlavn program mohl registry zrovna pouvat.

Inter	movwf	TempW	; schova registru W
			; ... instrukce MOVWF f ulo registr W do registru f

	swapf	3,0	; naten STATUS registru do W
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	movwf	TempS	; schova STATUS registru
			; ... instrukce MOVWF f ulo registr W do registru f

; K schov registr W a STATUS pouijeme dv datov promnn, TempW a TempS.
; Registr W ulome do pomocnho registru TempW bnm zpsobem, instrukc MOVWF.
; U registru STATUS (adresa 3) je situace ponkud obtnj. Tm vechny
; instrukce pro naten obsahu registru toti mn obsah stavovho registru.
; Pro schovu registr bez ovlivnn stavovho registru je urena instrukce
; SWAPF, kter zamn tetrdy (tveice) bit registru. Pro byla vrobcem
; zvolena prv instrukce SWAPF? Kvli minimalizaci kdu. Instrukce SWAPF
; pat mezi instrukce s nejmen etnost pouit a proto bude nejmn
; vadit nutnost dodatenho testu nulov hodnoty registru. Nen na zvadu,
; e u natenho registru dojde k zmn tetrd. Pi navrcen stavovho
; registru bude instrukce pouita znovu a tm dojde k navrcen pvodnho obsahu.

; ----- inicializace tae TMR0
; Jak jsme vypotali ve, pro sprvn asovn budeme inicializovat
; registr tae na hodnotu 217, m zajistme periodu peruen 5 ms.
; Zpisem do registru TMR0 zajistme i vynulovn peddliky.
; Jet jsme si ekli, e budeme potebovat doplujcch 8 us, kter
; nm chyb do pesn hodnoty 5 ms. To by mla bt doplujc prodleva
; od okamiku vyvoln peruen (tj. ta doshl stavu 0) do okamiku
; nov inicializace tae vetn vynulovn peddliky. Jak si mete
; vimnout, tuto doplujc prodlevu jsme zajistili pedchzejcmi
; instrukcemi pro obsluhu peruen: Instrukce "GOTO Inter" (2 takty),
; schova registr W a STATUS (3 takty), instrukce pro nastaven
; tae TRM0 vetn doplujcho zastaven tae po zpisu (3 takty),
; co in onch 8 us. Ve skutenosti jsme jet nepotali njakou us
; na aktivaci peruen, ale to je te pro ns ji dostaten vysok pesnost.

	movlw	217	; nov hodnota tae do W (doplnk do 256 je 39)
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	1	; nastaven registru asovae TMR0 (mus bt banka 0)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nulovn pznaku peruen
; ta TMR0 pi svm peteen nastavil bit T0IF (bit 2) registru INTCON
; (adresa 0Bh) jako poadavek procesoru, aby provedl peruen. Obsluha
; peruen mus tento pznakov bit vynulovat pro indikaci, e provedla
; obsluhu peruen. To je dleit pi vce typech peruen, aby procesor
; vdl, kter peruen jet nebyla obslouena. Pokud tento bit nevynulujete,
; procesor po ukonen obsluhy peruen vyvol peruen ihned znovu.
; (Poznmka: Nepokouejte se vyvolat peruen nastavenm bitu T0IF,
; tento bit je mon programem pouze nulovat)

	bcf	0Bh,2	; nulovn pznaku peruen
			; ... instrukce BCF f,b vynuluje bit b registru f

; ----- tn jednoho kroku indikace
; Obsluhu indikace budeme provdt po 125 ms. Protoe peruen je vyvolvno
; kadch 5 ms, pouvme pomocn ta Counter, tajc 25 prchod
; obsluhou peruen. Po dosaen 0 nastavme ta opt na stav 25.

	decfsz	Counter,1 ; tn pomocnho tae Counter
			; ... Instrukce DECFSZ f,d sn hodnotu registru f
			;     o 1. Je-li druh parametr d=1, ulo vsledek
			;     zpt do registru f (pro d=0 by byl vsledek
			;     uloen do W). Doshl-li vsledek operace 0,
			;     pesko se nsledujc instrukce.

	goto	IntRet	; skok na konec obslkuhy peruen, pokud
			; pomocn ta jet nedoshl jhodnoty 0

	movlw	25	; nov hodnota pro ta, ta ji doshl 0
			; ... instrukce MOVLW ulo do W zadanou konstantu

	movwf	Counter	; nastaven nov hodnoty pomocnho tae
			; ... instrukce MOVWF f ulo registr W do registru f


; ----- rozlien smru posunu indikace
; Pomocn registr pznak Flags indikuje bitem 0, zda je posun indikace nahoru

	btfss	Flags,0	; test bitu 0 pznak, zda je smr nahoru
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit b nastaven, dal instrukce se pesko

	goto	Down	; bit nen nastaven, bude posun dol

; nsleduje obsluha posunu nahoru

; ----- posun obsahu port smrem nahoru
; Budeme rotovat obsah portu A o jeden bit smrem vlevo instrukc RLF.
; Bn bychom mli nejdve vynulovat bit C (Carry) registru STATUS,
; protoe tento bit se pen do bitu 0 rotovanho bajtu. Protoe ale
; vme, e budeme chtt vsledek maskovat, meme stav bitu C ignorovat.

	rlf	5,0	; posun obsahu portu A vlevo, vsledek do W
			; ... instrukce RLF f,d posune bity registru f
			;     o 1 pozici vlevo (k vym bitm), do
			;     bitu 0 pitom vlo hodnotu pznaku C.
			;     Je-li druh parametr d=0, vsledek ulo
			;     do registru W (pro d=1 vsledek ulo do f)

	andlw	1110b	; maskovn posouvanch bit
			; ... instrukce ANDLW provede bitovou operaci
			;     AND bit registru W s konstantou (tj. ponech
			;     nastaveny pouze bity na tch pozicch, kde
			;     je v konstant bit s hodnotou 1)

; Vsledek operace posunu jsme ponechali v registru W, protoe ho
; chceme jet dle zpracovvat. Po posunu jsme bajt zamaskovali tak,
; e jsme ponechali pouze bity na pozicch 1, 2 a 3. Na pozici 0 jsme
; tak odstranili penesen bit C (Carry). Pokud se indikan bit penesl
; na pozici 4, byl tak zamaskovn, vsledkem operace by v tom ppad
; byla hodnota 0. To vyuijeme k detekci zmny smru posunu.

; ----- zmna posunu na smr dol
; Pokud byl vsledek pedel operace slo 0 (tj. indikan bit
; petekl na pozici 4), nastav procesor stavov bit Z (bit 2) registru
; STATUS (adresa 3). V tom ppad zmnme posun na smr dol
; a znovu vytvome indikan bit na pozici 3.

	btfss	3,2	; je nastaven bit Z registru STATUS?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit b nastaven, dal instrukce se pesko

	goto	IntNext	; nen-li vsledek operace 0, pokrauje se dle

	bcf	Flags,0	; vynulovn pznakovho bitu 0, smr bude dol
			; ... instrukce BCF f,b vynuluje bit b registru f

	iorlw	bit3	; nastaven indikanho bitu v registru W na pozici 3
			; ... instrukce IORLW provede bitovou operaci
			;     OR bit registru W s konstantou (tj. nastav
			;     bity na tch pozicch, kde je v konstan 1)

	goto	IntNext	; pokraovn spolenou obsluhou pro oba smry

; ----- posun obsahu port smrem dol
; Budeme rotovat obsah portu A o jeden bit smrem vpravo instrukc RRF.

Down	rrf	5,0	; posun obsahu portu A vpravo, vsledek do W
			; ... instrukce RRF f,d posune bity registru f
			;     o 1 pozici vpravo (k nim bitm), do
			;     bitu 7 pitom vlo hodnotu pznaku C.
			;     Je-li druh parametr d=0, vsledek ulo
			;     do registru W (pro d=1 vsledek ulo do f)

	andlw	111b	; maskovn posouvanch bit
			; ... instrukce ANDLW provede bitovou operaci
			;     AND bit registru W s konstantou (tj. ponech
			;     nastaveny pouze bity na tch pozicch, kde
			;     je v konstant bit s hodnotou 1)

; Vsledek operace posunu jsme ponechali v registru W, protoe ho
; chceme jet dle zpracovvat. Po posunu jsme bajt zamaskovali tak,
; e jsme ponechali pouze bity na pozicch 0, 1 a 2. Pokud se indikan
; bit penesl mimo registr (do pznaku C), vsledkem operace by byla
; hodnota 0. To vyuijeme k detekci zmny smru posunu.

; ----- zmna posunu na smr nahoru
; Pokud byl vsledek pedel operace slo 0 (tj. indikan bit
; petekl ven z bitu 0), nastav procesor stavov bit Z (bit 2) registru
; STATUS (adresa 3). V tom ppad zmnme posun na smr nahoru
; a znovu vytvome indikan bit na pozici 0.

	btfss	3,2	; je nastaven bit Z registru STATUS?
			; ... instrukce BTFSS f,b testuje bit b registru f;
			;     je-li bit b nastaven, dal instrukce se pesko

	goto	IntNext	; nen-li vsledek operace 0, pokrauje se dle

	bsf	Flags,0	; nastaven pznakovho bitu 0, smr bude nahoru
			; ... instrukce BSF f,b nastav bit b registru f

	iorlw	bit0	; nastaven indikanho bitu v registru W na pozici 0
			; ... instrukce IORLW provede bitovou operaci
			;     OR bit registru W s konstantou (tj. nastav
			;     bity na tch pozicch, kde je v konstan 1)

; ----- pidn stavu bitu 4 portu B
; Hlavn program bit 4 portu B pouv jako pomocnou indikaci. Proto
; musme stav bitu uchovat. Z pedelch operac je bit 4 registru W
; vynulovn, proto posta test bitu 4 portu B a jeho nastaven
; v registru W, pokud je nastaven.

IntNext	btfsc	6,4	; test bitu 4 portu B
			; ... instrukce BTFSC f,b testuje bit b registru f;
			;     je-li bit b vynulovn, dal instrukce se pesko

	iorlw	bit4	; nastaven bitu 4 registru W, je-li nastaven v portu B
			; ... instrukce IORLW provede bitovou operaci
			;     OR bit registru W s konstantou (tj. nastav
			;     bity na tch pozicch, kde je v konstan 1)

; ----- zpis nov hodnoty do port A a B
; Vslednou novou hodnotu port nyn odeleme na porty A i B. Nevad, e
; na port A odeslme i bit 4 portu B, pslun bit portu A je nastaven
; jako vstup, take bit bude ignorovn.

	movwf	5	; nastaven port A (registr PORTA)
	movwf	6	; nastaven port B (registr PORTB)
			; ... instrukce MOVWF f ulo registr W do registru f

; ----- nvrat registr W a STATUS
; Stejn jako jsme na zatku obsluhy peruen uchovali registry W
; a STATUS, musme je nyn navrtit. Nejdve navrtme STATUS registr.
; Opt pouijeme instrukci SWAPF, kter navrt tetrdy registru zpt
; na pvodn msta. Pi navracen registru W nememe pout bn
; naten z registru, protoe bychom ovlivnili STATUS registr. Pouijeme
; proto opt instrukci SWAPF a to tak, e nejdve v pechodnm registru
; zamnme tetrdy bajtu, m si bajt trochu pedpipravme, dal
; instrukc SWAPF bajt z registru nateme a souasn navrtme tetrdy
; bajtu na pvodn sprvn msta.

IntRet	swapf	TempS,0	; pvodn STATUS registr k navrcen
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	movwf	3	; nvrat pvodnho STATUS registru
			; ... instrukce MOVWF f ulo registr W do registru f

	swapf	TempW,1	; pedpprava pvodnho registru W
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	swapf	TempW,0	; nvrat pvodnho W registru
			; ... instrukce SWAPF f,d zamn tetrdy registru f,
			;     pro d=0 ulo vsledek do W, pro d=1 zpt do f

	retfie		; nvrat z obsluhy peruen
			; ... instrukce RETFIE navrt procesor na msto,
			;     odkud bylo peruen vyvolno, souasn opt
			;     povol pznak globlnho peruen GIE

; ----- start programu
; Vylete program do procesoru. Na jednotce P84-UNI se rozblikaj vechny
; indikan LED diodu tm zpsobem, e diody po lev a prav stran "cvi"
; symetricky proti sob nahoru a dol, zatmco dioda uprosted blik
; v sekundovch intervalech (v zvislosti na smru posunu indikace po
; stranch). Vezmete-li si stopky, mla by stedn dioda blikat
; s absolutn pesnost po 1 sekund.
