;**************************************************************************
; LEKCE07.ASM					(c) Ing. Miroslav Nmeek
;
; 7. lekce programovn procesor PIC - EEPROM
;--------------------------------------------------------------------------
; Poadovan hardware:
;	univerzln jednotka P84-UNI
;
; Program demonstruje pouit pamti EEPROM pi ukldn konfigurace.
; Program pehrv 2 melodie. Pehrvan melodie se vol tlatky A a B,
; tlatko C pehrvn vypne. Tlatko je nutn chvli podret, reaguje
; pouze pi zmn melodie na dal tn. Bhem pehrvn probh indikace
; svtelnch efekt na LED matici. Zvolen melodie se ulo do pamti
; EEPROM, take pi novm zapnut napjen bude pehrvna stejn melodie.
;
; Hlavn obsluha programu po sv inicializaci (nvt Start) pokrauje
; obsluhou pehrvn jednotlivch impuls zvuku (nvt Impuls).
; Obsluhu pehrvn melodie a obsluhu tlatek zajiuje obsluha peruen
; (nvt Inter). Obsluha peruen zajiuje asovn melodie, nastavuje
; frekvenci pro pehrvan tn v hlavn obsluze programu (obsluha Impuls).
;**************************************************************************
; Banky:
;  V celm programu se ponechv nastavena banka 0, pouze bhem
;  inicializace programu a pi ten a zpisu EEPROM se nastav
;  doasn banka 1.

	RADIX	DEC

; ----- datov promnn

	CBLOCK
Flags				; pznaky
				;	bit0=je pauza
				;	bit1=melodie 2
				;	bit2=zvuk zapnut
Tone				; schova sla tnu
ToneL				; konstanta pro tn LOW (1 a 256)
ToneH				; konstanta pro tn HIGH (1 a 256)
Length				; ta dlky tnu
CountL				; ta pro tn LOW
CountH				; ta pro tn HIGH
Pointer				; ukazatel v tabulce pehrvan melodie
TempW				; schova W pi peruen
TempS				; schova registru STATUS pi peruen
EAdr				; adresa pro zpis do EEPROM
EData				; data pro zpis do EEPROM
TempInt				; schova registru peruen pi zpisu EEPROM
	ENDC
        
; ----- start po resetu

	org	0
	goto	Start		; start programu

; ----- obsluha peruen

	org	4
	goto	Inter		; obsluha peruen

; ----- tabulka tn

TbToneL	addwf	PCL,F
	dt	0	; snen oktva
	dt	243		; 1: C
	dt	172		; 2: C#
	dt	104		; 3: D
	dt	41		; 4: D#
	dt	237		; 5: E
	dt	181		; 6: F
	dt	127		; 7: F#
	dt	76		; 8: G
	dt	29		; 9: G#
	dt	240		; 10: A
	dt	198		; 11: A#
	dt	158		; 12: H
			; zkladn oktva
	dt	120		; 13: C
	dt	84		; 14: C#
	dt	50		; 15: D
	dt	18		; 16: D#
	dt	245		; 17: E
	dt	217		; 18: F
	dt	190		; 19: F#
	dt	165		; 20: G
	dt	141		; 21: G#
	dt	118		; 22: A
	dt	97		; 23: A#
	dt	77		; 24: H
			; zven oktva
	dt	58		; 25: C
	dt	40		; 26: C#
	dt	23		; 27: D
	dt	7		; 28: D#
	dt	249		; 29: E
	dt	235		; 30: F
	dt	221		; 31: F#
	dt	209		; 32: G
	dt	197		; 33: G#
	dt	185		; 34: A
	dt	177		; 35: A#
	dt	165		; 36: H

TbToneH	addwf	PCL,F
	dt	0	; snen oktva
	dt	5		; 1: C
	dt	5		; 2: C#
	dt	5		; 3: D
	dt	5		; 4: D#
	dt	4		; 5: E
	dt	4		; 6: F
	dt	4		; 7: F#
	dt	4		; 8: G
	dt	4		; 9: G#
	dt	3		; 10: A
	dt	3		; 11: A#
	dt	3		; 12: H
			; zkladn oktva
	dt	3		; 13: C
	dt	3		; 14: C#
	dt	3		; 15: D
	dt	3		; 16: D#
	dt	2		; 17: E
	dt	2		; 18: F
	dt	2		; 19: F#
	dt	2		; 20: G
	dt	2		; 21: G#
	dt	2		; 22: A
	dt	2		; 23: A#
	dt	2		; 24: H
			; zven oktva
	dt	2		; 25: C
	dt	2		; 26: C#
	dt	2		; 27: D
	dt	2		; 28: D#
	dt	1		; 29: E
	dt	1		; 30: F
	dt	1		; 31: F#
	dt	1		; 32: G
	dt	1		; 33: G#
	dt	1		; 34: A
	dt	1		; 35: A#
	dt	1		; 36: H

; ----- definice not a dlek not

C1	EQU	1
D1	EQU	3
E1	EQU	5
F1	EQU	6
G1	EQU	8
A1	EQU	10
H1	EQU	12

C2	EQU	13
D2	EQU	15
E2	EQU	17
F2	EQU	18
G2	EQU	20
A2	EQU	22
H2	EQU	24

C3	EQU	25
D3	EQU	27
E3	EQU	29
F3	EQU	30
G3	EQU	32
A3	EQU	34
H3	EQU	36

R	EQU	255		; pauza

L32	EQU	1		; 1/32
L16	EQU	2		; 1/16
L16P	EQU	3		; 1/16.
L8	EQU	4		; 1/8
L8P	EQU	6		; 1/8.
L4	EQU	8		; 1/4
L4P	EQU	12		; 1/4.
L2	EQU	16		; 1/2
L2P	EQU	24		; 1/2.
L1	EQU	32		; 1

; ----- pehrvan melodie (doporuuje se dlka kad
; melodie 32 krok kvli indikaci, kter m 32 stav)

Melodi1	addwf	PCL,F

; Koka leze drou
	dt	C2,L8
	dt	D2,L8
	dt	E2,L8
	dt	F2,L8
	dt	G2,L4
	dt	G2,L4

	dt	A2,L4
	dt	A2,L4
	dt	G2,L2

	dt	A2,L4
	dt	A2,L4
	dt	G2,L2

	dt	F2,L8
	dt	F2,L8
	dt	F2,L8
	dt	F2,L8
	dt	E2,L4
	dt	E2,L4

	dt	D2,L4
	dt	D2,L4
	dt	G2,L2

	dt	F2,L8
	dt	F2,L8
	dt	F2,L8
	dt	F2,L8
	dt	E2,L4
	dt	E2,L4

	dt	D2,L4
	dt	D2,L4
	dt	C2,L4P
	dt	R,L16
	dt	R,L16

	dt	0

Melodi2	addwf	PCL,F

; Jak jsi krsn, nevitko
	dt	E2,L4
	dt	E2,L4
	dt	G2,L4
	dt	G2,L4

	dt	F2,L4
	dt	F2,L8
	dt	A2,L8
	dt	G2,L4
	dt	G2,L4

	dt	E2,L4
	dt	E2,L4
	dt	G2,L4
	dt	G2,L4

	dt	D2,L4
	dt	D2,L8
	dt	F2,L8
	dt	E2,L4
	dt	E2,L4

	dt	C3,L2
	dt	G2,L4
	dt	E2,L4

	dt	F2,L4
	dt	F2,L8
	dt	A2,L8
	dt	G2,L2

	dt	C3,L2
	dt	G2,L4
	dt	E2,L4

	dt	F2,L4
	dt	D2,L8
	dt	E2,L8
	dt	C2,L2

	dt	0

; ----- naten jednoho bajtu aktivn melodie

GetMel 	btfsc	Flags,1		; je melodie 2?
	goto	GetMel1
	call	Melodi1		; naten sla tnu
	goto	GetMel2

GetMel1 call	Melodi2		; je melodie 1
GetMel2	return

;*****************************************************************
;
; 			obsluha peruen
;
;*****************************************************************

; ----- nulovn pznaku peruen

Inter	bcf	INTCON,T0IF	; nulovn pznaku peruen

; ----- tn dlky jednoho tnu

	decfsz	Length,F	; tn dlky jednoho tnu
	retfie			; nen jet konec jednoho tnu

; ----- schova registr W a STATUS

	movwf	TempW		; schova registru W
	swapf	STATUS,W	; naten STATUS registru do W
	movwf	TempS		; schova STATUS registru

; ----- svteln efekty

	bcf	STATUS,RP0	; banka 0
	incf	PORTB,F		; zmna bit na vstupu B
	incf	PORTA,W		; zmna bit na vstupu A
	andlw	0Fh		; maskovn bit na vstup A
	movwf	PORTA		; uloen na vstup A

; ----- zapnut 1. melodie tlatkem A

	btfss	PORTB,5		; je stisknuto tlatko A?
	goto	NoA		; nen stisknuto tlatko A

	bcf	Flags,1		; zapnut melodie 1
	goto	InitMel		; inicializace melodie

; ----- zapnut 2. melodie tlatkem B

NoA	btfss	PORTB,6		; je stisknuto tlatko B?
	goto	NoB		; nen stisknuto tlatko B

	bsf	Flags,1		; zapnut melodie 2
InitMel	bsf	Flags,2		; zapnut zvuku
	clrf	Pointer		; nulovn ukazatele melodie
	clrf	PORTA		; nulovn indikace portu A
	clrf	PORTB		; nulovn indikace portu B
	goto	SetABC		; zpis stavu do EEPROM

; ----- vypnut zvuku tlatkem C

NoB	btfss	PORTB,7		; je stisknuto tlatko C?
	goto	NoC		; nen stisknuto tlatko C

	bcf	Flags,2		; vypnut zvuku

; ----- zpis novho nastaven do EEPROM

SetABC	movlw	EFlags		; adresa pznak v EEPROM
	movwf	EAdr		; adresa k zpisu pznak
	movf	Flags,W		; naten pznak do W
	call	ESet		; zpis nastaven do EEPROM

; ----- naten dalho znaku melodie

NoC	movf	Pointer,W	; naten ukazatele melodie
	incf	Pointer,F	; posun ukazatele melodie
	call	GetMel		; naten dalho tnu

; ----- peteen ukazatele na zatek

	addlw	0		; je konec melodie?
	btfss	STATUS,Z	; je-li 0, je konec melodie
	goto	ToneOK		; nen jet konec melodie
	movwf	Pointer		; vynulovn ukazatele melodie

; ----- naten prvnho tnu melodie

	movf	Pointer,W	; naten ukazatele melodie
	incf	Pointer,F	; posun ukazatele melodie
	call	GetMel		; naten dalho tnu

; ----- nastaven pauzy

ToneOK	bsf	Flags,0		; bude pauza
	xorlw	R		; je to pauza?
	btfsc	Flags,2		; je-li ticho, bude jako pauza
	btfsc	STATUS,Z	; nen-li pauza, pokrauje dle
	goto	SetLen		; je pauza, pesko se vka tnu
	xorlw	R		; nvrat hodnoty W
	bcf	Flags,0		; nen pauza

; ----- naten konstanty tnu

	movwf	Tone		; schova tnu pro pt
	call	TbToneL		; naten konstanty LOW
	movwf	ToneL		; nastaven konstanty LOW

	movf	Tone,W		; poadovan tn
	call	TbToneH		; naten konstanty HIGH
	movwf	ToneH		; nastaven konstanty HIGH

; ----- nov ta jednoho tnu

SetLen	movf	Pointer,W	; naten ukazatele melodie
	incf	Pointer,F	; posun ukazatele melodie
	call	GetMel		; naten dlky tnu
	movwf	Length		; nastaven dlky tnu

; ----- nvrat registr W a STATUS

IntRet	swapf	TempS,W		; pvodn STATUS registr k navrcen
	movwf	STATUS		; nvrat pvodnho STATUS registru
	swapf	TempW,F		; pedpprava pvodnho registru W
	swapf	TempW,W		; nvrat pvodnho W registru
	retfie

;*****************************************************************
;
; 				hlavn program
;
;*****************************************************************

; ----- inicializace portu A

Start 	bsf	STATUS,RP0	; banka 1
	movlw	0		; vechny bity portu A jako vstup
	movwf	TRISA		; nastaven portu A
	movlw	11100000b	; bitu pro port B
	movwf	TRISB		; nastaven portu B

; ----- nastaven mdu asovae a peddliky

	movlw	7 + bit7	; peddlika k TMR0
	movwf	OPTION_REG	; nastaven mdu tae
	bcf	STATUS,RP0	; banka 0
	movlw	01Fh		; vstup na port A a B
	movwf	PORTA		; nastaven portu A
	movwf	PORTB		; nastaven portu B

; ----- inicializace ukazatel melodie

	clrf	Pointer		; nulovn ukazatele melodie
	movlw	1		; W <= 1
	movwf	Length		; ta dlky tny (dlka poten pauzy)
	movlw	EFlags		; adresa pznak v EEPROM
	call	EGet		; naten bajtu z EEPROM
	iorlw	bit0		; zapnut pznaku pauzy
	movwf	Flags		; zane se pauzou

; ----- inicializace obsluhy peruen

	movlw	0ffh		; pi pt inkrementaci se vyvol peruen
	movwf	TMR0		; inicializace tae
	bsf	INTCON,T0IE	; povolen peruen pi peteen asovae
	bsf	INTCON,GIE	; povoleno globln peruen

;*****************************************************************
; asovn - asov konstanta zadna jako sla L a H:
;	vnitn smyka	DECFSZ ... 1T
;				GOTO ..... 2T
;	poprv se opakuje (L-1)x .......... 3*(L-1)T
;	pt se opakuje 255x ............ 3*255T = 765T
;	vnj smyka	DECFSZ ... 2T
;				DECFSZ ... 1T
;				GOTO ..... 2T
;	opakuje se (H-1)x ................. 5*(H-1)T
;	jednorzov	se pite 13T
;				
;	celkem ... 3*(L-1) + (H-1)*765 + 5*(H-1) + 13 = 
;		= 3L - 1 + 765H - 765 + 5H - 5 + 10 = 3L + 770H - 758
;
;	jedna zmna signlu je pouze jedna plvlna
;	pro krystal 4 MHz bude vsledn frekvence tnu:
;
;		f = 500000/(3L + 770H - 758)
;		n = 3L + 770H = 500000/f + 758
;
; Nap. pro frekvenci 440 Hz (tzv. komorn 'a') budou konstanty
;	n = 3L + 770H = 500000/440 + 758 = 1894
;	H = 2, L = 118
; 
; Frekvence a asov konstanty pro tny:
; -------------------------------------
; snen oktva:
;	C ... 130.81 Hz, n=4580, H=5, L=243
;	C# .. 138.59 Hz, n=4366, H=5, L=172
;	D ... 146.83 Hz, n=4163, H=5, L=104
;	D# .. 155.56 Hz, n=3972, H=5, L=41
;	E ... 164.81 Hz, n=3792, H=4, L=237
;	F ... 174.61 Hz, n=3622, H=4, L=181
;	F# .. 185.00 Hz, n=3461, H=4, L=127
;	G ... 196.00 Hz, n=3309, H=4, L=76
;	G# .. 207.65 Hz, n=3166, H=4, L=29
;	A ... 220.00 Hz, n=3031, H=3, L=240
;	A# .. 233.08 Hz, n=2903, H=3, L=198
;	H ... 246.94 Hz, n=2783, H=3, L=158
;
; zkladn oktva:
;	C ... 261.63 Hz, n=2669, H=3, L=120
;	C# .. 277.18 Hz, n=2562, H=3, L=84
;	D ... 293.66 Hz, n=2461, H=3, L=50
;	D# .. 311.13 Hz, n=2365, H=3, L=18
;	E ... 329.63 Hz, n=2275, H=2, L=245
;	F ... 349.23 Hz, n=2190, H=2, L=217
;	F# .. 369.99 Hz, n=2109, H=2, L=190
;	G ... 392.00 Hz, n=2034, H=2, L=165
;	G# .. 415.30 Hz, n=1962, H=2, L=141
;	A ... 440.00 Hz, n=1894, H=2, L=118
;	A# .. 466.16 Hz, n=1831, H=2, L=97
;	H ... 493.88 Hz, n=1770, H=2, L=77
;
; zven oktva:
;	C ... 523.25 Hz, n=1714, H=2, L=58
;	C# .. 554.37 Hz, n=1660, H=2, L=40
;	D ... 587.33 Hz, n=1609, H=2, L=23
;	D# .. 622.25 Hz, n=1562, H=2, L=7
;	E ... 659.26 Hz, n=1516, H=1, L=249
;	F ... 698.46 Hz, n=1474, H=1, L=235
;	F# .. 739.99 Hz, n=1434, H=1, L=221
;	G ... 783.99 Hz, n=1396, H=1, L=209
;	G# .. 830.61 Hz, n=1360, H=1, L=197
;	A ... 880.00 Hz, n=1326, H=1, L=185
;	A# .. 932.33 Hz, n=1300, H=1, L=177
;	H ... 987.77 Hz, n=1264, H=1, L=165
;*****************************************************************

; ----- pprava tae pro jeden impuls

Impuls	movf	ToneL,W		; konstanta pro tn LOW
	movwf	CountL		; uloen do tae LOW
	movf	ToneH,W		; konstanta pro tn HIGH
	movwf	CountH		; uloen do tae HIGH

; ----- tn po dobu jednoho impulsu

Wait	decfsz	CountL,F	; tn tae LOW
	goto	Wait
	decfsz	CountH,F	; tn tae HIGH
	goto	Wait

; ----- zmna signlu (jen nen-li pauza)

	btfsc	Flags,0		; test, zda je pauza
	goto	Impuls		; je pauza - bude dal impuls
	movlw	bit4		; bit pro zmnu signlu reproduktoru
	xorwf	PORTA,F		; zmna signlu reproduktoru
	goto	Impuls		; dal impuls tnu

;*****************************************************************
;
;			Obsluha EEPROM
;
;*****************************************************************

; ----- kontrola typu procesoru

#ifndef	PIC1662
#ifndef	PIC1684
	error	>>> Nepodporovan typ procesoru!!! <<<
#endif
#endif

;-----------------------------------------------------------------
;	naten bajtu z EEPROM (vstup: W=adresa, vstup: W=data)
;-----------------------------------------------------------------

EGet

; ----- obsluha pro architekturu 84

#ifdef	PIC1684

	bcf	STATUS,RP0	; volba banky 0
	movwf	EEADR		; nastaven adresy EEPROM
	bsf	STATUS,RP0	; volba banky 1
	bsf	EECON1,RD	; zapnut poadavku ke ten bajtu
	bcf	STATUS,RP0	; volba banky 0
	movf	EEDATA,W	; naten bajtu do registru W

#endif

; ----- obsluha pro architekturu 62

#ifdef	PIC1662

	bsf	STATUS,RP0	; volba banky 1
	movwf	EEADR		; nastaven adresy EEPROM
	bsf	EECON1,RD	; zapnut poadavku ke ten bajtu
	movf	EEDATA,W	; naten bajtu do registru W
	bcf	STATUS,RP0	; volba banky 0

#endif

	return

;-----------------------------------------------------------------
;	zpis bajtu do EEPROM (vstup: W=data, EAdr=adresa)
;-----------------------------------------------------------------

ESet	movwf	EData		; schova dat k zpisu

; ----- kontrola, zda je bajt v EEPROM ji uloen

	movf	EAdr,W		; naten adresy dat do W
	call	EGet		; naten bajtu dat
	subwf	EData,W		; kontrola shody bajtu
	btfsc	STATUS,Z	; je shoda dat?
	return			; pi shod konec

; ----- pprava adresy a dat k zpisu

#ifdef	PIC1684
	bcf	STATUS,RP0	; volba banky 0
#endif

#ifdef	PIC1662
	bsf	STATUS,RP0	; volba banky 1
#endif

	movf	EAdr,W		; naten adresy dat do W
	movwf	EEADR		; nastaven adresy EEPROM
	movf	EData,W		; naten dat k zpisu do W
	movwf	EEDATA		; nastaven dat k zpisu do EEPROM

; ----- zkaz vech peruen bhem operace zpisu

	movf	INTCON,W	; naten registru peruen do W
	movwf	TempInt		; schova registru peruen
	bcf	INTCON,GIE	; zkaz vech peruen

; ----- povolen zpisu do EEPROM

#ifdef	PIC1684
	bsf	STATUS,RP0	; volba banky 1
#endif

	bsf	EECON1,WREN	; povolen zpisu do EEPROM
	movlw	55h		; 1. odemykac kd
	movwf	EECON2		; zpis 1. odemykacho kdu
	movlw	0aah		; 2. odemykac kd
	movwf	EECON2		; zpis 2. odemykacho kdu

; ----- nastaven poadavku k zpisu bajtu do EEPROM

	bsf	EECON1,WR	; zapnut poadavku k zpisu bajtu

; ----- zkaz dalho zpisu do EEPROM

	bcf	EECON1,WREN	; zkaz zpisu do EEPROM

; ----- povolen peruen

	btfsc	TempInt,GIE	; bylo peruen povoleno?
	bsf	INTCON,GIE	; povolen peruen

; ----- ekn na dokonen zpisu do EEPROM

ESet2	btfsc	EECON1,WR	; je zpis dokonen?
	goto	ESet2		; ekn na dokonen zpisu

; ----- nvrat banky 0

	bcf	STATUS,RP0	; volba banky 0
	return

;-----------------------------------------------------------------
;	implicitn obsah pamti EEPROM
;-----------------------------------------------------------------

	org	2100h

EFlags	de	bit2		; pznaky
				;	bit1=melodie 2
				;	bit2=zvuk zapnut
